#include "alg_types.hpp"

#include "../util/util_registry.hpp"
#include "../cli/cli_wrap.hpp"
#include "../img/access/img_access_copy.hpp"
#include "../ipp/ipp_lincomb.hpp"

#include <iostream>
#include <sstream>

class alg_average: public alg_mono1 {
  int nimages_;
public:
  alg_average(const CtorArg&);
  virtual ~alg_average();
public:
  virtual char const* name(){return s_name();}
  static char const* s_name(){return "average";}
private:
  virtual void process(const img_data& src, 
    const win_event_vectorAbstract&, int msDayUtc,
    img_data& dst);
  virtual void config(cli_wrapAbstract& cli);
  void dumpParams(std::ostream& os, char const* intro);
};

namespace{
  util::RegistryCreator<alg_base,alg_average> creator;
}
  

alg_average::alg_average(const CtorArg& arg):
  alg_mono1( img_fmt(arg.fmt.width(), arg.fmt.height(), 1)),
  nimages_(0)
{}
alg_average::~alg_average(){}



void alg_average::process(
  const img_data& src,
  const win_event_vectorAbstract&, int,
  img_data& dst)
{
  dst.reformat(img_fmt(
    src.fmt().width(), 
    src.fmt().height(), 1)
  );
  
  if (nimages_==0){
    img_access_copy(src, dst);
  } else{
    double a=nimages_/(nimages_+1.0);
    double b=1/(nimages_+1.0);
    // std::cerr <<a <<" " <<b <<"\n";
    ipp::lincomb(src, b,a, dst);
  }
  
  ++nimages_;
}

void alg_average::config(
  cli_wrapAbstract& cli)
{
  //introduction
  std::ostringstream oss;
  dumpParams(oss, "Current alg_average configuration:\n");
  oss <<"Please supply the new value for nimages:\n";
  cli.put(oss.str().c_str());

  oss.str("");
  //input a line
  char const* buf=cli.get();
  if (strlen(buf)>0){
    std::istringstream iss(buf);
    iss >>nimages_;
    oss <<std::endl;
  }

  //report the new state
  dumpParams(oss, "The new values are:\n");
  cli.put(oss.str().c_str());
}

void alg_average::dumpParams(
  std::ostream& os, 
  char const* intro)
{
  os <<intro
     <<"nimages: "   <<nimages_ <<std::endl;
}

