#include "alg_base.hpp"

#include "../img/img_data.hpp"
#include "../cli/cli_wrapAbstract.hpp"
#include "../util/util_registry.hpp"

#include <string>
#include <sstream>
#include <typeinfo>
#include <stdexcept>

#include <assert.h>


///////////////////////////////////
// construction

alg_base::alg_base(){}
alg_base::~alg_base(){}

void alg_base::checkId(int id, char const* context){
  if (id<0 || id>=countDst()){
    assert(0);
    std::ostringstream oss;
    oss <<context <<" - illegal id: " <<id;
    throw std::runtime_error(oss.str());
  }
}

///////////////////////////////////
//default mandatory operations
void alg_base::process( 
  const img_vectorAbstract& src, 
  const win_event_vectorAbstract& events,
  int msDayUtc, int )
{
  process(src, events, msDayUtc);
}
void alg_base::process( 
  const img_vectorAbstract& , 
  const win_event_vectorAbstract& ,
  int )
{
}


///////////////////////////////////
// attributes

const img_fmt& alg_base::fmtDst(int id){
  checkId(id, "alg_base::fmtDst");
  return imgDst(id).fmt();
}
win_ann_abstract const* alg_base::pAnnDst(int id){
  checkId(id, "alg_base::pAnnDst");
  return 0;
}
char const* alg_base::captionDst(int id){
  checkId(id, "alg_base::captionDst");
  return name();
}

void alg_base::config(
  cli_wrapAbstract& cli)
{
  std::ostringstream oss;
  oss <<"The image processing procedure ";
  oss <<"'" <<name() <<"'" <<std::endl;
  oss <<"does not provide configurable parameters" <<std::endl;
  cli.msg(oss.str().c_str());
}

void alg_base::profile(ui_reportAbstract& ){}


////////////////////////
// alg_base static methods

alg_base* alg_base::create(
  const char* name_derived, 
  const CtorArg& arg)
{
  return util::Registry::instance().
    create<alg_base>(name_derived, arg);
}


