#include "alg_types.hpp"

#include "../img/img_data.hpp"
#include "../img/img_util.hpp"
#include "../img/access/img_access_copy.hpp"

#include "../util/util_registry.hpp"
#include "../util/util_time.hpp"

#include "../cli/cli_wrap.hpp"
#include "../ui/ui_reportAbstract.hpp"

#include <iostream>
#include <sstream>
#include <string>

#include <string.h>

// test command lines:
// -sf=e:/images/auti/proleterske.avi -i -a=crop
// -sf=e:/images/auti/proleterske.avi -df=e:\temp\auti.avi -a=crop -c="16 40 40 0"

class alg_crop:
  public alg_mono
{
public:
  alg_crop(const CtorArg&);
  virtual ~alg_crop();
public:
  virtual char const* name(){return s_name();}
  static char const* s_name(){return "crop";}
public:
  virtual void process(const img_data& src, 
    const win_event_vectorAbstract& events, int msDayUtc);
  virtual int countDst(){return 1;}
  virtual const img_data& imgDst(int id);
public:
  virtual void profile(ui_reportAbstract& pd);
  virtual void config(cli_wrapAbstract& cli);
private:
  void dumpParams(std::ostream& os, char const* intro);
private:  
  int tCopy_;
  int tCrop_;

  int left_;
  int right_;
  int top_;
  int bottom_;

  img_data imgCrop_;
};

///////////////////////////////////
// static creator object
namespace{
  util::RegistryCreator<alg_base,alg_crop> creator;
}



alg_crop::alg_crop(const CtorArg& arg):
  left_(0), right_(0),
  top_(0), bottom_(0),
  imgCrop_(arg.fmt)
{
}
  
alg_crop::~alg_crop(){
}


///////////////////////////////////
// mandatory operations
void alg_crop::process(
  const img_data& src,
  const win_event_vectorAbstract&, int)
{
  util_time timer;
  util_time::Data t0=timer();

  util_time::Data t1=timer();

  imgCrop_.reformat(img_fmt(
    src.fmt().width()-left_-right_,
    src.fmt().height()-top_-bottom_,
    src.fmt().szPixel())
  );

  img_access_copyWithOffset(
    left_, bottom_,
    src, imgCrop_);
  util_time::Data t2=timer();

  tCopy_=     t1-t0;
  tCrop_=     t2-t1;
}
const img_data& alg_crop::imgDst(int id){
  checkId(id, "alg_crop::imgDst");
  return imgCrop_;
}

///////////////////////////////////
//facultative operations
void alg_crop::profile(ui_reportAbstract& pd){
  pd.push_back("copy", tCopy_);
  pd.push_back("crop", tCrop_);
}

void alg_crop::config(
  cli_wrapAbstract& cli)
{
  //introduction
  std::ostringstream oss;
  dumpParams(oss, "Current alg_crop configuration:\n");
  oss <<"Values for left and right must be multiples of 4.\n";
  oss <<"Please supply the new values for \n";
  oss <<"left, bottom, right and top offset:\n";
  cli.put(oss.str().c_str());

  oss.str("");
  //input a line
  char const* buf=cli.get();
  if (strlen(buf)>0){
    std::istringstream iss(buf);
    iss >>left_   >>bottom_ 
        >>right_  >>top_;
    oss <<std::endl;
  }

  //report the new state
  dumpParams(oss, "The new values are:\n");
  cli.put(oss.str().c_str());
}

void alg_crop::dumpParams(
  std::ostream& os, 
  char const* intro)
{
  os <<intro
     <<"left offset: "   <<left_ <<std::endl
     <<"bottom offset: " <<bottom_ <<std::endl
     <<"right offset: "  <<right_ <<std::endl
     <<"top offset: "    <<top_ <<std::endl;
}

