#include "alg_decoratorScale.hpp"

#include "../ipp/ipp_subsample.hpp"
#include "../ui/ui_reportAbstract.hpp"
#include "../util/util_time.hpp"

#include <stdexcept>
#include <limits>
#include <sstream>
#include <iostream>



namespace{
  int calcLocalScale(
    int maxDim,
    const img_fmt& fmt)
  {
    int scale=1;
    if (maxDim>0){  
      int maxdim=0;
      maxdim=std::max(fmt.width(), maxdim);
      maxdim=std::max(fmt.height(), maxdim);
      scale=1+maxdim/maxDim;
    }
    return scale;
  }
}

alg_decoratorScale::alg_decoratorScale(
  boost::shared_ptr<alg_base>& alg,
  int maxdim)
:
  spWorker_(alg),
  name_(spWorker_->name()),
  maxDim_(maxdim),
  scale_(1),
  t_(0)
{
}
alg_decoratorScale::alg_decoratorScale(
  alg_base* alg,
  int maxdim)
:
  spWorker_(alg),
  name_(spWorker_->name()),
  maxDim_(maxdim),
  scale_(1),
  t_(0)
{
}

alg_decoratorScale::~alg_decoratorScale() {}


void alg_decoratorScale::reset(
  boost::shared_ptr<alg_base>& alg)
{
  spWorker_=alg;
}

//////////////////////////////////
// scaling

void alg_decoratorScale::setScale(int s){
  if (s<1){
    throw std::runtime_error("alg_decoratorScale::setScale - illegal scale");
  }
  scale_=s; 
  maxDim_=0;
}

void alg_decoratorScale::scaleToDimension(int d){
  if (d<1){
    d=0;
  }
  maxDim_=d;
}

void alg_decoratorScale::calcGlobalScale(){
  if (maxDim_>0){  
    int maxdim=0;
    for (int i=0; i<spWorker_->countDst(); ++i){
      maxdim=std::max(spWorker_->fmtDst(i).width(), maxdim);
      maxdim=std::max(spWorker_->fmtDst(i).height(), maxdim);
    }

    scale_=1+(maxdim+1)/maxDim_;
  }
  name_ =spWorker_->name();
  if (scale_>1){
    std::ostringstream oss;
    oss <<" M 1:" <<scale_;
    name_+=oss.str();   
  }
}


//////////////////////////////////
// operations


void alg_decoratorScale::process( 
  const img_vectorAbstract& src, 
  const win_event_vectorAbstract& events,
  int msDayUtc, int nframe)
{
  const bool localScale_=true;
  spWorker_->process(src, events, msDayUtc, nframe);

  int t0=util_time::msDayUtc();

  calcGlobalScale();
  if (scale_>1){
    //std::cerr <<"scale=" <<scale_ 
    //  <<" width=" <<spWorker_->fmtDst(0).width()/scale_ <<"\n";
    
    anns_.resize(spWorker_->countDst());
    imgs_.resize(spWorker_->countDst());
    
    for (int i=0; i<spWorker_->countDst(); ++i){
      const img_fmt& fmt=spWorker_->fmtDst(i);
      //localScale_ switches from local to global scale
      int myscale=localScale_?calcLocalScale(maxDim_,fmt):scale_;
      
      if (myscale>1){
        img_fmt fmt_s(
          fmt.width()/myscale,
          fmt.height()/myscale, 
          fmt.szPixel());
        imgs_[i].reformat(fmt_s);
        ipp::subsample(spWorker_->imgDst(i), myscale, imgs_[i]);
      } else{
        imgs_[i]=spWorker_->imgDst(i);
      }
      const win_ann_abstract* pAnn=spWorker_->pAnnDst(i);
      if (pAnn)
        anns_[i].assign(*pAnn, myscale);
      else
        anns_[i].clear();
        
      std::ostringstream oss;
      oss <<"M 1:" <<myscale;
      anns_[i].addStr(grid::Pixel(
        1*fmt.width()/myscale/20,
        fmt.height()/myscale/20), 
        0, oss.str().c_str(), 2);
    }
  }
  
  t_=util_time::msDayUtc()-t0;
}

void alg_decoratorScale::config(
  cli_wrapAbstract& cli)
{
  spWorker_->config(cli);
}
void alg_decoratorScale::profile(
  ui_reportAbstract& pd)
{
  spWorker_->profile(pd);
  if (scale_!=1){
    pd.push_back("scaling", t_);
  }
}


//////////////////////////////////
// output

int alg_decoratorScale::countDst(){
  return spWorker_->countDst();
}
char const* alg_decoratorScale::captionDst(int id){
  checkId(id, "alg_decoratorScale");
  return spWorker_->captionDst(id);
}

const img_data& alg_decoratorScale::imgDst(int id){
  checkId(id, "alg_decoratorScale");
  if (scale_==1){
    return spWorker_->imgDst(id);
  } else{
    return imgs_[id];
  }
}

win_ann_abstract const* alg_decoratorScale::pAnnDst(int id){
  checkId(id, "alg_decoratorScale");
  if (scale_==1){
    return spWorker_->pAnnDst(id);
  } else{
    return &anns_[id];
  }
}


const img_data& alg_decoratorScale::imgDstOrig(int id){
  checkId(id, "alg_decoratorScale");
  return spWorker_->imgDst(id);
}
