#ifndef alg_decoratorScale_hpp
#define alg_decoratorScale_hpp

/******************************************************
*
 ** alg_decoratorScale by Sinisa Segvic:
 *
 ** changes the given alg_base's output 
 ** (images and annotations)
 ** so that they can fit to average screen
 *
 ** alg_decoratorScale owns the decorated alg_base 
 ** (ie, the decorator is responsible for deleting 
 **  the decorated object)
 *
 ** unlike other alg_base-s, object factory 
 ** is not needed (objects are instantiated directly)
 *
 *******************************************************/

#include "alg_base.hpp"

#include "../win/ann/win_ann.hpp"
#include "../img/img_data.hpp"

#include<boost/shared_ptr.hpp>

#include<vector>
#include<string>

class alg_decoratorScale:
  public alg_base
{
public:
  alg_decoratorScale(alg_base* palg, int maxdim);
  alg_decoratorScale(boost::shared_ptr<alg_base>& alg, int maxdim);
  virtual ~alg_decoratorScale();

  void reset(boost::shared_ptr<alg_base>& newalg);
//scaling
public:
  void setScale(int s);
  void scaleToDimension(int d);
private:
  void calcGlobalScale();
  
//identification
public:
  virtual char const* name(){return name_.c_str();}

//mandatory operations
public:
  virtual void process( 
    const img_vectorAbstract& src, 
    const win_event_vectorAbstract& events,
    int msDayUtc, int nframe); 
//discretionary operations
public:
  virtual void config(cli_wrapAbstract& cli);
  virtual void profile(ui_reportAbstract& );

//output
public:
  virtual int countDst(); // >=1!
  virtual char const* captionDst(int id);
  virtual const img_data& imgDst(int id);
  virtual win_ann_abstract const* pAnnDst(int id); 

  const img_data& imgDstOrig(int i);

//data
private:
  boost::shared_ptr<alg_base> spWorker_;
  std::string name_;
  int maxDim_;
  int scale_;
  int t_;
  std::vector<win_ann> anns_;
  std::vector<img_data> imgs_;
};

#endif
