#include "../alg/alg_types.hpp"
#include "../ipp/ipp_equalize.hpp"

#include "../util/util_registry.hpp"
#include "../util/util_time.hpp"

#include "../ui/ui_reportAbstract.hpp"
#include "../cli/cli_wrapAbstract.hpp"

#include <vector>
#include <string>
#include <sstream>
#include <iostream>

#include <assert.h>

class alg_equalize:
  public alg_mono1
{
  std::vector<int> t_;
  double minval_;
  double maxval_;
public:
  alg_equalize(const CtorArg& arg);
  virtual ~alg_equalize();
public:
  virtual char const* name(){return s_name();}
  static char const* s_name(){return "equalize";}
public:
  virtual void process(
    const img_data& src, 
    const win_event_vectorAbstract& , int , 
    img_data& dst);
  virtual void profile(ui_reportAbstract& p);
  virtual void config(cli_wrapAbstract& cli);
private:
  void dumpParams(std::ostream& os,char const*);
};

namespace{
  util::RegistryCreator<alg_base,alg_equalize> creator;
}
  

alg_equalize::alg_equalize(const CtorArg& arg):
  alg_mono1(arg.fmt),
  minval_(-1),
  maxval_(-1)
{
  t_.reserve(10);
}
alg_equalize::~alg_equalize(){}



void alg_equalize::process(
  const img_data& src,
  const win_event_vectorAbstract& , int , 
  img_data& dst)
{
  util_time timer;
  t_.clear();
  t_.push_back(timer());

  dst.reformat(src.fmt());

  t_.push_back(timer());
  ipp::equalize(src, minval_,maxval_,255, dst);
          
  t_.push_back(timer());
}



void alg_equalize::profile(
  ui_reportAbstract& p)
{
  assert(t_.size()==3);
  p.push_back("preparation  ",  t_[1]-t_[0]);
  p.push_back("equalization ",  t_[2]-t_[1]);
}


void alg_equalize::config(
  cli_wrapAbstract& cli)
{
  std::ostringstream oss;
  std::string str;
  str=str+"Current " + name() +" configuration:";  
  dumpParams(oss, str.c_str());
  oss <<"Please supply the new values for\n"
     <<"  minval, maxval:\n";
  cli.put(oss.str().c_str());

  char const* buf=cli.get();
  if (strlen(buf)>0){
    std::istringstream iss(buf);
    iss >>minval_;
    iss >>maxval_;
  }

  dumpParams(oss, "The new values are:");
  cli.put(oss.str().c_str());
}
void alg_equalize::dumpParams(
  std::ostream& os,
  char const* intro)
{
  os <<"\n" <<intro <<"\n"
     <<"  minval:  "<<minval_  <<"\n"
     <<"  maxval:  "<<maxval_  <<"\n"
     <<"\n";
}
