#include "alg_types.hpp"

#include "../util/util_registry.hpp"

#include "../img/img_data.hpp"

#include "../ipp/ipp_histogram.hpp"
#include "../ipp/ipp_vector.hpp"


class alg_histogram:
  public alg_mono
{
public:
  alg_histogram(const CtorArg&);
  virtual ~alg_histogram();

  virtual char const* name(){return s_name();}
  static char const* s_name(){return "histogram";}

private:
  virtual void process(const img_data& src, const win_event_vectorAbstract& events, int msDayUtc);

  virtual int countDst(){return 2;}; 
  virtual const img_data& imgDst(int id);
  virtual char const* captionDst(int id);

  const int height;
  img_data imgSrc_;
  img_data imgHist_;
};


namespace{
  util::RegistryCreator<alg_base,alg_histogram> creator;
}


alg_histogram::alg_histogram(const CtorArg& arg):
  height(256),
  imgSrc_(arg.fmt),
  imgHist_(img_fmt(256,height,1))
{}

alg_histogram::~alg_histogram(){
}

void alg_histogram::process(
  const img_data& src, 
  const win_event_vectorAbstract&, int)
{
  imgSrc_=src;

  std::vector<int> h;
  ipp::histogram(imgSrc_, h);
  ipp::drawVectorHor(h, imgHist_);
}

const img_data& alg_histogram::imgDst(int id){
  checkId(id, "alg_histogram::imgDst");
  if (id==0)
    return imgSrc_;
  else
    return imgHist_;
}
char const* alg_histogram::captionDst(int id){
  checkId(id, "alg_histogram::captionDst");
  if (id==0)
    return "destination image";
  else
    return "source image";
}
