#include "alg_base.hpp"

#include "../util/util_registry.hpp"

#include "../img/img_vector.hpp"
#include "../img/access/img_access_fill.hpp"

#include <iostream>

// vd_file_srs vdf(img_fmt(384,288,1),"/var/tmp/proba.srs");

////////////////////////////////////
// alg_nop declaration

class alg_nop: 
  public alg_base
{
public:
  alg_nop(const CtorArg& arg);
  virtual ~alg_nop();
public:
  virtual char const* name(){return s_name();}
  static char const* s_name(){return "nop";}
private:
  virtual void process( 
    const img_vectorAbstract& src, 
    const win_event_vectorAbstract& events,
    int msDayUtc); 
private:
  virtual int countDst(); // >=1!
  virtual const img_data& imgDst(int id);
private:
  img_vector imgs_;
};

////////////////////////////////////
// static registry creator object

namespace{
  util::RegistryCreator<alg_base,alg_nop> creator;
} // unnamed namespace end

  
////////////////////////////////////
// alg_nop implementation

alg_nop::alg_nop(const CtorArg& arg):
  imgs_(1)
{
  imgs_[0].reformat(arg.fmt);
}

alg_nop::~alg_nop(){}

void alg_nop::process(
  const img_vectorAbstract& src,
  const win_event_vectorAbstract&, int)
{
  //std::cerr <<src.size() <<"\n";
  imgs_.resize(src.size());
  for (int i=0; i<src.size(); ++i){
    imgs_[i]=src[i];
  }
}

int alg_nop::countDst(){
  return static_cast<int>(imgs_.size());
}

const img_data& alg_nop::imgDst(int id){
  checkId(id, "alg_nop::imgDst");
  return imgs_[id];
}


////////////////////////////////////
// alg_null declaration

class alg_null: 
  public alg_base
{
public:
  alg_null(const CtorArg& arg);
public:
  virtual char const* name(){return s_name();}
  static char const* s_name(){return "null";}
private:
  virtual void process( 
    const img_vectorAbstract& src, 
    const win_event_vectorAbstract& events,
    int msDayUtc);
private:
  virtual int countDst(){ return 1;}
  virtual const img_data& imgDst(int){ return imgNull_;}
private:
  img_data imgNull_;
};

////////////////////////////////////
// static registry creator object

namespace{
  util::RegistryCreator<alg_base,alg_null> creator2;
} // unnamed namespace end

  
////////////////////////////////////
// alg_null implementation

alg_null::alg_null(const CtorArg& ):
  imgNull_(img_fmt(32,32,1))
{
  img_access_fillCheckboard(imgNull_,4);
}

void alg_null::process(
  const img_vectorAbstract&,
  const win_event_vectorAbstract&, int)
{
}
