#include "../alg/alg_types.hpp"
#include "../ipp/ipp_radial.hpp"

#include "../util/util_registry.hpp"
#include "../util/util_time.hpp"

#include "../img/access/img_access_copy.hpp"

#include "../ui/ui_reportAbstract.hpp"
#include "../cli/cli_wrapAbstract.hpp"

#include <vector>
#include <string>
#include <sstream>
#include <iostream>

#include <assert.h>

class alg_radial:
  public alg_mono2
{
  std::vector<int> t_;
  double alpha_;
  double u0_, v0_;
  double k1_, k2_;
public:
  alg_radial(const CtorArg& arg);
  virtual ~alg_radial();
public:
  virtual char const* name(){return s_name();}
  static char const* s_name(){return "radial";}
public:
  virtual void process(
    const img_data& src, 
    const win_event_vectorAbstract& , int , 
    img_data& dst);
  virtual void profile(ui_reportAbstract& p);
  virtual void config(cli_wrapAbstract& cli);
private:
  void dumpParams(std::ostream& os,char const*);
};

namespace{
  util::RegistryCreator<alg_base,alg_radial> creator;
}
  

alg_radial::alg_radial(const CtorArg& arg):
  alg_mono2(arg.fmt),
  alpha_(514.7),
  u0_(-4.1),
  v0_(8.4),
  k1_(-0.2413),
  k2_(0.0982)
{
  t_.reserve(10);
}
alg_radial::~alg_radial(){}



void alg_radial::process(
  const img_data& src,
  const win_event_vectorAbstract& , int , 
  img_data& dst)
{
  util_time timer;
  t_.clear();
  t_.push_back(timer());

  img_fmt f(src.fmt(),1);
  img_data tmp(f);
  img_access_copy(src, tmp);
  dst.reformat(tmp.fmt());

  t_.push_back(timer());
  ipp::radial(tmp, 
    alpha_, u0_,v0_, k1_,k2_, 
    dst);
          
  t_.push_back(timer());
}



void alg_radial::profile(
  ui_reportAbstract& p)
{
  assert(t_.size()==3);
  p.push_back("preparation ",  t_[1]-t_[0]);
  p.push_back("correction  ",  t_[2]-t_[1]);
}
void alg_radial::config(
  cli_wrapAbstract& cli)
{
  std::ostringstream oss;
  std::string str;
  str=str+"Current " + name() +" configuration:";  
  dumpParams(oss, str.c_str());
  oss <<"Please supply the new values for\n"
     <<"  alpha, u0,v0, k1,k2:\n";
  cli.put(oss.str().c_str());

  char const* buf=cli.get();
  if (strlen(buf)>0){
    std::istringstream iss(buf);
    iss >>alpha_;
    iss >>u0_ >>v0_;
    iss >>k1_ >>k2_;
  }

  dumpParams(oss, "The new values are:");
  cli.put(oss.str().c_str());
}
void alg_radial::dumpParams(
  std::ostream& os,
  char const* intro)
{
  os <<"\n" <<intro <<"\n"
     <<"  alpha:  "<<alpha_  <<"\n"
     <<"  u0,v0:  "<<u0_ <<"," <<v0_ <<"\n"
     <<"  k1,k2:  "<<k1_ <<"," <<k2_ <<"\n"
     <<"\n";
}
