#include "../alg/alg_types.hpp"

#include "../util/util_registry.hpp"
#include "../util/util_fs.hpp"

#include "../img/img_data.hpp"
#include "../img/img_util.hpp"
#include "../img/access/img_access_copy.hpp"

#include "../vd/vd_file.hpp"
#include "../vd/vd_file_util.hpp"

#include "../win/event/win_event_abstract.hpp"
#include "../win/ann/win_ann.hpp"
#include "../cli/cli_wrapAbstract.hpp"
#include "../ui/ui_reportAbstract.hpp"

#include <boost/scoped_ptr.hpp>

#include <string>
#include <sstream>
#include <iostream>

#include <assert.h>


namespace{
  std::string getFileName(char const* basename){
    std::string dirLog(util_fs::getLogDir());
    std::string fnBase(dirLog+basename);
    std::string fnBaseUnique=
      util_fs::uniqueFilename(fnBase.c_str());
    return fnBaseUnique;
  }
}

class alg_saver:
  public alg_mono
{
  std::string name_;
  boost::scoped_ptr<vd_file> pvd_;
  img_data imgDst_;
  win_ann ann_;
  bool switchDisplay_;
  
public:
  alg_saver(const CtorArg&);
  virtual ~alg_saver();
public:
  virtual char const* name(){return s_name();}
  static char const* s_name(){return "saver";}
public:
  virtual void process(
    const img_data& src,
    const win_event_vectorAbstract& events,
    int msDayUtc);
  virtual void config(cli_wrapAbstract& cli);
public:
  virtual int countDst(){return 1;}
  virtual const img_data& imgDst(int id);
  virtual win_ann_abstract const* pAnnDst(int id); 
private:
  void dumpParams(std::ostream& os,char const*);
};


namespace{
  util::RegistryCreator<alg_base,alg_saver> creator;
}
  

//////////////////////////////
//alg_saver implementation

alg_saver::alg_saver(
  const CtorArg& arg)
:
  name_(getFileName("algSaver.pgm")),
  pvd_(vd_file_util::create(arg.fmt, name_.c_str())),
  imgDst_(arg.fmt)
{
  std::cerr <<"alg_saver: " <<name_ <<"\n";
  switchDisplay_=false;
}
alg_saver::~alg_saver(){}

void alg_saver::process(
  const img_data& src,
  const win_event_vectorAbstract& events, int)
{
  vd_base& vd(*pvd_);
  bool saved=false;
  for (int i=0; i<events.size(); ++i){
    const win_event_abstract& e(events[i]);
    if (e.type()==win_event_abstract::MouseLeft ||
        e.type()==win_event_abstract::KeyboardAscii &&
        e.par1()==' ')
    {
      saved=true;
      break;
    }
  }

  ann_.clear();
  if (saved){
    std::cerr <<"alg_saver: got one\n";
    vd(src);
    ann_.addSound(50,200);
  }

  if (!switchDisplay_){  
    imgDst_=src;
  } else{
    img_util::flipHorizontal(src, imgDst_);
    img_util::flipVertical(imgDst_);
  }
}
void alg_saver::config(
  cli_wrapAbstract& cli)
{
  std::ostringstream oss;
  std::string str;
  str=str+"Current " + name() +" configuration:";  
  dumpParams(oss, str.c_str());
  oss <<"Please supply the new values for\n"
     <<"  switchDisplay:\n";
  cli.put(oss.str().c_str());

  char const* buf=cli.get();
  if (strlen(buf)>0){
    std::istringstream iss(buf);
    iss >>switchDisplay_;
  }

  dumpParams(oss, "The new values are:");
  cli.put(oss.str().c_str());
}
void alg_saver::dumpParams(
  std::ostream& os,
  char const* intro)
{
  os <<"\n" <<intro <<"\n"
     <<"  switchDisplay:  "<<switchDisplay_  <<"\n"
     <<"\n";
}


win_ann_abstract const* alg_saver::pAnnDst(int id){ 
  checkId(id, "alg_saver::pAnnDst");
  return &ann_;
}
const img_data& alg_saver::imgDst(int id){
  checkId(id, "alg_saver::imgDst");
  return imgDst_;
}
  
  
