#include "alg_types.hpp"

#include "../util/util_registry.hpp"
#include "../img/img_vector.hpp"

#include "../ipp/ipp_subsample.hpp"

#include "../cli/cli_wrapAbstract.hpp"

#include <iostream>
#include <vector>
#include <sstream>

#include <string.h>


class alg_subsample: 
  public alg_base
{
public:
  alg_subsample(const CtorArg& arg);
  virtual ~alg_subsample();
public:
  virtual char const* name(){return s_name();}
  static char const* s_name(){return "subsample";}
private:
  virtual void process( 
    const img_vectorAbstract& src, 
    const win_event_vectorAbstract& events,
    int msDayUtc); 
  virtual int countDst(){return static_cast<int>(imgs_.size());}
  virtual const img_data& imgDst(int id);
private:
  virtual void config(cli_wrapAbstract& cli);
private:
  void init();
  void dumpParams(std::ostream& os, char const* intro);
  
private:
  int scale_;
  img_vector imgs_;
};

namespace{
  util::RegistryCreator<alg_base,alg_subsample> creator;
}

////////////////////////////////////
// construction

alg_subsample::alg_subsample(const CtorArg& arg):
  scale_(2),
  imgs_(1)
{
  imgs_[0].reformat(img_fmt(
    arg.fmt.width()/scale_,
    arg.fmt.height()/scale_,
    arg.fmt.szPixel()));
}

alg_subsample::~alg_subsample(){}


///////////////////////////////////
// mandatory operations

void alg_subsample::process(
  const img_vectorAbstract& src,
  const win_event_vectorAbstract&, int)
{
  imgs_.resize(src.size());
  for (int i=0; i<src.size(); ++i){
    imgs_[i].reformat(img_fmt(
      src[i].fmt().width()/scale_,
      src[i].fmt().height()/scale_,
      src[i].fmt().szPixel()));
    ipp::subsample(src[i], scale_, imgs_[i]);
  }
}
const img_data& alg_subsample::imgDst(int id){
  checkId(id, "alg_crop::imgDst");
  return imgs_[id];
}


///////////////////////////////////
// facultative operations

void alg_subsample::config(
  cli_wrapAbstract& cli)
{
  int newscale=scale_;

  //introduction
  std::ostringstream oss;
  dumpParams(oss, "Current subsample configuration:\n");
  oss <<"Please supply the new value for scale: ";
  cli.put(oss.str().c_str());

  oss.str("");
  //input a line
  char const* buf=cli.get();
  if (strlen(buf)>0){
    std::istringstream iss(buf);
    iss >>newscale;
    oss <<std::endl;
  }
  
  if (newscale<2 || newscale>32)
    throw std::runtime_error("alg_subsample::config -"
      " got illegal value for scale");

  // resize the output images
  for (int i=0; i<imgs_.size(); ++i){
    imgs_[i].reformat(img_fmt(
      imgs_[i].fmt().width()*scale_/newscale,
      imgs_[i].fmt().height()*scale_/newscale,
      imgs_[i].fmt().szPixel()));
  }
  scale_=newscale;

  //report the new state
  dumpParams(oss, "The new value is:\n");
  cli.put(oss.str().c_str());
}

void alg_subsample::dumpParams(
  std::ostream& os, 
  char const* intro)
{
  os <<intro <<"scale: " <<scale_ <<std::endl;
}

