#include "alg_types.hpp"

#include "../img/img_vectorAbstract.hpp"

#include <stdexcept>

#include <assert.h>

namespace{
  inline void myassert(bool exp, char const* msg){
    if (!exp){
      assert(0);
      throw std::runtime_error(msg);
    }
  }
}

//////////////////////////////////
// alg_mono

void alg_mono::process(
  const img_vectorAbstract& src,
  const win_event_vectorAbstract& evs, int t)
{
  myassert(src.size()==1, "alg_mono::process - illegal source size");
  process(src[0], evs, t);
}



//////////////////////////////////
// alg_stereo

void alg_stereo::process(
  const img_vectorAbstract& src,
  const win_event_vectorAbstract& evs, int t)
{
  myassert(src.size()==2, "alg_stereo::process - illegal source size");
  process(src[0], src[1], evs, t);
}




///////////////////////////////////
// alg_mono1

void alg_mono1::process(
  const img_data& src,
  const win_event_vectorAbstract& evs, int t)
{
  process(src, evs, t, imgDst_);
}

const img_data& alg_mono1::imgDst(int id){
  checkId(id, "alg_mono1::imgDst");
  return imgDst_;
}
char const* alg_mono1::captionDst(int id){
  checkId(id, "alg_mono1::imgDst");
  return name();
}





///////////////////////////////////
// alg_mono2

void alg_mono2::process(
  const img_data& src,
  const win_event_vectorAbstract& evs, int t)
{
  imgSrc_=src;
  process(imgSrc_, evs, t, imgDst_);
}

const img_data& alg_mono2::imgDst(int id){
  checkId(id, "alg_mono2::imgDst");
  if (id==0)
    return imgDst_;
  else
    return imgSrc_;
}
char const* alg_mono2::captionDst(int id){
  checkId(id, "alg_mono2::captionDst");
  if (id==0)
    return "destination image";
  else
    return "source image";
}

