#ifndef alg_types_hpp
#define alg_types_hpp

/******************************************************
*
** alg_types by Sinisa Segvic:
*
** simplified alg_base-s serving as a base for classes 
** which do not need a complete interface
*
** alg_mono:
**  * only one input image required
*
** alg_stereo:
**  * exactly two input images required
*
** alg_mono1 (alg_mono):
**  * single output, dstFmt(0) == srcFmt()
*
** alg_mono2 (alg_mono):
**  * source image is displayed in a separate window
*
*******************************************************/
#include "alg_base.hpp"


// the base class for algorithms operating
// on one image at a time
class alg_mono:
  public alg_base
{
protected:
  alg_mono(){}

private:
//operations
  virtual void process( 
    const img_data& src,
    const win_event_vectorAbstract& events, 
    int msDayUtc) =0; 

//template method
  virtual void process( 
    const img_vectorAbstract& src,
    const win_event_vectorAbstract& events, 
    int msDayUtc); 
};



class alg_stereo:
  public alg_base
{
protected:
  alg_stereo(){}

private:
//operations
  virtual void process( 
    const img_data& src1,
    const img_data& src2,
    const win_event_vectorAbstract& events, 
    int msDayUtc) =0; 

//template method
  virtual void process( 
    const img_vectorAbstract& src,
    const win_event_vectorAbstract& events, 
    int msDayUtc); 
};



class alg_mono1:
  public alg_mono
{
protected:
  alg_mono1(const img_fmt& f): imgDst_(f) {}

private:
//operations
  virtual void process(
    const img_data& src,
    const win_event_vectorAbstract& events, 
    int msDayUtc, 
    img_data& dst) =0;

//output
  virtual int countDst(){return 1;}; 
  virtual const img_data& imgDst(int id);
  virtual char const* captionDst(int id);

//template method
  virtual void process( 
    const img_data& src,
    const win_event_vectorAbstract& events, 
    int msDayUtc); 

//data
private:
  img_data imgDst_;
};









class alg_mono2:
  public alg_mono
{
protected:
  alg_mono2(const img_fmt& f): imgSrc_(f), imgDst_(f) {}

private:
//operations
  virtual void process(
    const img_data& src,
    const win_event_vectorAbstract& events, 
    int msDayUtc, 
    img_data& dst) =0;

//output
  virtual int countDst(){return 2;}; 
  virtual const img_data& imgDst(int id);
  virtual char const* captionDst(int id);

//template method
  virtual void process( 
    const img_data& src,
    const win_event_vectorAbstract& events, 
    int msDayUtc); 

//data
private:
  img_data imgSrc_;
  img_data imgDst_;
};
#endif
