#include "cli_dev.hpp"

#include <iostream>
#include <stdexcept>

//////////////////////////////////////////////
// construction

cli_dev::cli_dev(
  char const* name,
  cli_dev::Type t,
  char const* id)
:
  name_(name),
  type_(t),
  id_(id), 
  from_(0), to_(-1)
{}
cli_dev::~cli_dev(){}


//////////////////////////////////////////////
// setters

void cli_dev::setName(char const* s){
  name_=s;
}
void cli_dev::setType(cli_dev::Type t){
  type_=t;
}
void cli_dev::setId(char const* s){
  id_=s;
}
  
void cli_dev::pushPar(int p){
  par_.push_back(p);
}
void cli_dev::setPar(const std::vector<int>& p){
  par_=p;
}

void cli_dev::setRange(int f, int t){
  from_=f;
  to_=t;
}
  
  
//////////////////////////////////////////////
// getters

char const* cli_dev::name() const{
  return name_.c_str();
}
cli_dev::Type cli_dev::type() const{
  return type_;
}
char const* cli_dev::id() const{
  return id_.c_str();
}

int cli_dev::szPar() const{
  return static_cast<int>(par_.size());
}
int cli_dev::par(int i) const{
  if (i>=static_cast<int>(par_.size())){
    throw std::runtime_error("cli_dev::par() - illegal argument!");
  }
  return par_[i];
}

int cli_dev::from() const{
  return from_;
}
int cli_dev::to() const{
  return to_;
}



//////////////////////////////////////////////
// serialization (output)

std::ostream& operator<<(std::ostream& os, const cli_dev& d){
  char const* strType[]= {
    "Grabber", "Window", "File",  "Network", 
      "Null", "Camera", "PTU"};
  if (
    d.type()>=cli_dev::Grab && 
    d.type() <cli_dev::Null)
  {
    os <<strType[d.type()] 
       <<" (" <<d.name() <<") ";
  } else{
    os <<strType[d.type()] 
       <<" (" <<d.name() 
       <<", " <<d.id()
       <<") ";
  }
  for (int i=0; i<d.szPar(); i++) 
    os <<d.par(i) <<" ";
  if (d.from() || d.to())
    os <<"from " <<d.from() <<" to " <<d.to() <<".";
  return os;
}

