#ifndef cli_parser_hpp
#define cli_parser_hpp

#include "cli_dev.hpp"

#include <vector>
#include <string>

/******************************************************
*
** cli_parser.hpp by Sinisa Segvic:
*
** parses command line and provides access 
** to the obtained information
*
*******************************************************/


class cli_wrap;

class cli_parser{
public:
  cli_parser(int argc, char* argv[]);

//attributes
public:
  char const* alg(){return alg_.c_str();}
  char const* config(){return config_.c_str();}
  char const* command(){return command_.c_str();}
  bool interactive(){return interactive_;}
  int minDelayMs(){return minDelayMs_;}

  const std::vector<cli_dev>& src(){return vec_src_;}
  const std::vector<cli_dev>& dst(){return vec_dst_;}

  const cli_dev& ptu(){return ptu_;}
  const cli_dev& cam(){return cam_;}

//static operations
public:
  static void usageSrc(cli_wrap&);
  static void usageDst(cli_wrap&);
  static void usageAlg(cli_wrap&);
  static void usageOpt(cli_wrap&);
  static void usageCam(cli_wrap&);
  static void parseDst(char const* param, cli_dev& dev);
  static void parseSrc(char const* param, cli_dev& dev);
private:
  static void usageRange(cli_wrap&);

//private operations
private:
  void processParam(char const* param);

  static void parseRange(std::string& str, cli_dev& dev);
  static void parseFile(const std::string& str, cli_dev& dev);
  static void parseNet(const std::string& str, cli_dev& dev);
  static void parseGrab(const std::string& str, cli_dev& dev);
  static void parseWnd(const std::string& str, cli_dev& dev);

  static void parseCam(const std::string& str, cli_dev& dev);

//data
private:
  std::string alg_;
  std::string config_;
  std::string command_;
  std::vector<cli_dev> vec_src_;
  std::vector<cli_dev> vec_dst_;
  bool interactive_;
  int minDelayMs_;

  cli_dev ptu_;
  cli_dev cam_;

  // context
  std::string curPar_;
};

#endif
