#include "cli_wrap.hpp"

#include "../util/util_string.hpp"

#include <iostream>






/////////////////////////////////
// cli_input implementation

class cli_input{
public:
  cli_input(std::istream&);
  ~cli_input();
public:
  //the returned buffer is valid
  //until the next call to getLine
  char const* getLine();
  bool eof();
private:
  std::istream& is_;
private:
  const cli_input& operator=(const cli_input& );
};

cli_input::cli_input(
  std::istream& is)
:
  is_(is)
{}

cli_input::~cli_input(){}

char const* cli_input::getLine(){
  static std::string line;

  // the following does not work 
  // correctly on MSVC 6.0:
  // std::getline(is, line);

  char buf[1024];
  is_.getline(buf, sizeof(buf));
  line=buf;
  
  util_string::trimWs(line);

  return line.c_str();
}

bool cli_input::eof(){
  return is_.eof();
}
  

/////////////////////////////////
// cli_output implementation

class cli_output{
public:
  cli_output(std::ostream&);
  ~cli_output();
public:
  void putText(char const* str);
private:
  std::ostream& os_;
private:
  const cli_output& operator=(const cli_output& );
};

cli_output::cli_output(
  std::ostream& os)
:
  os_(os)
{}

cli_output::~cli_output(){}

void cli_output::putText(
  char const* str)
{
  os_ <<str;
}


/////////////////////////////////
// private implementations

class cli_wrap_imp{
public:
  cli_wrap_imp(
    std::istream& is, 
    std::ostream& os1,
    std::ostream& os2);
  cli_input  in_;
  cli_output out_;
  cli_output msg_;
private:
  const cli_wrap_imp& operator=(const cli_wrap_imp& );
};

cli_wrap_imp::cli_wrap_imp(
  std::istream& is, 
  std::ostream& os1,
  std::ostream& os2)
:
  in_(is),
  out_(os1),
  msg_(os2)
{}

/////////////////////////////////
// cli_wrap implementation

cli_wrap::cli_wrap(
  std::istream& is, 
  std::ostream& os1,
  std::ostream& os2)
:
  pimp(new cli_wrap_imp(is,os1,os2))
{}

cli_wrap::~cli_wrap(){}

char const* cli_wrap::get(){
  return pimp->in_.getLine();
}
bool cli_wrap::eof(){
  return pimp->in_.eof();
}

void cli_wrap::put(char const* s){
  pimp->out_.putText(s);
}
void cli_wrap::msg(char const* s){
  pimp->msg_.putText(s);
}
