#include "cvsh_createVS.hpp"

#include "../cli/cli_dev.hpp"
#include "../cli/cli_wrap.hpp"

#include "../img/img_data.hpp"

//#include "vs/vs_net.hpp"
#include "../vs/vs_file.hpp"
#include "../vs/vs_file_util.hpp"
#include "../vs/vs_grab.hpp"


#include <assert.h>

#include <iostream>
#include <sstream>



  
//////////////////////////////////////////////////////
// create video sources

namespace{

vs_base* createVS(const cli_dev& src)
{
  vs_base* pvs=0;

  switch (src.type()){
  case cli_dev::Grab:{
    assert (src.szPar()==6);

    //we must handle the default
    //parameter value =0
    int w= src.par(0);
    if (w==0) 
      w=384;
    int h= src.par(1);
    if (h==0) 
      h=3*w/4;
    int szpix=src.par(2);
    if (szpix==0) 
      szpix=1;
    int board=src.par(3);
    int ch=   src.par(4);
    int mode= src.par(5);

    vs_grab::CtorArg arg(board,ch,mode, img_fmt(w,h,szpix));
    pvs=vs_grab::create(src.name(), arg);
    break;
  }
  case cli_dev::File:{
    pvs=vs_file_util::create(src.name());
    break;
  }case cli_dev::Net:
    assert(0);
    // host, port
    // return new vs_net(src.name_.c_str(), src.par(0);
    break;
  default:
    //should not happen...
    assert(0);
    break;
  }

  //check consistency of input range
  pvs->setRange(src.from(), src.to());

  return pvs;
}

} //end unnamed namespace


  
//////////////////////////////////////////////////////
// the exported functions

void cvsh_createVS(
  std::ostream& os,
  const std::vector<cli_dev>& vec_src,
  std::vector<boost::shared_ptr<vs_base> >& vecSpVs)
{
  for (unsigned i=0; i<vec_src.size(); i++){
    boost::shared_ptr<vs_base> spvs(createVS(vec_src[i]));
    assert(spvs.use_count()==1);
    vecSpVs.push_back(spvs);
    assert(spvs.use_count()==2);

    const img_fmt& fmt=spvs->fmt();

    os <<"  Created vs #" <<i <<": "
       <<spvs->name() <<'@'
       <<fmt.width() <<" x " 
       <<fmt.height() <<" x " 
       <<fmt.szPixel() <<" (" 
       <<fmt.szLine() <<")\n";

    //report the final range
    os <<"  Source range: [" 
       <<spvs->from()  <<", " 
       <<spvs->to() <<">\n";
  }

}



