#include "grid_alg_rasterLine.hpp"

//#include "../util/util_math.hpp"
//#include "../util/util_c.hpp"

#include <utility>

#include <assert.h>

using namespace grid;


namespace{
// constructs a line from (0,0) to (xe,ye), xe>ye
void rl_worker(
  int xe, int ye,
  Contour& c)
{
  // std::cerr <<"rl_worker: xe=" <<xe <<" ye=" <<ye <<"\n";
  int y=0;
  int e=0; //error measured in "xe"-s
  const int e_unit=(xe-1);
  const int e_inc=(ye-1);

  c.push_back(grid::Pixel(0,0));
  for (int x=1; x<xe; x++){
    e+=e_inc;

    // distance from y to the exact point: e
    // distance from y+1 to the exact point: unit_e-e
    if (e>e_unit-e){
      y++;
      e-=e_unit;
    }
    //std::cerr <<"rl_worker: x=" <<x <<" y=" <<y <<" e=" <<e <<"\n";
    c.push_back(grid::Pixel(x,y));
  }
}


void rl_entry(
  const grid::Line& ln,
  Contour& c
){
  const Pixel& t0=ln.first();
  const Pixel& t1=ln.second();

  int x=t1.x()-t0.x();
  int y=t1.y()-t0.y();

  bool inv_x=x<0;
  if (inv_x) x=-x;

  bool inv_y=y<0;
  if (inv_y) y=-y;

  // find a line from (0,0) to (x,y)
  if (x>y){
    rl_worker(x+1,y+1,c);
  } else{
    rl_worker(y+1,x+1,c);
    for (unsigned i=0; i<c.size(); i++){
      c[i]=grid::Pixel(c[i].y(), c[i].x());
    }
  }

  // translate and mirror the line 
  for (unsigned i=0; i<c.size(); i++){
    int ox=c[i].x();
    if (inv_x) ox=-ox;

    int oy=c[i].y();
    if (inv_y) oy=-oy;

    c[i]=grid::Pixel(t0.x()+ox, t0.y()+oy);
  }
}

}// end unnamed namespace


void grid::alg::rasterLine(
  const grid::Line& ln,
  grid::Contour& c
){
  rl_entry(ln,c);
}
