#ifndef grid_draw_file_hpp
#define grid_draw_file_hpp


/********************************************
 *
 *  component:
 **    grid_draw_file
 *
 ** provides a grid::draw interface for
 ** drawing geometrical primitives to a
 ** single file such as .eps, .svg or .wmf
 ** (see grid_draw for details)
 *
 ** additionally, the class also supports
 ** specifying the background raster image
 *
 ** concrete classes define member functions
 ** for dumping img_data and geometric
 ** primitives while the base class handles the
 ** file creation and closing
 *
 ** USES: img_data
 *
 ** author: Sinisa Segvic
 ** version 1.0 (2001-06-17)
 *
 ********************************************/
#include "grid_draw_rgb.hpp"

class img_data;
#include <iosfwd>

namespace grid{

class draw_file:
  public draw_rgb{

//construction
public:
  draw_file(int x_factor, int x_offset,
           int y_factor, int y_offset,           
           char const* filename);
  draw_file(int x_factor, int x_offset,
           int y_factor, int y_offset,
           std::ostream& os);
  virtual ~draw_file();
private:
  void checkStream();

// Data
protected:
  std::ostream& os();
private:
  std::ostream& os_;
  bool ownsStream_;

//draw operations (see grid_draw.h for details)
private:
  //flushing is done at closing 
  //time in destructor
  virtual void flush(){}
//point drawing
  virtual void drawPtDotImp(int, int) =0;
  virtual void drawPtBoldImp(int,int,int) =0;
//string drawing
  virtual void drawStrImp(int,int,char const*, int) =0;
//line drawing
  virtual void drawLineImp(int,int,int,int ye,int) =0;
  virtual void drawLineDashedImp(int,int,int,int ye,int) =0;
//circle drawing
  virtual void drawCircleImp(int,int,int,int) =0;
  virtual void drawCircleDashedImp(int,int,int,int) =0;
  virtual void drawCircleFilledImp(int,int,int) =0;
//contour drawing (not mandatory)
  virtual void drawContourDotImp(
    const std::vector<std::pair<int,int> >& points) =0;
  virtual void drawContourBoldImp(
    const std::vector<std::pair<int,int> >& points) =0;
//area drawing (not mandatory)
  virtual void drawAreaImp(
    const std::vector<std::pair<int,int> >& points) =0;


private:
  virtual void colourImp(int r, int g, int b) =0;
};

} //namespace grid

#endif
