#include "grid_draw_fileEps.hpp"

#include "../../img/img_data.hpp"

#include "../../img/access/img_access_copy.hpp"


#include <fstream>

#include <stdio.h>

/********************************************
*
**  grid::draw_fileEps
*
**  grid::draw to an eps file
*
** ---------------------------
** PostScript implementation
** kalfa, 13.07.2001
** ---------------------------
*
** ---------------------------
** treba mi copy za imgove
** kalfa, 12.7.2001
** ---------------------------
*
** ----------------------------------
** Izmjena: setgray -> setrgbcolor
** Kalfa, 04.10.2002.
** ----------------------------------
*
*********************************************/




////////////////////////////////////////////
//  construction

grid::draw_fileEps::draw_fileEps(
  char const* filename, 
  int width, int height)
:
  draw_file(1,0, 1,0, filename)
{
  init(width, height);
}

grid::draw_fileEps::draw_fileEps(
  char const* filename, 
  int width, int height,
  const img_data& img)
:
  draw_file(1,0, 1,0, filename)
{
  init(width, height);
  drawImg(img);
}

grid::draw_fileEps::draw_fileEps(
  std::ostream& os, 
  int width, int height,
  const img_data& img)
:
  draw_file(1,0, 1,0, os)
{
  init(width, height);
  drawImg(img);
}

grid::draw_fileEps::~draw_fileEps(){
  os() <<"% Writing eps file epilog." <<std::endl;

  os() <<"showpage" << std::endl;
  os() <<"%%EndDocument" << std::endl << std::endl; 
}



void grid::draw_fileEps::init(
  int width, int height)
{
// kalfa, 11.07.2001.

// najprije prolog
 	os() <<"%!PS-Adobe-2.0 EPSF-2.0" <<std::endl; 
	os() <<"%%BoundingBox: 0 0 "<<width <<' '<<height<<std::endl; 
	os() <<"%%Pages: 1" <<std::endl; 
	os() <<"%%EndComments" <<std::endl; 
	os() <<"%%EndProlog\n" <<std::endl; 


 // definicija skracenica 
 	os() <<"/m /moveto load def" <<std::endl; 
	os() <<"/l /lineto load def" <<std::endl; 
	os() <<"/r /rlineto load def" <<std::endl; 
	os() <<"/s /stroke load def" <<std::endl; 
	os() <<"/f /fill load def" <<std::endl; 
	os() <<"/w /show load def" <<std::endl; 
	os() <<"/c /closepath load def" <<std::endl; 
	os() <<"/n /newpath load def" <<std::endl; 

 // definicija funkcija, npr. za kruznicu
 // necu postavljati debljinu linije - neka o tome vodi racuna tko crta
 // kruznica: 
 // !PAZI: izbacujem stroke ! da bih mogao isto koristiti za fillanu
 //    usage: $x $y $r $arc1 $arc2 kruznica
 	os() <<"\n/kruznica \n{" <<std::endl; 
 	os() <<"newpath" <<std::endl; 
 	os() <<"4 index" <<std::endl; 
 	os() <<"3 index add" <<std::endl; 
 	os() <<"4 index" <<std::endl; 
 	os() <<"moveto" <<std::endl; 
 	os() <<"arc" <<std::endl; 
 	os() <<"} bind def" <<std::endl; 
// 	os() <<"stroke\n  } bind def" <<std::endl; 

 // kruznica (o) zadana sredistem i radiusom 
 //    usage: $x $y $r o
	os() <<"\n/o { 0 360 kruznica stroke } bind def" <<std::endl; 

 // fillana kruznica (of) zadana sredistem i radiusom 
 //    usage: $x $y $r of
 	os() <<"\n/of { 0 360 kruznica fill } bind def\n" <<std::endl; 


// !!! NAZALOST to tako ne radi (-1)
//   ako se koristi [1 0 0 -1 0 240] concat
//   onda se sve zrcali (i slova!)
// ako je potrebno okretanje koordinatnog sustava gore-dolje:
// 	os() << std::endl << "1 -1 scale" << std::endl;
// 	os() << "0 " << height << " translate" << std::endl << std::endl;


// Dodajem: (radi DSC Error - Document Structuring Convention)
 	os() <<"%%BeginDocument" <<std::endl; 
 	os() <<"%%Page: 1 1" <<std::endl; 

	
// TO DO:
// prikazuje se cijela slika, moglo bi se dodati isijecanje
// postavljanjem drugacijeg BoundingBox-a i koristenjem clippinga !
// za sada clipping na cijeloj slici
  // dodano clipanje 
  //ukomentirao sinisa, necu okvir
 	//os() <<"n 0 0 m "<< width <<" 0 r 0 "<< height 
  //     <<" r -" << width <<" 0 r c clip \n"<<std::endl; 

 //    fprintf(ps,"clip\n");


    // ako je definirana pozadinska bitmapa, iscrtaj je
    //ss ukomentirao sinisa, sad se drawImg poziva iz ctora!
    //if (img.valid())
    //  drawImg(img);

 // mogli bi postaviti pocetne vrijednosti sivog i debljinu linije
 	os() <<"0.2 setlinewidth" <<std::endl; 
 	os() <<"0.0 setgray\n" <<std::endl; // inicijalno crna boja



}


////////////////////////////////////////////
//  draw_fileEps operations 
////////////////////////////////////////////

void grid::draw_fileEps::drawImg(const img_data& img){
  // napravi sivu
  img_fmt fmt(img.fmt().width(), img.fmt().height(),1);
  img_data siva(fmt);
  img_access_copy(img, siva); // kopiraj u sivu i dalje radi s njom

  // a sad oprzi sliku nutra
  // ima li veze sto je char umjesto unsigned char ?
  unsigned char *slika = (unsigned char *)siva.bits();
  // pazi na img.fmt().szPixel() - byte per pixel
  //         img.fmt().szLine() - ne mora biti poravnata !!!
  //         img.fmt().szBits() - const {return szLine()*height();}
  //         img.fmt().bottomUp() - const {return height_>0;}

  int Width = img.fmt().width();
  int Height = img.fmt().height();

  os() <<"% slika " << Width << 'x' << Height <<std::endl; 
  // vazno: string mora biti dugacak kao duljina linije
	os() <<"/picstr "<< Width << " string def" << std::endl; 

  // necu translatirati ni skalirati - ne zelim sve crtati u jedinicnom kvadratu
  // 45 140 translate
  // 132 132 scale

  os() << Width <<' '<< Height <<" 8"<<std::endl;
  os() <<"[1 0 0 1 0 0]" << std::endl; 
  //ss fmt je po novom uvijek bottomup! if(img.fmt().bottomUp())
  //else
  //  os() <<"[1 0 0 -1 0 "<< Height << "]" << std::endl; 

	os() <<"{currentfile picstr readhexstring pop}"<<std::endl; 
	os() <<"image"<<std::endl; 

  // sada peri sliku kao hexa brojeve
  long i; // eh, kako cu pratiti koliko ih je u retku ?
  int x,y;
  char str[200]; // kako formatirano pisati u stream ?

  for(y=0,i=0;y<Height;y++)
  for(x=0;x<Width;x++,i++)
  {
    if(i%40==0)
      os() <<std::endl; 
 
    sprintf(str,"%02x",slika[y*siva.fmt().szLine()+ x]); 
    // ! siva slika, 1 byte
	  
    os() << str; 
  }
  os() << std::endl<< std::endl; 
}

////////////////////////////////////////////
//  draw operations 
////////////////////////////////////////////

//////////////////////////////////////
// Pixel drawing functions

void grid::draw_fileEps::drawPtDotImp(
  int x, int y)  
{
  // kako nacrtati samo tocku ?
  os() <<"% Drawing pixel at (" <<x <<", " <<y <<"), "
       <<"with current foreground color" <<std::endl; 

  // moze li fillani malecki kruzic radijusa 0.5 ?
   os() << x << " " << y << " 0.5 of" << std::endl;
}

void grid::draw_fileEps::drawPtBoldImp(
  int x, int y, int size)  
{
  // moze kruzic na tom mjestu ?
  // fillani, suplji, koliki ?

  os() <<"% Drawing bold pixel of size " <<size
       <<", at (" <<x <<", " <<y <<"), "
       <<"with current foreground color" <<std::endl; 

  // kako cemo definirati size ?
   double rsize;
   rsize = 1.0 + size*.75;

   os() << "gsave 1.0 setgray" << std::endl;  
     // neka bude bijeli kruzic u pozadini
   os() << x << " " << y << ' ' << rsize << " of" << std::endl;
   os() << "grestore" << std::endl;
   
   os() << x << " " << y << ' ' << rsize << " o" << std::endl;


}


//////////////////////////////////////
// String drawing functions

void grid::draw_fileEps::drawStrImp(
  int x, int y, 
  char const* str, int sz)  
{
  //int xoff=x>20?-5:5;
  //x+=xoff;
  //int yoff=y>20?-4:4;
  //y+=yoff;


  // size: -1, 0, 1 : 10, 12, 14 pt font ?
  int size = 12 + 2*sz;

  os() << "/CourierBold findfont" <<std::endl;
  os() << size << " scalefont setfont" <<std::endl;
  os() << x << ' ' << y << " m" <<std::endl;
  os() << "(" << str << ") w" << std::endl << std::endl;




  os() <<"% Drawing string of height " <<sz
       <<", at (" <<x <<", " <<y <<"), "
       <<"with current foreground color:" <<std::endl; 
  os() <<"%-->" <<str <<"<--" <<std::endl; 
}


//////////////////////////////////////
// Line drawing functions

void grid::draw_fileEps::drawLineImp(
  int xs, int ys, int xe, int ye, 
  int width) 
{

   // kako reprezentiramo debljinu linije ?
   double rwidth;
   rwidth = 0.2 * width;

   os() << "gsave " << rwidth << " setlinewidth" << std::endl;  
   os() << xs << " " << ys << " m " 
	      << xe << " " << ye << " l s" << std::endl;
   os() << "grestore" << std::endl;
	
   os() <<"% Drawing line of width " <<width
        <<", from (" <<xs <<", " <<ys <<"),"
        <<"to (" <<xe <<", " <<ye <<"), "
        <<"with current foreground color" <<std::endl; 
}

void grid::draw_fileEps::drawLineDashedImp(
  int xs, int ys, int xe, int ye, 
  int width) 
{
   // kako reprezentiramo debljinu linije ?
   double rwidth;
   rwidth = 0.2 * width;

   os() << "gsave [5] 0 setdash" << std::endl;
   os() << rwidth << " setlinewidth" << std::endl;  
   os() << xs << " " << ys << " m " 
	    << xe << " " << ye << " l s" << std::endl;
   os() << "grestore" << std::endl;

	
	os() <<"% Drawing dashed line of width " <<width
       <<", from (" <<xs <<", " <<ys <<"),"
       <<"to (" <<xe <<", " <<ye <<"), "
       <<"with current foreground color" <<std::endl; 
}

//////////////////////////////////////
// Circle drawing functions

void grid::draw_fileEps::drawCircleImp(
  int xs, int ys, int radius, 
  int width) 
{
   // kako reprezentiramo debljinu linije ?
   double rwidth;
   rwidth = 0.2 * width;

   os() << "gsave " << rwidth << " setlinewidth" << std::endl;  
   os() << xs << " " << ys << ' ' << radius << " o" << std::endl;
   os() << "grestore" << std::endl;

	os() <<"% Drawing circle of width " <<width
       <<" and radius " <<radius <<", centred at"
       <<" (" <<xs <<", " <<ys <<"), "
       <<"with current foreground color" <<std::endl; 
}

void grid::draw_fileEps::drawCircleDashedImp(
  int xs, int ys, int radius, 
  int width) 
{
   // kako reprezentiramo debljinu linije ?
   double rwidth;
   rwidth = 0.2 * width;

   os() << "gsave [5] 0 setdash" << std::endl;
   os() << rwidth << " setlinewidth" << std::endl;  
   os() << xs << " " << ys << ' ' << radius << " o" << std::endl;
   os() << "grestore" << std::endl;

	
	
	os() <<"% Drawing dashed circle of width " <<width
       <<" and radius " <<radius <<", centred at"
       <<" (" <<xs <<", " <<ys <<"), "
       <<"with current foreground color" <<std::endl; 
}

void grid::draw_fileEps::drawCircleFilledImp(
  int xs, int ys, int radius) 
{
   // cheap kludge by sinisa:
   int width=1;

   // kako reprezentiramo debljinu linije ?
   double rwidth;
   rwidth = 0.2 * width;

   os() << "gsave " << rwidth << " setlinewidth" << std::endl;  
   os() << xs << " " << ys << ' ' << radius << " of" << std::endl;
   os() << "grestore" << std::endl;

	
	os() <<"% Drawing filled circle of radius " 
       <<radius <<", centred at"
       <<" (" <<xs <<", " <<ys <<"), "
       <<"with current foreground color" <<std::endl; 
}


//////////////////////////////////////
// Contour drawing

void grid::draw_fileEps::drawContourDotImp(
  const std::vector<std::pair<int,int> >& points)
{

  // crta tocku po tocku (ali ipak ih pospaja - 
  // radi skaliranja je to bolje !) 

  // postavi prvu tocku, a onda lineto ostale
  os() << points[0].first  << ' ' 
	    << points[0].second << " m ";

  for (unsigned i=1; i<points.size(); ++i)
	{
	 if(i%25 == 0)
        os() << std::endl;
	
     os() << points[i].first  << ' ' 
	      << points[i].second << " l ";
	}
  os() <<" s"<< std::endl;


  os() <<"% Drawing dotted (dense) contour of " 
       <<static_cast<int>(points.size()) <<" points "
       <<"with current foreground color" <<std::endl; 
}

void grid::draw_fileEps::drawContourBoldImp(
  const std::vector<std::pair<int,int> >& points)
{

  // postavi prvu tocku, a onda lineto ostale
  os() << points[0].first  << ' ' 
	    << points[0].second << " m ";

  for (unsigned i=1; i<points.size(); i++)
	{
	 if(i%25 == 0)
        os() << std::endl;
	
     os() << points[i].first  << ' ' 
	      << points[i].second << " l ";
	}
  os() <<" s"<< std::endl;

  // sad jos opici kruzice (kolikog radijusa ?) na cvorove konture
  // bijele ispune
  os() << "gsave 1.0 setgray";  
  {for (unsigned i=0; i<points.size(); i++)
	{
	 if(i%25 == 0)
        os() << std::endl;
	
     os() << points[i].first  << ' ' 
	      << points[i].second << " 1 of ";
  }}	
  os() << "grestore" << std::endl;

  // i kruznice u foreground boji
  {for (unsigned i=0; i<points.size(); i++)
	{
	 if(i%25 == 0)
        os() << std::endl;
	
     os() << points[i].first  << ' ' 
	      << points[i].second << " 1 o ";
	}}
  os() << std::endl << std::endl;


  os() <<"% Drawing bold (discrete) contour of " 
       <<static_cast<int>(points.size()) <<" points "
       <<"with current foreground color" <<std::endl; 

}

//////////////////////////////////////
// Area drawing

void grid::draw_fileEps::drawAreaImp(
  const std::vector<std::pair<int,int> >& points)
{

  // postavi prvu tocku, a onda lineto ostale
  os() << points[0].first  << ' ' 
	    << points[0].second << " m ";

  for (unsigned i=1; i<points.size(); i++)
	{
	 if(i%25 == 0)
        os() << std::endl;
	
     os() << points[i].first  << ' ' 
	      << points[i].second << " l ";
	}
  os() <<" c f"<< std::endl;
	
	
	
	os() <<"% Drawing filled contour of " 
       <<static_cast<int>(points.size()) <<" points "
       <<"with current foreground color" <<std::endl; 
}



//////////////////////////////////////
// Colours

void grid::draw_fileEps::colourImp(
  int r, int g, int b) 
{
  os() <<"% Changing current foreground color to [" 
       <<r <<"," <<g <<"," <<b <<"]" <<std::endl;
  
  char str[200];

  //float gray;
  //gray = r + g + b;
  //gray = gray/(3*255);
  //sprintf(str,"%.2f setgray", gray);

  sprintf(str,"%.2f %.2f %.2f setrgbcolor", r/255.0, g/255.0, b/255.0);

  os() << str << std::endl;
}


