#ifndef grid_draw_fileEps_hpp
#define grid_draw_fileEps_hpp


/********************************************
 *
 *  component:
 **    grid_draw_fileEps
 *
 ** provides a grid::draw interface for
 ** drawing into an .eps file
 ** (see grid_draw_file for details)
 *
 ** interface:      Sinisa Segvic
 ** implementation: Zoran Kalafatic
 *
 ********************************************/
#include "grid_draw_file.hpp"


namespace grid{

class draw_fileEps:
  public draw_file{

//construction
public:
  draw_fileEps(char const* filename,
    int width, int height);
  draw_fileEps(char const* filename,
    int width, int height,
    const img_data&  img);
  draw_fileEps(std::ostream& os,
    int width, int height,
    const img_data&  img);
  virtual ~draw_fileEps();
//operations
private:
  void init(int width, int height);
  void drawImg(const img_data&);

//draw operations
private:
//point drawing
  virtual void drawPtDotImp(int, int);
  virtual void drawPtBoldImp(int,int,int);
//string drawing
  virtual void drawStrImp(int,int,char const*, int);
//line drawing
  virtual void drawLineImp(int,int,int,int ye,int);
  virtual void drawLineDashedImp(int,int,int,int ye,int);
//circle drawing
  virtual void drawCircleImp(int,int,int,int);
  virtual void drawCircleDashedImp(int,int,int,int);
  virtual void drawCircleFilledImp(int,int,int);
//contour drawing (not mandatory)
  virtual void drawContourDotImp(
    const std::vector<std::pair<int,int> >& points);
  virtual void drawContourBoldImp(
    const std::vector<std::pair<int,int> >& points);
//area drawing (not mandatory)
  virtual void drawAreaImp(
    const std::vector<std::pair<int,int> >& points);
//colours
  virtual void colourImp(int r, int g, int b);

private:
  const draw_fileEps& operator=(const draw_fileEps& );
};

} //namespace grid

#endif
