#ifndef grid_draw_rgb_hpp
#define grid_draw_rgb_hpp


/********************************************
 *
 *  component:
 **    grid_draw_rgb
 *
 * provides a default colour handling scheme
 * using an implementation function receiving
 * a RGB triple for the new current colour
 *
 * grid_draw_rgb ISA grid_draw
 * see grid_draw.h for more information
 *
 ** author: Sinisa Segvic
 ** version 1.0 (2001-11-13)
 *
 ********************************************/

#include "grid_draw.hpp"


namespace grid{

class draw_rgb:
  public draw
{

//construction
protected:
  draw_rgb(int x_factor, int x_offset,
              int y_factor, int y_offset);
  virtual ~draw_rgb();


//pure vituals
public:
  virtual void flush()=0;
private:
  virtual void drawStrImp(int x, int y, char const*, int sz) =0;
  virtual void drawPtDotImp(int x, int y) =0;
  virtual void drawPtBoldImp(int x, int y, int sz) =0;
  virtual void drawLineImp(int xs, int ys, int xe, int ye, int width) =0;
  virtual void drawLineDashedImp(int xs, int ys, int xe, int ye, int width) =0;
  virtual void drawCircleImp(int x, int y, int radius, int width)=0;
  virtual void drawCircleDashedImp(int x, int y, int radius, int width)=0;
  virtual void drawCircleFilledImp(int x, int y, int radius)=0;
//these calls are redirected to the base implementation
private:
  virtual void drawPtDotBigImp(int x, int y);
  virtual void drawContourDotImp(const std::vector<std::pair<int,int> >& points);
  virtual void drawContourBoldImp(const std::vector<std::pair<int,int> >& points);
  virtual void drawAreaImp(const std::vector<std::pair<int,int> >& points);
public:
  virtual void playSound(int t, int f);

//these functions are defined in terms of colourImp
public:
  virtual void colour(int i);
  virtual void grey(int i);
  virtual void bw(bool white);
private:
  virtual void colourImp(int r, int g, int b) =0;
//forbidden
private:
  const draw_rgb& operator=(const draw_rgb& other);
};

} //namespace grid

#endif
