#include "grid_draw_w32.hpp"
#include "grid_draw_colours.hpp"

#include <windows.h>
#include <assert.h>


/********************************************
*
**  grid::draw_w32 
*
**  grid::draw under MS win32 api
*
*********************************************/

//////////////////////////////////////
// Local class Fonts

namespace grid{

class Fonts{

//Fonts is singleton
private:
  Fonts();
public:
  static Fonts& instance();

  HFONT font(int i) const;
  int height(int i) const;
  ~Fonts();

private:
  HFONT small_;
  HFONT normal_;
  HFONT large_;
  HFONT Large_;
  HFONT huge_;
  HFONT Huge_;
};

Fonts::Fonts(){
  small_=
    CreateFont(8, 0, 0, 0, FW_THIN, 0, 0, 0, 
      DEFAULT_CHARSET,  OUT_DEFAULT_PRECIS, 
      CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, 
      FF_SWISS | VARIABLE_PITCH, NULL);
  normal_=
    CreateFont(10, 0, 0, 0, FW_THIN, 0, 0, 0, 
      DEFAULT_CHARSET,  OUT_DEFAULT_PRECIS, 
      CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, 
      FF_SWISS | VARIABLE_PITCH, NULL);
  large_=
    CreateFont(12, 0, 0, 0, FW_THIN, 0, 0, 0, 
      DEFAULT_CHARSET,  OUT_DEFAULT_PRECIS, 
      CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, 
      FF_SWISS | VARIABLE_PITCH, NULL);
  Large_=
    CreateFont(16, 0, 0, 0, FW_NORMAL, 0, 0, 0, 
      DEFAULT_CHARSET,  OUT_DEFAULT_PRECIS, 
      CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, 
      FF_SWISS | VARIABLE_PITCH, NULL);
  huge_=
    CreateFont(20, 0, 0, 0, FW_NORMAL, 0, 0, 0, 
      DEFAULT_CHARSET,  OUT_DEFAULT_PRECIS, 
      CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, 
      FF_SWISS | VARIABLE_PITCH, NULL);
  Huge_=
    CreateFont(24, 0, 0, 0, FW_NORMAL, 0, 0, 0, 
      DEFAULT_CHARSET,  OUT_DEFAULT_PRECIS, 
      CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, 
      FF_SWISS | VARIABLE_PITCH, NULL);
}

Fonts::~Fonts(){
  DeleteObject(small_);
  DeleteObject(normal_);
  DeleteObject(large_);
  DeleteObject(Large_);
  DeleteObject(huge_);
  DeleteObject(Huge_);
}


Fonts& Fonts::instance(){
  static Fonts theInstance;
  return theInstance;
}

HFONT Fonts::font(int i) const{
  if (i<-1)
    return small_;

  switch(i){
  case -1: return normal_;
  case 0: return large_;
  case 1: return Large_;
  case 2: return huge_;
  case 3: return Huge_;
  default: break;
  }

  assert (i>3);

  return Huge_;
}
int Fonts::height(int i) const{
  if (i<0)
    return 8;

  switch(i){
  case 0: return 10;
  case 1: return 14;
  case 2: return 16;
  case 3: return 20;
  }

  assert (i>3);

  return 24;
}


} //namespace grid


//////////////////////////////////////
// construction
grid::draw_w32::draw_w32(
  int x_f, int x_o, int y_f, int y_o,
  HDC hdc)
:
  grid::draw_rgb(x_f, x_o, y_f, y_o),
  hdc_(hdc), red_(0), green_(0), blue_(0) 
{
}

grid::draw_w32::~draw_w32(){
}


//////////////////////////////////////
// operations

void grid::draw_w32::flush() {
}



//////////////////////////////////////
// Pixel drawing functions

void grid::draw_w32::drawPtDotImp(
  int x, int y)  
{
  ::SetPixelV(hdc_, x, y, 
      RGB(red_,green_,blue_));
}

void grid::draw_w32::drawPtBoldImp(
  int x, int y, int size)  
{
  //create brushes
  HBRUSH br_frame=
    ::CreateSolidBrush(0x00404040);
  HBRUSH br_colour=
    ::CreateSolidBrush(RGB(red_,green_,blue_));

  //grey border
  RECT rc1={x-size, y-size, 
            x+size+1, y+size+1};
  FrameRect(hdc_, &rc1, br_frame);

  //main fill
  RECT rc2={x-(size-1),   y-(size-1), 
            x+(size-1)+1, y+(size-1)+1};
  FillRect(hdc_, &rc2, br_colour);

  DeleteObject(br_frame);
  DeleteObject(br_colour);
}


//////////////////////////////////////
// String drawing functions

void grid::draw_w32::drawStrImp(
  int x, int y, 
  char const* str, int sz)  
{
  int yoff=y>20?-1-9*Fonts::instance().height(sz)/8
               : 1+1*Fonts::instance().height(sz)/8;
  int xoff=x>20?-10:0;

  SetBkMode(hdc_, TRANSPARENT);
  SetTextColor(hdc_, RGB(red_,green_,blue_));

  HGDIOBJ hold=SelectObject(hdc_, 
    Fonts::instance().font(sz));
  ExtTextOut(hdc_, x+xoff, y+yoff, 0, 
     NULL, str, static_cast<UINT>(strlen(str)), NULL);
  SelectObject(hdc_, hold);
}


//////////////////////////////////////
// Line drawing functions

void grid::draw_w32::drawLineImp(
  int xs, int ys, int xe, int ye, 
  int width) 
{
  HPEN hpen=CreatePen(PS_SOLID, width, 
    RGB(red_,green_,blue_));
  drawLinePrivate(hpen, xs,ys, xe,ye);
  DeleteObject(hpen);
}

void grid::draw_w32::drawLineDashedImp(
  int xs, int ys, int xe, int ye, 
  int ) 
{
  //SetBkColor (hdc_, RGB(0,0,0)) ;
  SetBkMode (hdc_, TRANSPARENT) ;
  HPEN hpen=CreatePen(PS_DOT, 1, 
    RGB(red_,green_,blue_));
  drawLinePrivate(hpen, xs,ys, xe,ye);
  DeleteObject(hpen);
}

void grid::draw_w32::drawLinePrivate(
  HPEN hpen,
  int xs, int ys, int xe, int ye) 
{
  HGDIOBJ hold=SelectObject(hdc_, hpen);
  MoveToEx(hdc_, xs, ys, 0);
  LineTo(hdc_, xe, ye);
  SelectObject(hdc_, hold);
}


///////////////////////////////////////////////
// circle functions

void grid::draw_w32::drawCircleImp(
  int x, int y, int radius, 
  int width)
{
  HPEN hpen=CreatePen(PS_SOLID, width, 
    RGB(red_,green_,blue_));
  drawCirclePrivate(hpen, x,y, radius);
  DeleteObject(hpen);
}

void grid::draw_w32::drawCircleDashedImp(
  int x, int y, int radius, 
  int )
{
  HPEN hpen=CreatePen(PS_DASH, 1, 
    RGB(red_,green_,blue_));
  drawCirclePrivate(hpen, x,y, radius);
  DeleteObject(hpen);
}

void grid::draw_w32::drawCircleFilledImp(
  int x, int y, int r)
{
  COLORREF col=RGB(red_,green_,blue_);
  HPEN hpen=CreatePen(PS_SOLID, 1,col); 
  HBRUSH hbrush=CreateSolidBrush(col);
  Ellipse(hdc_, x-r, y+r, x+r, y-r);
  DeleteObject(hbrush);
  DeleteObject(hpen);
}

void grid::draw_w32::drawCirclePrivate(
  HPEN hpen,
  int x, int y, int r) 
{
  HGDIOBJ hold=SelectObject(hdc_, hpen);
  Arc(hdc_, x-r, y+r, x+r, y-r,
            0,0, 0,0);
  SelectObject(hdc_, hold);
}


///////////////////////////////////////////////
// colour functions

void grid::draw_w32::colourImp(int r, int g, int b)
{
  red_  =r;
  green_=g;
  blue_ =b;
}

///////////////////////////////////////////////
// music
void grid::draw_w32::playSound(int t, int f)
{
  Beep(f,t);
}
