#ifndef grid_draw_w32_hpp
#define grid_draw_w32_hpp


/********************************************
 *
 *  component:
 **    grid_draw_w32
 *
 * provides a grid::draw interface to 
 * drawing primitives using win32
 * (points, lines and text)
 *
 ** author: Sinisa Segvic
 ** version 2.0 (2001-05-29)
 *
 ********************************************/
#include "grid_draw_rgb.hpp"

#include <windows.h>

namespace grid{
class Fonts;

class draw_w32:
  public draw_rgb{

//construction
public:
  draw_w32(int x_factor, int x_offset,
          int y_factor, int y_offset,
          HDC hdc);
  virtual ~draw_w32();

//operations
public:
  virtual void flush();

//point drawing
public:
  virtual void drawPtDotImp(
    int x, int y);
  virtual void drawPtBoldImp(
    int x, int y, int sz);
  
//string drawing
public:
  virtual void drawStrImp(
    int x, int y, char const*, int sz);

//line drawing
public:
  virtual void drawLineImp(
    int xs, int ys, int xe, int ye,  
    int width);
  virtual void drawLineDashedImp(
    int xs, int ys, int xe, int ye,  
    int width);
private:
  void drawLinePrivate(
    HPEN hpen,
    int xs, int ys, int xe, int ye);

//circle drawing 
public:
  virtual void drawCircleImp(
    int x, int y, int radius, 
    int width);
  virtual void drawCircleDashedImp(
    int x, int y, int radius, 
    int width);
  virtual void drawCircleFilledImp(
    int x, int y, int radius);
private:
  void drawCirclePrivate(HPEN hpen,
    int xs, int ys, int r);

//colours
private:
  virtual void colourImp(int r, int g, int b);

//music
public:
  virtual void playSound(int t, int f);


//data
private:
  HDC hdc_;
  int red_;
  int green_;
  int blue_;
};

} //namespace grid

#endif
