#include "grid_draw_x.hpp"

//#include <X11/X.h>
//#include <X11/Xlib.h>

#include <assert.h>
#include <string.h>


/********************************************
*
**  grid::draw_x 
*
**  grid::draw under XWindows
*
*********************************************/


//////////////////////////////////////
// Local class Fonts

class Fonts{
public:
  Fonts(Display* pd);
  ~Fonts();
    
  Font font(int i) const;
  int height(int i) const;
private:
  Display* pd_;
  Font small_;
  Font normal_;
  Font large_;
  Font Large_;
  Font huge_;
  Font Huge_;
};

Fonts::Fonts(Display* pd):
  pd_(pd)
{
  small_=XLoadFont(pd_, 
    "-*-*-medium-r-*-sans-10-*-*-*-p-*-*-*");
  normal_=XLoadFont(pd_, 
    "-*-*-medium-r-*-sans-12-*-*-*-p-*-*-*");
  large_=XLoadFont(pd_, 
    "-*-*-medium-r-*-sans-14-*-*-*-p-*-*-*");
  Large_=XLoadFont(pd_, 
    "-*-*-medium-r-*-sans-16-*-*-*-p-*-*-*");
  huge_=XLoadFont(pd_, 
    "-*-*-medium-r-*-sans-20-*-*-*-p-*-*-*");
  Huge_=XLoadFont(pd_, 
    "-*-*-medium-r-*-sans-24-*-*-*-p-*-*-*");
}

Fonts::~Fonts(){
  XUnloadFont(pd_, small_);
  XUnloadFont(pd_, normal_);
  XUnloadFont(pd_, large_);
  XUnloadFont(pd_, Large_);
  XUnloadFont(pd_, huge_);
  XUnloadFont(pd_, Huge_);
}


Font Fonts::font(int i) const{
  if (i<0)
    return small_;

  switch(i){
  case 0: return normal_;
  case 1: return large_;
  case 2: return Large_;
  case 3: return huge_;
  default: break;
  }

  assert (i>3);

  return Huge_;
}

int Fonts::height(int i) const{
  if (i<0)
    return 10;

  switch(i){
  case 0: return 12;
  case 1: return 14;
  case 2: return 16;
  case 3: return 20;
  }

  assert (i>3);

  return 24;
}

//////////////////////////////////////
// Local class XColours

class XColours{
public:
  XColours(Display* pd);
public:
  unsigned long colour(int i);
  unsigned long grey(int i);
  unsigned long bw(bool white){
    return white?white_:black_;
  }
private:
  std::vector<unsigned long> colours_;
  std::vector<unsigned long> greys_;
  unsigned long black_;
  unsigned long white_;
};

// (1) it is a good idea to set
//     the count of colours 
//     to be a prime number
XColours::XColours(Display* pd){
  Colormap cmap = DefaultColormap(pd, 
      DefaultScreen (pd));
  XColor exact;
  XColor closest;

  char* colours[]={
    "blue", "green",  "red",
    "cyan", "dark olive green", "orange", 
    "purple", "sea green", "brown",
    "yellow", "aquamarine", //"darkolivegreen", (1)
    "turquoise", "lavender", "navy",
    "seagreen", "chartreuse", "wheat",
    "tan", "pink", "maroon"
  };
  //vecGrey
  char* greys[]={
    "black", //the first entry must be black
    "white", //the second entry must be white
    //even entries gradually rise 
    //from black towards 50% grey;
    //odd entries gradually fall
    //from white towards 50% grey.
    "grey12", "grey94", 
    "grey25", "grey87",
    "grey37", "grey75",
    "grey62", "grey50",
    "grey50" //50% grey must be the last entry
  };

  for (unsigned i=0; i<sizeof(colours)/sizeof(char*); i++){
    if (i>12) break;
    if (XAllocNamedColor(pd,cmap, colours[i], 
        &closest, &exact)) 
      colours_.push_back(closest.pixel);
  }
  assert(colours_.size()>0);

  for (unsigned i=0; i<sizeof(greys)/sizeof(char*); i++){
    if (XAllocNamedColor(pd,cmap, 
      greys[i], &closest, &exact)) 
    {
      greys_.push_back(closest.pixel);
    }
  }
  assert(greys_.size()>0);

  int rv=XAllocNamedColor(pd,cmap, 
    "white", &closest, &exact); 
  if (!rv) assert(0); 
  white_=closest.pixel;
  
  rv=XAllocNamedColor(pd,cmap, 
    "black", &closest, &exact); 
  if (!rv) assert(0); 
  black_=closest.pixel;
}

unsigned long XColours::colour(int i){
  if (i>=0){
    int index=i%colours_.size();
    return colours_[index];
  } else{
    return grey(-i-1);
  }
}

unsigned long XColours::grey(int i){
  assert(i>=0);
  if (i<0) i=0;

  //oo maps to grey 50%
  if (i>=static_cast<int>(greys_.size()))
    i=greys_.size()-1;

  return greys_[i];
}

//////////////////////////////////////
// draw_x private implementation

namespace grid{

class draw_x_imp{
public:
  draw_x_imp(Display* p, Drawable d);
public:
  Display* pd;
  Drawable dr;
  GC gc;
  XGCValues xgcv;

  Fonts fonts;
  XColours cols;
};

} //namespace grid

grid::draw_x_imp::draw_x_imp(
  Display* p, Drawable d)
:
  pd(p), dr(d),
  gc(DefaultGC(pd, DefaultScreen(pd))),
  fonts(pd), cols(pd)
{}


//////////////////////////////////////
// construction
grid::draw_x::draw_x(
  int x_f, int x_o, int y_f, int y_o,
  Display* pd, Drawable d)
:
  grid::draw(x_f, x_o, y_f, y_o),
  pimp(new draw_x_imp(pd, d))
{
}

grid::draw_x::~draw_x(){
  delete pimp;
}

void grid::draw_x::reformat(
  int x_f, int x_o, int y_f, int y_o)
{
  grid::draw::reformat(x_f, x_o, y_f, y_o);
}

//////////////////////////////////////
// operations

void grid::draw_x::flush() {
  XFlush(pimp->pd);
}



//////////////////////////////////////
// Point2D drawing functions

void grid::draw_x::drawPtDotImp(
  int x, int y)  
{
  XDrawPoint(pimp->pd, pimp->dr, pimp->gc, x, y);
}

void grid::draw_x::drawPtBoldImp(
  int x, int y, int size)  
{
  unsigned long original= pimp->xgcv.foreground;

  XSetLineAttributes(pimp->pd,pimp->gc, 1, 
    LineSolid,CapButt,JoinRound);

  //grey border
  XSetForeground(pimp->pd, pimp->gc, pimp->cols.grey(1));
  XDrawRectangle(pimp->pd, pimp->dr, pimp->gc, 
      x-size, y-size, 2*size, 2*size);

  //main fill
  XSetForeground(pimp->pd, pimp->gc, original);
  XFillRectangle(pimp->pd, pimp->dr, pimp->gc, 
    x-(size-1), y-(size-1), 
    2*(size-1)+1, 2*(size-1)+1);
}


//////////////////////////////////////
// String drawing functions

void grid::draw_x::drawStrImp(
  int x, int y, 
  char const* str, int sz)  
{
  int yoff=y>20?-1-1*pimp->fonts.height(sz)/8
               : 1+9*pimp->fonts.height(sz)/8;
  int xoff=x>20?-10:0;

  XSetFont(pimp->pd, pimp->gc, pimp->fonts.font(sz));
  XDrawString(pimp->pd,pimp->dr,pimp->gc, x+xoff,y+yoff, str,strlen(str));
}


//////////////////////////////////////
// Line drawing functions

void grid::draw_x::drawLineImp(
  int xs, int ys, int xe, int ye, 
  int width) 
{
  XSetLineAttributes(pimp->pd,pimp->gc, width, 
    LineSolid,CapButt,JoinRound);
  drawLinePrivate(xs,ys, xe,ye);
}

void grid::draw_x::drawLineDashedImp(
  int xs, int ys, int xe, int ye, 
  int width) 
{
  XSetLineAttributes(pimp->pd,pimp->gc, width, 
    LineOnOffDash,CapButt,JoinRound);
  drawLinePrivate(xs,ys, xe,ye);
}

void grid::draw_x::drawLinePrivate(
  int xs, int ys, int xe, int ye) 
{
  XDrawLine(pimp->pd,pimp->dr,pimp->gc, xs,ys, xe,ye);
}



///////////////////////////////////////////////
// circle functions

void grid::draw_x::drawCircleImp(
  int x, int y, int r, 
  int width)
{
  XSetLineAttributes(pimp->pd,pimp->gc, width, 
    LineSolid,CapButt,JoinRound);
  drawCirclePrivate(x,y,r);
}

void grid::draw_x::drawCircleDashedImp(
  int x, int y, int r, 
  int width)
{
  XSetLineAttributes(pimp->pd,pimp->gc, width, 
    LineOnOffDash,CapButt,JoinRound);
  drawCirclePrivate(x,y,r);
}

void grid::draw_x::drawCirclePrivate(
  int x, int y, int r)
{
  XDrawArc(pimp->pd,pimp->dr,pimp->gc,
    x-r, y-r, 2*r+1, 2*r+1, 0,360*64);
}

void grid::draw_x::drawCircleFilledImp(
  int x, int y, int r)
{
  XFillArc(pimp->pd,pimp->dr,pimp->gc,
    x-r, y-r, 2*r+1, 2*r+1, 0,360*64);
}



//////////////////////////////////////
// XColours

void grid::draw_x::colour(int i){
  pimp->xgcv.foreground = pimp->cols.colour(i);
  XChangeGC(pimp->pd, pimp->gc, GCForeground, &pimp->xgcv);
}
void grid::draw_x::grey(int i){
  pimp->xgcv.foreground = pimp->cols.grey(i);
  XChangeGC(pimp->pd, pimp->gc, GCForeground, &pimp->xgcv);
}
void grid::draw_x::bw(bool white){
  pimp->xgcv.foreground = pimp->cols.bw(white);
  XChangeGC(pimp->pd, pimp->gc, GCForeground, &pimp->xgcv);
}

//////////////////////////////////////
// XBell

void grid::draw_x::playSound(int, int){
  // t and f are ignored
  XBell(pimp->pd, 0);
}

