#ifndef grid_draw_x_hpp
#define grid_draw_x_hpp


/********************************************
 *
 *  component:
 **    grid_draw_x
 *
 * provides a grid::draw interface to 
 * drawing primitives using Xlib
 * (points, lines and text)
 *
 ** author: Sinisa Segvic
 ** version 2.1 (2001-08-24)
 *
 ********************************************/
#include "grid_draw.hpp"

#include <X11/X.h>
#include <X11/Xlib.h>

namespace grid{
class draw_x_imp;

class draw_x:
  public draw{

//construction
public:
  draw_x(
    int x_factor, int x_offset,
    int y_factor, int y_offset,
    Display* pd, Drawable d);
  virtual ~draw_x();

  void reformat(
    int x_factor, int x_offset,
    int y_factor, int y_offset);

//operations
  virtual void flush();

//point drawing
public:
  virtual void drawPtDotImp(
    int x, int y);
  virtual void drawPtBoldImp(
    int x, int y, int sz);
  
//string drawing
public:
  virtual void drawStrImp(
    int x, int y, char const*, int sz);

//line drawing
public:
  virtual void drawLineImp(
    int xs, int ys, int xe, int ye,  
    int width);
  virtual void drawLineDashedImp(
    int xs, int ys, int xe, int ye,  
    int width);
private:
  void drawLinePrivate(
    int xs, int ys, int xe, int ye);

//circle drawing 
public:
  virtual void drawCircleImp(
    int x, int y, int radius, 
    int width);
  virtual void drawCircleDashedImp(
    int x, int y, int radius, 
    int width);
  virtual void drawCircleFilledImp(
    int x, int y, int radius);
private:
  void drawCirclePrivate(
    int x, int y, int radius);
 
//colours 
public:
  virtual void colour(int i);
  virtual void grey(int i);
  virtual void bw(bool white);

//music (optional)
public:
  virtual void playSound(int t, int f);


//data
private:
  draw_x_imp* pimp;
};

} //namespace grid

#endif
