#ifndef img_comb_hpp
#define img_comb_hpp

/******************************************************
*
** img_pix_comb by Sinisa Segvic
*
** classes representing objects able to hold
** linear combination of multiple img_pix objects
*
*******************************************************/

#include "img_access_pix.hpp"
#include <math.h>

//concept PixelCombination
// must have:

// PixelCombination()

// void reset();
// void add(Pixel, double);
// void sub(Pixel, double); 
// void put(Pixel& );
// void divide(int divisor)
// double value();
// double valueL2();



class img_comb_scalar{
  double value_;
public:
  img_comb_scalar():value_(0){}
  void reset(){value_=0;}
public:
  template<class PixelSrc> 
  inline void add(PixelSrc& p, double f=1){
    value_+=p.scalar()*f;
  }
  template<class PixelSrc> 
  inline void sub(PixelSrc& p, double f=1){
    value_-=p.scalar()*f;
  }
  void divide(int divisor){value_/=divisor;}
public:
  double value(){return value_;}
  double valueL2(){return value();}
  template<class PixelDst> 
  void put(PixelDst& p){
    p.setScalar(value_);
  }
};






class img_comb_rgb{
  double r_;
  double g_;
  double b_;
public:
  img_comb_rgb():r_(0),g_(0),b_(0){}
  void reset(){r_=0;g_=0;b_=0;}
public:
  template<class PixelSrc> 
  inline void add(PixelSrc& p, double f=1){
    r_+=p.red()*f;
    g_+=p.green()*f;
    b_+=p.blue()*f;
  }
  template<class PixelSrc> 
  inline void sub(PixelSrc& p, double f=1){
    r_-=p.red()*f;
    g_-=p.green()*f;
    b_-=p.blue()*f;
  }
  void divide(int divisor){
    r_/=divisor;
    g_/=divisor;
    b_/=divisor;
  }
public:
  double value(){return (r_+g_+b_)/3;}
  double valueL2(){
    double sum=(r_*r_+g_*g_+b_*b_);
    double rv=sqrt(sum/3);
    if (r_+g_+b_<0) rv=-rv;
    return rv;
  }
  template<class PixelDst> 
  void put(PixelDst& p){
    p.setRgb(r_, g_, b_);
  }
};


#endif
