#include "img_access_copy.hpp"

#include "../img_data.hpp"
#include "img_access_dispatch.hpp"
#include "img_access_guard.hpp"
#include "img_access_wrap.hpp"
#include "img_access_pix.hpp"

#include <algorithm>
//#include <iostream>

///////////////////////////////////////////////
// class copy worker with offset 

// member template functions defined in-class
// because of the buggy MS compiler

// dummy needed because the buggy MSVC supports 
// only implicit resolution of template functions

class CopyWorkerOffset{
private:
  int offx_;
  int offy_;
public:
  typedef const img_data& Param1;
  typedef img_data& Param2;
public:
  CopyWorkerOffset(int offsetx, int offsety):
    offx_(offsetx), offy_(offsety){}
  template <class PixelSrc, class PixelDst>
  inline int operator() (
    const img_data& img_src, img_data& img_dst, 
    const PixelSrc&, const PixelDst&)
  {
    //std::cout <<"src " <<sizeof(PixelSrc) <<"\n";
    //std::cout <<"dst " <<sizeof(PixelDst) <<"\n";

    typedef img_access_guard<img_access_wrap_const<PixelSrc> > AccessorSrc;
    typedef img_access_guard<img_access_wrap<PixelDst> > AccessorDst;

    int width = std::min(
      img_src.fmt().width()-offx_,
      img_dst.fmt().width());
    int height = std::min(
      img_src.fmt().height()-offy_,
      img_dst.fmt().height());

    AccessorDst rowDst(img_dst);
    AccessorSrc rowSrc(img_src);
    rowSrc.seek(offy_);
    AccessorSrc rowSrcEnd(img_src); 
    rowSrcEnd.seek(offy_+height);

    while (rowSrc<rowSrcEnd){
      AccessorDst dst(rowDst);
      AccessorSrc src(rowSrc);
      src+=offx_;
      AccessorSrc srcEnd(rowSrc);
      srcEnd+=offx_+width;

      while (src<srcEnd){
        *dst=*src;
        //std::cout <<(*src).uint() <<"\n";
        ++src;
        ++dst;
      }
      //std::cout <<"\n";
      rowSrc.incLine();
      rowDst.incLine();
    }//all rows
    return 0;
  }
};







void img_access_copy(const img_data& src, img_data& dst){
  dst.reformat(img_fmt(src.fmt(), dst.fmt().szPixel()));
  img_access_copyWithOffset(0,0, src,dst);
}

void img_access_copyWithOffset(
  int from_x, int from_y,
  const img_data& src, 
  img_data& dst)
{
  CopyWorkerOffset w(from_x, from_y);
  img_access_dispatch2(src,dst,w);
}

