#ifndef img_access_dispatch_hpp
#define img_access_dispatch_hpp

/******************************************************
*
** img_access_dispatch by Sinisa Segvic
*
** dispatching functions: for example, see 
** img_access_copy.cpp
*
** the dummy parameters (img_pix) in dispatch functions 
** allow for calling the required member template function
*
*******************************************************/

#include "img_access_pix.hpp"
#include <stdexcept>


namespace{

template <class Algorithm>
int img_access_dispatch1(
  typename Algorithm::Param1 img, 
  Algorithm& a)
{
  switch (img.fmt().szPixel()){
  case 1:
    return a(img,img_pix_char());
  case 2:
    return a(img,img_pix_fixed16());
  case 3:
    return a(img,img_pix_3char());
  case 4:
    if (img.fmt().rational())
      return a(img,img_pix_float32());
    else
      return a(img,img_pix_int32());
  case 6:
    return a(img,img_pix_3fixed16());
  case 8:
    return a(img,img_pix_float64());
  }
  throw std::runtime_error(
    "img_access_dispatch1 - unsupported format");
}

template <class Algorithm, class PixSrc>
int img_access_dispatch2helper(
  typename Algorithm::Param1 src, 
  typename Algorithm::Param2 dst, 
  Algorithm& a,
  const PixSrc& )
{
  switch (dst.fmt().szPixel()){
  case 1:
    return a(src,dst, PixSrc(), img_pix_char());
  case 2:
    return a(src,dst, PixSrc(), img_pix_fixed16());
  case 3:
    return a(src,dst, PixSrc(), img_pix_3char());
  case 4:
    if (dst.fmt().rational())
      return a(src,dst, PixSrc(), img_pix_float32());
    else
      return a(src,dst, PixSrc(), img_pix_int32());
  case 6:
    return a(src,dst, PixSrc(), img_pix_3fixed16());
  case 8:
    return a(src,dst, PixSrc(), img_pix_float64());
  }
  throw std::runtime_error(
    "img_access_dispatch2helper - unsupported format");
}


template <class Algorithm>
int img_access_dispatch2(
  typename Algorithm::Param1 src,
  typename Algorithm::Param2 dst,
  Algorithm& a)
{
  switch (src.fmt().szPixel()){
  case 1:
    return img_access_dispatch2helper(src,dst, a, img_pix_char());
  case 2:
    return img_access_dispatch2helper(src,dst, a, img_pix_fixed16());
  case 3:
    return img_access_dispatch2helper(src,dst, a, img_pix_3char());
  case 4:
    if (dst.fmt().rational())
      return img_access_dispatch2helper(src,dst, a, img_pix_float32());
    else
      return img_access_dispatch2helper(src,dst, a, img_pix_int32());
  case 6:
    return img_access_dispatch2helper(src,dst, a, img_pix_3fixed16());
  case 8:
    return img_access_dispatch2helper(src,dst, a, img_pix_float64());
  }
  throw std::runtime_error(
    "img_access_dispatch2 - unsupported format");
}



//////////////////////////////////////////////
//////////////////////////////////////////////
// the code below is DEPRECATED!!!
// all functionality has been incorporated in:
// * the two dispatch functions above
// * methods img_fmt::makeFmt* 
//////////////////////////////////////////////
//////////////////////////////////////////////

template <class Algorithm>
int img_access_dispatchToFloat(
  typename Algorithm::Param1 p1,
  typename Algorithm::Param2 p2,
  Algorithm& a)
{
  switch (p1.fmt().szPixel()){
  case 1:
    switch (p2.fmt().szPixel()){
    case 1:
      return a(p1,p2, img_pix_char(), img_pix_char());
    case 2:
      return a(p1,p2, img_pix_char(), img_pix_fixed16());
    }
    break;
  case 2:
    switch (p2.fmt().szPixel()){
    case 2:
      return a(p1,p2, img_pix_fixed16(), img_pix_fixed16());
    case 8:
      return a(p1,p2, img_pix_fixed16(), img_pix_float64());
    }
    break;
  case 3:
    switch (p2.fmt().szPixel()){
    case 3:
      return a(p1,p2, img_pix_3char(), img_pix_3char());
    case 6:
      return a(p1,p2, img_pix_3char(), img_pix_3fixed16());
    }
    break;
  case 4:
    switch (p2.fmt().szPixel()){
    case 4:
      return a(p1,p2, img_pix_float32(), img_pix_float32());
    case 8:
      return a(p1,p2, img_pix_float32(), img_pix_float64());
    }
  case 6:
    return a(p1,p2, img_pix_3fixed16(), img_pix_3fixed16());
  case 8:
    return a(p1,p2, img_pix_float64(), img_pix_float64());
  }
  throw std::runtime_error(
    "img_access_dispatchToFloat - unsupported format");
}

template <class Algorithm>
int img_access_dispatchToScalar(
  typename Algorithm::Param1 p1,
  typename Algorithm::Param2 p2,
  Algorithm& a)
{
  switch (p1.fmt().szPixel()){
  case 1:
    switch (p2.fmt().szPixel()){
    case 1:
      return a(p1,p2, img_pix_char(), img_pix_char());
    case 2:
      return a(p1,p2, img_pix_char(), img_pix_fixed16());
    }
    break;
  case 2:
    switch (p2.fmt().szPixel()){
    case 2:
      return a(p1,p2, img_pix_fixed16(), img_pix_fixed16());
    case 8:
      return a(p1,p2, img_pix_fixed16(), img_pix_float64());
    }
    break;
  case 3:
    switch (p2.fmt().szPixel()){
    case 2:
      return a(p1,p2, img_pix_3char(), img_pix_fixed16());
    case 3:
      return a(p1,p2, img_pix_3char(), img_pix_3char());
    }
    break;
  case 4:
    switch (p2.fmt().szPixel()){
    case 4:
      return a(p1,p2, img_pix_int32(), img_pix_int32());
    case 8:
      return a(p1,p2, img_pix_int32(), img_pix_float64());
    }
  case 6:
    switch (p2.fmt().szPixel()){
    case 2:
      return a(p1,p2, img_pix_3fixed16(), img_pix_fixed16());
    case 6:
      return a(p1,p2, img_pix_3fixed16(), img_pix_3fixed16());
    }
  case 8:
    return a(p1,p2, img_pix_float64(), img_pix_float64());
  }
  throw std::runtime_error(
    "img_access_dispatchToFloat - unsupported format");
}


template <class Algorithm>
int img_access_dispatchToHuge(
  typename Algorithm::Param1 p1,
  typename Algorithm::Param2 p2,
  Algorithm& a)
{
  switch (p1.fmt().szPixel()){
  case 1:
    return a(p1,p2, img_pix_char(), img_pix_int32());
  case 2:
    return a(p1,p2, img_pix_fixed16(), img_pix_float64());
  case 3:
    return a(p1,p2, img_pix_3char(), img_pix_float64());
  case 4:
    return a(p1,p2, img_pix_int32(), img_pix_float64());
  case 6:
    return a(p1,p2, img_pix_3fixed16(), img_pix_float64());
  case 8:
    return a(p1,p2, img_pix_float64(), img_pix_float64());
  }
  throw std::runtime_error(
    "img_access_dispatchToHuge - unsupported format");
}

template <class Algorithm>
int img_access_dispatchScalar(
  typename Algorithm::Param1 img, 
  Algorithm& a)
{
  switch (img.fmt().szPixel()){
  case 1:
    return a(img,img_pix_char());
  case 2:
    return a(img,img_pix_fixed16());
  case 4:
    return a(img,img_pix_int32());
  case 8:
    return a(img,img_pix_float64());
  }
  throw std::runtime_error(
    "img_access_dispatchScalar - unsupported format");
}

template <class Algorithm>
int img_access_dispatch2Scalars(
  typename Algorithm::Param1 p1,
  typename Algorithm::Param2 p2,
  Algorithm& a)
{
  switch (p1.fmt().szPixel()){
  case 1:
    switch (p2.fmt().szPixel()){
    case 1:
      return a(p1,p2, img_pix_char(), img_pix_char());
    case 2:
      return a(p1,p2, img_pix_char(), img_pix_fixed16());
    case 4:
      return a(p1,p2, img_pix_char(), img_pix_int32());
    case 8:
      return a(p1,p2, img_pix_char(), img_pix_float64());
    }
    break;
  case 2:
    switch (p2.fmt().szPixel()){
    case 1:
      return a(p1,p2, img_pix_fixed16(), img_pix_char());
    case 2:
      return a(p1,p2, img_pix_fixed16(), img_pix_fixed16());
    case 4:
      return a(p1,p2, img_pix_fixed16(), img_pix_int32());
    case 8:
      return a(p1,p2, img_pix_fixed16(), img_pix_float64());
    }
    break;
  case 4:
    switch (p2.fmt().szPixel()){
    case 1:
      return a(p1,p2, img_pix_int32(), img_pix_char());
    case 2:
      return a(p1,p2, img_pix_int32(), img_pix_fixed16());
    case 4:
      return a(p1,p2, img_pix_int32(), img_pix_int32());
    case 8:
      return a(p1,p2, img_pix_int32(), img_pix_float64());
    }
  case 8:
    switch (p2.fmt().szPixel()){
    case 1:
      return a(p1,p2, img_pix_float64(), img_pix_char());
    case 2:
      return a(p1,p2, img_pix_float64(), img_pix_fixed16());
    case 4:
      return a(p1,p2, img_pix_float64(), img_pix_int32());
    case 8:
      return a(p1,p2, img_pix_float64(), img_pix_float64());
    }
  }
  throw std::runtime_error(
    "img_access_dispatch2Scalars - unsupported format");
}

}

#endif
