#include "img_access_fill.hpp"

#include "../img_data.hpp"
#include "img_access_dispatch.hpp"
#include "img_access_guard.hpp"
#include "img_access_wrap.hpp"
#include "img_access_pix.hpp"


#if defined(_MSC_VER)
  #if _MSC_VER<1300
    #define min _MIN
    #define max _MAX
  #endif
#endif

namespace{
static const int white_=255;
static const int gray_ =4*white_/5;
static const int colours_[2]= {white_, gray_};

void fillChbdIndex(int size, int width, int height, char* p){
  bool reverseRow=false;
  int ctRow=0;
  for (int row=0; row<height; ++row){
    ++ctRow;
    if (ctRow%size==0){
      reverseRow=!reverseRow;
    }

    bool reverseCol=false;
    int ctCol=0;
    for (int col=0; col<width; ++col){
      ++ctCol;
      if (ctCol%size==0){
        reverseCol=!reverseCol;
      }

      int value=colours_[reverseCol ^ reverseRow];
      p[row*width+col]=static_cast<char>(value);
    }
  }//all rows
}

void fillChbdPtr(int size, int width, int height, char* p){
  typedef char* AccessorDst;

  AccessorDst rowDst(p);
  AccessorDst rowDstEnd(p); 
  rowDstEnd+=height*width;

  bool reverseRow=false;
  int ctRow=0;
  while (rowDst<rowDstEnd){
    AccessorDst dst(rowDst);
    AccessorDst dstEnd(dst);
    dstEnd+=width;

    ++ctRow;
    if (ctRow%size==0){
      reverseRow=!reverseRow;
    }

    bool reverseCol=false;
    int ctCol=0;
    while (dst<dstEnd){
      ++ctCol;
      if (ctCol%size==0){
        reverseCol=!reverseCol;
      }

      int value=colours_[reverseCol ^ reverseRow];
      *dst=static_cast<char>(value);

      ++dst;
    }
    rowDst+=width;
  }//all rows
}

}



///////////////////////////////////////////////
// bugs in MSVC 6.0:
// - member template functions must be defined in-class
// - only implicit resolution of template functions 
//   is supported (hence we need dummy variables)



///////////////////////////////////////////////
// fill with the square checkboard pattern

class FillWorkerOffsetIndex {
private:
  int size_;
public:
  typedef img_data& Param1;
public:
  FillWorkerOffsetIndex(int sz):size_(sz){}
public:
  template <class PixelDst>
  inline int operator() (
    img_data& img_dst, const PixelDst&)
  {
    typedef img_access_guard<img_access_wrap<PixelDst> > AccessorDst;
    AccessorDst dst(img_dst);

    int width  = img_dst.fmt().width();
    int height = img_dst.fmt().height();
    //int white=PixelDst::MaxScalar;
    //int gray =4*white/5;

    bool reverseRow=false;
    int ctRow=0;
    for (int row=0; row<height; ++row){
      ++ctRow;
      if (ctRow%size_==0){
        reverseRow=!reverseRow;
      }

      bool reverseCol=false;
      int ctCol=0;
      for (int col=0; col<width; ++col){
        ++ctCol;
        if (ctCol%size_==0){
          reverseCol=!reverseCol;
        }

        int value=colours_[reverseCol ^ reverseRow];
        dst.seek(row,col);
        dst->setUint(value);
      }
    }//all rows
    return 0;
  }
};

class FillWorkerOffset {
private:
  int size_;
public:
  typedef img_data& Param1;
public:
  FillWorkerOffset(int sz):size_(sz){}
public:
  template <class PixelDst>
  inline int operator() (
    img_data& img_dst, const PixelDst& )
  {
    typedef img_access_guard<img_access_wrap<PixelDst> > AccessorDst;

    int width  = img_dst.fmt().width();
    int height = img_dst.fmt().height();
    //int white=PixelDst::MaxScalar;
    //int gray =4*white/5;

    AccessorDst rowDst(img_dst);
    rowDst.seek(0);
    AccessorDst rowDstEnd(img_dst); 
    rowDstEnd.seek(height);

    bool reverseRow=false;
    int ctRow=0;
    while (rowDst<rowDstEnd){
      AccessorDst dst(rowDst);
      AccessorDst dstEnd(dst);
      dstEnd+=width;

      ++ctRow;
      if (ctRow%size_==0){
        reverseRow=!reverseRow;
      }

      bool reverseCol=false;
      int ctCol=0;
      while (dst<dstEnd){
        ++ctCol;
        if (ctCol%size_==0){
          reverseCol=!reverseCol;
        }

        int value=colours_[reverseCol ^ reverseRow];
        dst->setUint(value);

        ++dst;
      }
      rowDst.incLine();
    }//all rows
    return 0;
  }
};




///////////////////////////////////////////////
// fill with the gradient square checkboard pattern

class FillGradWorkerOffset {
private:
  int size_;
public:
  typedef img_data& Param1;
public:
  FillGradWorkerOffset(int sz):size_(sz){}
public:
  template <class PixelDst>
  inline int operator() (
    img_data& img_dst, const PixelDst&)
  {
    typedef img_access_guard<img_access_wrap<PixelDst> > AccessorDst;

    int width  = img_dst.fmt().width();
    int height = img_dst.fmt().height();
    int white=PixelDst::MaxScalar;

    const bool rowSwitch=(((width-1)/size_)%2)==0;
    const int stepPixel=white/size_;
    bool up=true;

    AccessorDst rowDst(img_dst);
    rowDst.seek(0);
    AccessorDst rowDstChange(rowDst);
    AccessorDst rowDstEnd(img_dst); 
    rowDstEnd.seek(height);

    while (rowDst<rowDstEnd){
      if (rowdiff(rowDstEnd,rowDst)<size_)
        rowDstChange=rowDstEnd;
      else
        rowDstChange.incLine(size_);
      while (rowDst<rowDstChange){
        AccessorDst dst(rowDst);
        AccessorDst dstChange(dst);
        AccessorDst dstEnd(dst);
        dstEnd+=width;

        while (dst<dstEnd){
          if (dstEnd-dst<size_)
            dstChange=dstEnd;
          else
            dstChange+=size_;

          int i, stepAmount;
          if (up){
            i=0;
            stepAmount=stepPixel;
          } else{
            i=white;
            stepAmount=-stepPixel;
          }
          while (dst<dstChange){
            dst->setUint(i);
            ++dst;
            i+=stepAmount;
          }

          up^=1;
        }//all columns
        up^=rowSwitch;
        rowDst.incLine();
      }//rows same colour
      up^=1;
    }//all rows

    return 0;
  }
};










///////////////////
// entry functions

void img_access_fillChbdIndex(
  img_data& dst, int sz)
{
  fillChbdIndex(sz,
    dst.fmt().width(), dst.fmt().height(),
    dst.bits());
}

void img_access_fillChbdPtr(
  img_data& dst, int sz)
{
  fillChbdPtr(sz,
    dst.fmt().width(), dst.fmt().height(),
    dst.bits());
}

void img_access_fillChbdTemplate(
  img_data& dst, int sz)
{
  FillWorkerOffset w(sz);
  img_access_dispatch1(dst,w);
}

void img_access_fillChbdTemplateIndex(
  img_data& dst, int sz)
{
  FillWorkerOffsetIndex w(sz);
  img_access_dispatch1(dst,w);
}


void img_access_fillCheckboard(
  img_data& dst, int sz)
{
  img_access_fillChbdTemplate(dst,sz);
}

void img_access_fillChbdGrad(
  img_data& dst, int sz)
{
  FillGradWorkerOffset w(sz);
  img_access_dispatch1(dst,w);
}




