#ifndef img_data_hpp
#define img_data_hpp

/******************************************************
*
*  img_data by Sinisa Segvic
*
** img_fmt : encapsulation of the image format 
** img_data: image as a contiguous sequence of rows (scanlines)
*
*******************************************************/
#include <assert.h>

class img_fmt{
public:
  enum Masks{MDefault=0, MIntegral=2, MRational=4, MTriple=16};
  enum Type{TDefault=MDefault, 
            TSigned=MIntegral+0, TUnsigned=MIntegral+1,
            TFixed =MRational+0, TFloat   =MRational+17,
            TTriple=MTriple};
//construction
public:
  img_fmt(int w, int h, int szPix, Type t=TDefault);
  img_fmt(const img_fmt& f, int szPix, Type t=TDefault);
  img_fmt makeFmtFloat() const;
private:
  void init();
//attributes
public:
  img_fmt();
  int width() const {return width_;}
  int height() const {return height_;}
  int szPixel() const {return szPixel_;}

  int szLine() const {return szLine_;}
  int szBits() const {return szLine()*height();}
//pixel type
public:
  bool integral() const {return  (type_ & MIntegral)!=0;}
  bool rational() const {return  (type_ & MRational)!=0;}
  bool scalar() const   {return  (type_ & MTriple)==0;}
private:
  int width_;
  int height_;
  int szPixel_;
  int szLine_;
  Type type_;
};
bool operator==(const img_fmt& lhs, const img_fmt& rhs);
inline bool operator!=(const img_fmt& lhs, const img_fmt& rhs){
  return ! (lhs==rhs);
}
inline img_fmt operator/(const img_fmt& lhs, int c){
  return img_fmt(lhs.width()/c, lhs.height()/c, lhs.szPixel());
} 






class img_data{
  img_fmt fmt_;         //should always be valid
  mutable char *pBits_; //!=0 iff bits are valid (format must be valid too)
  bool allocated_;      //true iff instance owns the bits 
//construction
public:
  img_data();
  img_data(const img_fmt& df);
  img_data(const img_data&);
  // constructs a temporary image using an external buffer
  img_data(const img_fmt& df, void* buf, int szbuf=0);
  ~img_data();
//restructuring
  const img_data& operator=(const img_data&);
  void reformat(const img_fmt& df);
//attributes
public:
  const img_fmt& fmt() const {return fmt_;}
  char*       bits();
  char const* bits() const;
// operations
public:
  void setBits(void const* p);
  void setLine(int line, void const* p);
private:
  void unalloc();
};


#endif
