#include "img_util.hpp"
#include "img_data.hpp"
#include "../util/util_endian.hpp"

#include <string.h>
#include <assert.h>
#include <stdexcept>
#include <algorithm>

void img_util::fillBits(img_data& img, int byte){
  memset(img.bits(), byte, img.fmt().szBits());
}


void img_util::depaletise(
  img_data& img,
  unsigned char* pal,
#ifdef NDEBUG
  int )
#else
  int sz)
#endif
{
  if (img.fmt().szPixel()!=1)
    throw std::runtime_error("img_util::depaletise "
        "- unexpected image format");

  char *buf=img.bits();
  for (int i=0; i<img.fmt().szBits(); ++i){
    int index=static_cast<unsigned char>(buf[i]);
    assert(index>=0);
    assert(index<sz);
    buf[i]=pal[index];
  }
}



void img_util::depaletise(
  char* pbits,
  unsigned char* pr,
  unsigned char* pg,
  unsigned char* pb,
#ifdef NDEBUG
  int ,
#else
  int sz,
#endif
  img_data& img)
{
  if (img.fmt().szPixel()!=3)
    throw std::runtime_error("img_util::depaletise "
        "- unexpected image format");

  int width=img.fmt().width();
  int height=img.fmt().height();

  for (int i=0; i<height; ++i){
    char *src=pbits+i*width;
    char *dst=img.bits()+i*img.fmt().szLine();
    for (int j=0; j<width; ++j){
      int index=static_cast<unsigned char>(
        *src);
      assert(index>=0);
      assert(index<sz);

      dst[0]=pb[index];
      dst[1]=pg[index];
      dst[2]=pr[index];

      src+=1;
      dst+=3;
    }
  }

}

void img_util::flipVertical(img_data& img){

  int szLine=img.fmt().szLine();
  char *buf= new char[szLine];
  char* pSrc=img.bits();
  char* pDst=img.bits()+szLine*(img.fmt().height()-1);

  for (int i=0; i<img.fmt().height()/2; ++i){
    memcpy(buf,  pDst, szLine);
    memcpy(pDst, pSrc, szLine);
    memcpy(pSrc, buf,  szLine);

    pSrc+=szLine;
    pDst-=szLine;
  }
  delete [] buf;
}
void img_util::flipVertical(
  const img_data& imgSrc,
  img_data& imgDst)
{
  if (imgSrc.bits()==imgDst.bits()){
    flipVertical(imgDst);
    return;
  }

  imgDst.reformat(imgSrc.fmt());
  int szLine=imgDst.fmt().szLine();
  char const* pSrc=imgSrc.bits();
  char *      pDst=imgDst.bits()+szLine*(imgDst.fmt().height()-1);

  for (int i=0; i<imgDst.fmt().height(); ++i){
    memcpy(pDst, pSrc, szLine);

    pSrc+=szLine;
    pDst-=szLine;
  }
}


void img_util::flipHorizontal(img_data& img){
  int szLine=img.fmt().szLine();
  int szPixel=img.fmt().szPixel();
  for (int i=0; i<img.fmt().height(); ++i){
    char* pSrc=img.bits()+i*szLine;
    char* pDst=pSrc+szLine-szPixel;
    for (int j=0; j<img.fmt().width()/2; ++j){
      assert(pSrc<pDst);
      for (int k=0; k<szPixel; ++k){
        std::swap(pSrc[k], pDst[k]);
      }

      pSrc+=szPixel;
      pDst-=szPixel;
    }
  }
}
void img_util::flipHorizontal(
  const img_data& imgSrc,
  img_data& imgDst)
{
  if (imgSrc.bits()==imgDst.bits()){
    flipHorizontal(imgDst);
    return;
  }

  imgDst.reformat(imgSrc.fmt());
  int szLine=imgDst.fmt().szLine();
  int szPixel=imgDst.fmt().szPixel();
  for (int i=0; i<imgDst.fmt().height(); ++i){
    char const* pSrc=imgSrc.bits()+i*szLine;
    char* pDst=imgDst.bits()+(i+1)*szLine-szPixel;
    for (int j=0; j<imgDst.fmt().width(); ++j){
      for (int k=0; k<szPixel; ++k){
        pDst[k]=pSrc[k];
      }

      pSrc+=szPixel;
      pDst-=szPixel;
    }
  }
}



void img_util::fixEndian(img_data& img, bool endianBig){
  if (endianBig != util_endian::endianBig()){
    flipEndian(img);
  }
}
void img_util::flipEndian(img_data& img){
  switch (img.fmt().szPixel()){
  case 1:
    break;
  case 2:
    flipEndian2(img); break;
  case 3:
    flipEndian3(img); break;
  case 4:
    flipEndian4(img); break;
  default:
    throw std::runtime_error("img_util::flipEndian -"
        "- unknown image format");
  }
}

void img_util::flipEndian2(img_data& img){
  assert(img.fmt().szPixel()==2);

  int szLine=img.fmt().szLine();
  char* pRow=img.bits();

  for (int i=0; i<img.fmt().height(); ++i){
    char* p=pRow;
    for (int j=0; j<img.fmt().width(); ++j){
      char pom=*p;
      *p=*(p+1);
      *(p+1)=pom;

      p+=2;
    }
    pRow+=szLine;
  }
}

void img_util::flipEndianBits3(
  const img_fmt& fmt, char* bits)
{
  assert(fmt.szPixel()==3);

  int szLine=fmt.szLine();
  char* pRow=bits;

  for (int i=0; i<fmt.height(); ++i){
    char* p=pRow;
    for (int j=0; j<fmt.width(); ++j){
      char pom=*p;
      *p=*(p+2);
      *(p+2)=pom;

      p+=3;
    }
    pRow+=szLine;
  }
}

void img_util::flipEndian3(img_data& img){
  flipEndianBits3(img.fmt(), img.bits());
}

void img_util::flipEndian4(img_data& img){
  assert(img.fmt().szPixel()==4);

  int szLine=img.fmt().szLine();
  char* pRow=img.bits();

  for (int i=0; i<img.fmt().height(); ++i){
    char* p=pRow;
    for (int j=0; j<img.fmt().width(); ++j){
      char pom=*p;
      *p=*(p+2);
      *(p+2)=pom;

      p+=4;
    }
    pRow+=szLine;
  }
}


////////////////////////////////
// building test images

void img_util::fillWithRGBGradient(img_data& img){
  int i,j;
  int w=img.fmt().width();
  int h=img.fmt().height();
  char *I=img.bits();

  if (img.fmt().szPixel()!=3)
    img.reformat(img_fmt(w,h,3));
  assert(img.fmt().szPixel()==3);

  for (i=0; i<w; ++i){
    for (j=0; j<h; ++j){
      int d, offset;
      int d1=i-j;
      int d2=i+j-h;
      if (d1<0) d1=-d1;
      if (d2<0) d2=-d2;
      d=d1+d2;
      offset= 3*(j*w+i);
      *(I+offset)=
      *(I+offset+1)=
      *(I+offset+2)= 
        static_cast<char>(d*255/(w+h));
    }
  }
}

