#include "img_io_load.hpp"

#include "../../util/util_registry.hpp"
#include "../../util/util_string.hpp"

#include <boost/smart_ptr.hpp>

#include <sstream>

////////////////////////
// img_io_load static methods

img_io_load* img_io_load::create(
  const char* extension)
{
  CtorArg dummy;
  return util::Registry::instance().
    create<img_io_load>(extension, dummy);
}



void img_io_load::load(
  const char* filename, img_data& img)
{
  std::string ext(
    util_string::getFileExtension(filename));
  util_string::tolower(ext);
  boost::scoped_ptr<img_io_load> spio(create(ext.c_str()));
  (*spio)(filename, img);
}


/////////////////////
// img_io_load::ReadFile

img_io_load::ReadFile::ReadFile(
  char const* name, 
  char const* context)
:
  pf_(fopen(name, "rb")),
  ctx_(context),
  name_(name)
{
  if (!pf_) {
    std::ostringstream oss;
    oss <<ctx_ <<" - can't open " <<name;
    throw std::runtime_error(oss.str());
  }

  ::fseek(pf_, 0, SEEK_END);
  size_=::ftell(pf_);
  ::fseek(pf_, 0, SEEK_SET);
}
img_io_load::ReadFile::~ReadFile(){
  fclose(pf_);
}

size_t img_io_load::ReadFile::size()
{
  return size_;
}

void img_io_load::ReadFile::seek(
  size_t fpos)
{
  ::fseek(pf_, fpos, SEEK_SET);
}

void img_io_load::ReadFile::advance(
  long fpos)
{
  ::fseek(pf_, fpos, SEEK_CUR);
}

void img_io_load::ReadFile::read(
  void* buf, int count)
{
  int rv=static_cast<int>(::fread(buf, 1, count, pf_));
  if ( rv!=count){
    std::ostringstream oss;
    oss <<ctx_ <<" - got premature eof while reading " <<name_;
    throw std::runtime_error(oss.str());
  }
  if (ferror(pf_)){
    std::ostringstream oss;
    oss <<ctx_ <<" - got error while reading " <<name_;
    throw std::runtime_error(oss.str());
  }
}

