#ifndef img_io_load_hpp
#define img_io_load_hpp

class img_fmt;
class img_data;
class img_io_load_imp;

#include <string>

#include <stdio.h>

/******************************************************
*
** img_io_load by Sinisa Segvic
*
** class hierarchy based at img_io_load
** manages reading images from various
** file formats to a img_data structure
*
 ** in order to support factory method,
 ** each concrete class should:
 **  - declare public constructor with a single 
 **    argument of the type CtorArg 
 **  - define static data member of the type 
 **    util::RegistryCreator<alg_base,alg_concrete>
 **    which interfaces to the registry object
 **  - define static member s_name() returning a 
 **    distinguishing name used for identification;
 **    s_name() may be used for implementing name()
 *
*******************************************************/


class img_io_load{
protected:
  img_io_load(){}
public:
  virtual ~img_io_load(){}

// identification
public:
  virtual char const* name() =0;
  
// mandatory operations
public:
  virtual void operator()(char const* file, img_data&)=0;

// img_io_load construction
public:
  // concrete classes ctor argument
  class CtorArg{
    char dummy;
  };
  
public:
  //factory for the concrete classes 
  static img_io_load* create(char const* extension);
  static void load(char const* file, img_data& img);

public:
  class ReadFile{
  public:
    ReadFile(char const* file, char const* context);
    ~ReadFile();
    size_t size();
    void seek(size_t fpos);
    void advance(long fpos);
    void read(void* p, int count);
  private:
    FILE* pf_;
    std::string ctx_;
    std::string name_;
    size_t size_;
  };

//forbidden
private:
  img_io_load(img_io_load&);
  void operator=(img_io_load&);
};


#endif

