#include "img_io_load.hpp"

#include "../img_data.hpp"
#include "../img_util.hpp"
#include "../../util/util_endian.hpp"
#include "../../util/util_registry.hpp"

#include <assert.h>
#include <string.h>

#include <stdexcept>
#include <sstream>
#include <string>

/******************************************************
*
** img_io_load_pxm.cpp by Sinisa Segvic
*
** reading images from bmp file format
** to a img_data structure
*
** depends on:
**   img_data, img_io_load, img_util, 
**   util_pxm 
*
*******************************************************/

class img_io_load_pxm:
  public img_io_load
{
public:
  virtual char const* name() =0;

  virtual void operator()(char const* name, img_data&);
};


////////////////////////////////////////
// img_io_load_pxm

void img_io_load_pxm::operator()(
  char const* name, 
  img_data& img)
{
  ReadFile rf(name, "img_io_load_pxm");
    
  // get magic string
  char c, n;
  rf.read(&c,1);
  rf.read(&n,1);
  if (c!='P' || (n!='5' && n!='6'))
    throw std::runtime_error("img_io_load_pxm - illegal magic string");
  int szpix=1;
  if (n=='6')
    szpix=3;

  //skip spaces and comments
  rf.read(&c,1);
  while(!isdigit(c)){
    if (c=='#') while(c!='\n') rf.read(&c,1);
    while(isspace(c)) rf.read(&c,1);
  }
  //get width
  std::string str;
  while(!isspace(c)){
   str+=c;
   rf.read(&c,1);
  }
  int w=atoi(str.c_str());
  if (w<0)
    throw std::runtime_error("img_io_load_pxm - negative width");

  //skip spaces and comments
  while(!isdigit(c)){
    if (c=='#') while(c!='\n') rf.read(&c,1);
    while(isspace(c)) rf.read(&c,1);
  }
  //get height
  str.clear();
  while(!isspace(c)){
   str+=c;
   rf.read(&c,1);
  }
  int h=atoi(str.c_str());
  if (w<0)
    throw std::runtime_error("img_io_load_pxm - negative height");
  
  //skip spaces and comments
  while(!isdigit(c)){
    if (c=='#') while(c!='\n') rf.read(&c,1);
    while(isspace(c)) rf.read(&c,1);
  }
  //get maxval
  str.clear();
  while(!isspace(c)){
   str+=c;
   rf.read(&c,1);
  }
  int mv=atoi(str.c_str());
  if (mv>255){
    std::ostringstream oss;
    oss <<"img_io_load_pxm - illegal maxval: " <<mv;
    throw std::runtime_error(oss.str());
  }
  
  //read bits
  img.reformat(img_fmt(w,h,szpix));
  rf.read(img.bits(), img.fmt().szBits());

  //we use rgbrgbrgb for 24bpp images

  //pxm maps scanlines in a top-down manner
  img_util::flipVertical(img);

}

////////////////////////////////////////
// img_io_load_ppm

class img_io_load_ppm: public img_io_load_pxm{
public:
  img_io_load_ppm(const CtorArg& ) {}
  ~img_io_load_ppm() {}
  static char const* s_name(){return "ppm";}
  virtual char const* name(){return s_name();}
};

////////////////////////////////////////
// img_io_load_pgm

class img_io_load_pgm: public img_io_load_pxm{
public:
  img_io_load_pgm(const CtorArg& ) {}
  ~img_io_load_pgm() {}
  static char const* s_name(){return "pgm";}
  virtual char const* name(){return s_name();}
};


////////////////////////////////////
// static registry creator objects

namespace{
  util::RegistryCreator<img_io_load,img_io_load_ppm> creator_ppm;
  util::RegistryCreator<img_io_load,img_io_load_pgm> creator_pgm;
} // unnamed namespace end


