#include <stdio.h>


#include "../../util/util_bmp.hpp"
#include "../img_data.hpp"
#include "../img_util.hpp"

#include "img_io_save_bmp.hpp"


img_io_save_bmp::img_io_save_bmp(char const* name):
  img_io_save(name)
{}

img_io_save_bmp::img_io_save_bmp():
  img_io_save()
{}

img_io_save_bmp::~img_io_save_bmp(){
}

void img_io_save_bmp::operator()(
  const img_data& img)
{
  util_bmp_BFH bfh;
  util_bmp_BI bi;
  int sz_bi=bi.init(
    img.fmt().szPixel()*8, img.fmt().width(), img.fmt().height());
  int sz_bfh=bfh.init(sz_bi, img.fmt().szBits());

  write(&bfh, sz_bfh);
  write(&bi, sz_bi);

  img_data imgTmp(img);

  //16bpp imapges are stored in little endian
  //unlike BMP format, we use rgbrgbrgb for 24bpp images
  //unlike BMP format, we use rgbxrgbxrgbx for 32bpp images
  switch (imgTmp.fmt().szPixel()){
    case 1: break;
    case 2:
      if (util_endian::endianBig())
        img_util::flipEndian2(imgTmp);
      break;
    case 3:
      img_util::flipEndian(imgTmp);
      break;
    default:
      throw "img_io_save_bmp::operator() - "
            "illegal szPixel()";
  }
  //bmp maps scanlines in a bottom-up manner!
  
  int szline=imgTmp.fmt().szLine();
  int padding=4*((szline+3)/4)-szline;
  
  char const*p=imgTmp.bits();
  for(int i=0; i<imgTmp.fmt().height(); ++i){
    write(p, szline);
    write(p, padding);
    p+=szline;
  }
}


//  static 
char const* img_io_save_bmp::extension(){
  return "bmp";
}


