#include "img_io_save_pxm.hpp"

#include "../../util/util_endian.hpp"
#include "../img_data.hpp"
#include "../img_util.hpp"
#include "../access/img_access_copy.hpp"

#include <assert.h>
#include <string.h>

#include <string>
#include <iostream>
#include <sstream>
#include <stdexcept>



img_io_save_pxm::img_io_save_pxm(char const* name):
  img_io_save(name)
{}

img_io_save_pxm::img_io_save_pxm():
  img_io_save()
{}


img_io_save_pxm::~img_io_save_pxm(){
}

void img_io_save_pxm::operator()(
  const img_data& img)
{
  std::string magic("P6\n");
  if (img.fmt().szPixel()<3)
    magic[1]='5';
  write(magic.c_str(), static_cast<int>(magic.size()));

  std::ostringstream oss;
  oss <<img.fmt().width() <<" " <<img.fmt().height() <<"\n";
  if (img.fmt().szPixel()==2)
    oss <<"65535\n";
  else
    oss <<"255\n";
  write(oss.str().c_str(), static_cast<int>(oss.str().size()));
  

  //16bpp images are stored in little endian
  //we use rgbrgbrgb for 24bpp images
  //we use rgbxrgbxrgbx for 32bpp images
  img_data imgTmp;
  switch (imgTmp.fmt().szPixel()){
    case 1:
    case 3:
      imgTmp=img;
      break;
    case 4:
      imgTmp.reformat(img_fmt(
        img.fmt().width(), img.fmt().height(), 3));
      img_access_copy(img, imgTmp);
      break;
    case 2:
      imgTmp=img;
      if (util_endian::endianBig())
        img_util::flipEndian2(imgTmp);
      break;
    default:
      throw std::runtime_error("img_io_save_pxm::operator() "
          "- illegal szPixel()");
  }
  //PXM maps scanlines in a top-down manner
  img_util::flipVertical(imgTmp);
  write(imgTmp.bits(), imgTmp.fmt().szBits());
}



//  static 
char const* img_io_save_pxm::extension1(){
  return "pgm";
}
char const* img_io_save_pxm::extension3(){
  return "ppm";
}


//Source:
/*******************************************************************************
* FILE: pgm_io.c
* This code was written by Mike Heath. heath@csee.usf.edu (in 1995).
*******************************************************************************/

