#include "img_io_save_ras.hpp"

#include "../img_data.hpp"
#include "../img_util.hpp"

#include "../access/img_access_copy.hpp"
#include "../access/img_access_fill.hpp"

#include "../../util/util_ras.hpp"
#include "../../util/util_endian.hpp"

#include <stdio.h>


img_io_save_ras::img_io_save_ras(char const* name):
  img_io_save(name)
{}

img_io_save_ras::img_io_save_ras():
  img_io_save()
{}


img_io_save_ras::~img_io_save_ras(){
}

void img_io_save_ras::operator()(
  const img_data& img)
{
  // 16bpp images are not supported
  int szpix=img.fmt().szPixel();
  if (szpix==2) szpix=3;
  img_data imgTmp(img_fmt(
    img.fmt().width(), img.fmt().height(), szpix));
  img_access_copy(img, imgTmp);

  util_ras_hdr rfh;
  rfh.init(imgTmp.fmt().szPixel()*8, 
    img.fmt().width(), 
    img.fmt().height());
  write(&rfh, sizeof(rfh));


  //16bpp images are stored in little endian
  //we use rgbrgbrgb for 24bpp images
  //we use rgbxrgbxrgbx for 32bpp images
  switch (imgTmp.fmt().szPixel()){
    case 1: 
      break;
    case 2:
      if (util_endian::endianBig())
        img_util::flipEndian2(imgTmp);
      break;
    case 3:
    case 4:
      img_util::flipEndian(imgTmp);
      break;
    default:
      throw "img_io_save_ras::operator() - "
            "illegal szPixel()";
  }
  //sr maps scanlines in a top-down manner
  img_util::flipVertical(imgTmp);
  write(imgTmp.bits(), imgTmp.fmt().szBits());
}




//  static 
char const* img_io_save_ras::extension(){
  return "ras";
}


//source:
//from /usr/openwin/share/include/pixrect/rasterfile.h

//#define RT_OLD		0	/* Raw pixrect image in 68000 byte order */
//#define RT_STANDARD	1	/* Raw pixrect image in 68000 byte order */
//#define RT_BYTE_ENCODED	2	/* Run-length compression of bytes */
//#define RT_FORMAT_RGB	3	/* XRGB or RGB instead of XBGR or BGR */
//#define RT_FORMAT_TIFF	4	/* tiff <-> standard rasterfile */
//#define RT_FORMAT_IFF	5	/* iff (TAAC format) <-> standard rasterfile */
//#define RT_EXPERIMENTAL 0xffff	/* Reserved for testing */

///* Sun registered ras_maptype's */
//#define RMT_RAW		2
///* Sun supported ras_maptype's */
//#define RMT_NONE	0	/* ras_maplength is expected to be 0 */
//#define RMT_EQUAL_RGB	1	/* red[ras_maplength/3],green[],blue[] */


