#include "img_io_save_util.hpp"

#include "img_io_save.hpp"
#include "img_io_save_bmp.hpp"
#include "img_io_save_ras.hpp"
#include "img_io_save_pxm.hpp"

#include "../img_data.hpp"
#include "../access/img_access_copy.hpp"

#include "../../util/util_string.hpp"

#include <stdio.h>
#include <string.h>

#include <memory>
#include <stdexcept>


// sun raster file format is assumed
void img_io_save_util::save(
  const img_data& img)
{
  img_io_save_ras ds;//stdout assumed
  ds(img);
}

// file format is deduced from extension
void img_io_save_util::save(
  const img_data& img, 
  char const* outfile)
{
  std::auto_ptr<img_io_save> pds(
    createSaver(outfile));
  (*pds)(img);
}
void img_io_save_util::savegrey(
  const img_data& d, char const* p)
{
  if (d.fmt().szPixel()!=1){
    img_data tmp(img_fmt(d.fmt(),1));
    img_access_copy(d,tmp);
    save(tmp,p);
  } else{
    save(d,p);
  }
}




img_io_save* img_io_save_util::createSaver(char const* outfile){
  char const* ext=util_string::getFileExtension(outfile);
  if (!util_string::compareNoCase(img_io_save_bmp::extension(), ext))
    return new img_io_save_bmp(outfile);
  if (!util_string::compareNoCase(img_io_save_ras::extension(), ext))
    return new img_io_save_ras(outfile);
  if (!util_string::compareNoCase(img_io_save_pxm::extension1(), ext) ||
      !util_string::compareNoCase(img_io_save_pxm::extension3(), ext))
    return new img_io_save_pxm(outfile);

  throw std::runtime_error("img_io_save_util::createSaver "
      "- unknown extension!");
}






