#include "ipp_add.hpp"

#include "../img/img_data.hpp"
#include "../img/access/img_access_dispatch.hpp"
#include "../img/access/img_access_guard.hpp"
#include "../img/access/img_access_wrap.hpp"
#include "../img/access/img_access_pix.hpp"
#include "../img/access/img_access_comb.hpp"

#include <stdexcept>




template<class PixelSrc,class PixelDst>
void AddWorker(
  const img_data& img_src, 
  img_data& img_dst)
{
  //standard prolog
  typedef img_access_guard<img_access_wrap_const<PixelSrc> > AccessorSrc;
  typedef img_access_guard<img_access_wrap<PixelDst> > AccessorDst;

  int w=img_src.fmt().width();
  int h=img_src.fmt().height();
  if (w !=img_dst.fmt().width() ||
      h !=img_dst.fmt().height())
  {
    throw std::runtime_error("ipp::add - incompatible formats");
  }

  AccessorSrc itSrcRow(img_src);
  AccessorDst itDstRow(img_dst);
  for (int row=0; row<h; ++row){
    AccessorSrc itSrc(itSrcRow);
    AccessorDst itDst(itDstRow);
    for (int col=0; col<w; ++col){
      itDst->setScalar(itSrc->scalar()+itDst->scalar());
      ++itDst;
      ++itSrc;
    }
    itSrcRow.incLine();
    itDstRow.incLine();
  }
}






///////////////////////////////////////////////
// class AddCaller
// typedefs are used by dispatch template functions

class AddCaller{
  img_data& dst_;
public:
  typedef const img_data& Param1;
  typedef img_data& Param2;
public:
  AddCaller(img_data& dst): dst_(dst) {}
public:
  template <class PixelSrc>
  inline int operator() (const img_data& src, const PixelSrc&)
  {
    switch (sizeof(PixelSrc)){
    case 1:
      AddWorker<PixelSrc, img_pix_int32>(src, dst_);
      break;
    case 3:
    case 6:
      AddWorker<PixelSrc, img_pix_3fixed16>(src, dst_);
      break;
    default:
      AddWorker<PixelSrc, img_pix_float64>(src, dst_);
    }
    return 0;
  }
};



void ipp::add(
  const img_data& src, 
  img_data& dst)
{
  AddCaller ac(dst);
  img_access_dispatch1(src,ac);
}
  

