#include "ipp_addMask.hpp"
#include "ipp_loop.hpp"

#include "../img/access/img_access_dispatch.hpp"
#include "../img/access/img_access_pix.hpp"

#include "../util/util_cxx.hpp"

#include <iostream>
#include <algorithm>
#include <stdlib.h>


//////////////////////////////////////////////
// the worker class
// * provides function: process
// * supposed to be called from loop

template <class PixelSrc, class PixelDst>
class AddMaskThWorker{
public:
  typedef img_access_guard<img_access_wrap_const
    <PixelSrc> > AccessorSrc;
  typedef img_access_guard<img_access_wrap
    <PixelDst> > AccessorDst;
  typedef typename PixelSrc::EffScalar SrcScalar;
private:
  SrcScalar th_;
public:
  AddMaskThWorker(double t);
  void process(
      const AccessorSrc& psrc, 
      const AccessorDst& pdst); 
};

template <class PixelSrc, class PixelDst>
AddMaskThWorker<PixelSrc,PixelDst>::
AddMaskThWorker(double t){
  PixelSrc p;
  p.setScalar(t);
  th_=p.effScalar();
}

template <class PixelSrc, class PixelDst>
void AddMaskThWorker<PixelSrc,PixelDst>::
process(
  const AccessorSrc& psrc, 
  const AccessorDst& pdst)
{
  SrcScalar value=psrc->effScalar();
  if (value<th_){
    pdst->setScalar(0);
    //std::cout <<"X";
  }
  //std::cout <<"C" <<value <<" to th: " <<th_ <<"\n";
}



///////////////////////////////////////////////
// classes for marshalling parameters
// * member template functions defined in-class
//   because of the buggy MS compiler
// * dummies needed because the buggy MSVC supports 
//   only implicit resolution of template functions
// * typedefs are used by dispatch template functions

class AddMaskThCaller{
  double th_;
public:
  typedef const img_data& Param1;
  typedef img_data& Param2;
public:
  AddMaskThCaller(double t):th_(t){}

public:
  template <class PixelSrc, class PixelDst>
  inline int operator() (
    const img_data& src, img_data& dst, 
    const PixelSrc& dummy1, const PixelDst& dummy2)
  {
    AddMaskThWorker<PixelSrc,PixelDst> w(th_);
    loop2(src, dst, 0, w, dummy1, dummy2);
    return 0;
  }
};

class AddMaskMarginCaller{
  int margin_;
public:
  typedef img_data& Param1;
public:
  AddMaskMarginCaller(int m):margin_(m){}

public:
  template <class Pixel>
  inline int operator() (
    img_data& img, const Pixel& )
  {
    //standard prolog
    int width =img.fmt().width();
    int height=img.fmt().height();
    typedef img_access_guard<img_access_wrap
      <Pixel> > Accessor;

    //bottom margin rows
    Accessor row(img);
    Accessor rowEnd(row); 
    rowEnd.seek(margin_);
    while (row<rowEnd){
      Accessor col(row);
      Accessor colEnd(row);
      colEnd+=width;
      while (col<colEnd){
        img_pix_black(*col);
        ++col;
      }
      row.incLine();
    }//rows

    //middle margin columns
    rowEnd.seek(height-margin_);
    while (row<rowEnd){
      Accessor col(row);
      Accessor colEnd(row);
      colEnd+=margin_;
      while (col<colEnd){
        img_pix_black(*col);
        ++col;
      }
      colEnd+=width-margin_;
      col+=width-2*margin_;
      while (col<colEnd){
        img_pix_black(*col);
        ++col;
      }
      row.incLine();
    }//rows

    //top margin rows
    rowEnd.seek(height);
    while (row<rowEnd){
      Accessor col(row);
      Accessor colEnd(row);
      colEnd+=width;
      while (col<colEnd){
        img_pix_black(*col);
        ++col;
      }
      row.incLine();
    }//rows

    return 0;
  }
};


// entry function
void ipp::addMaskThreshold(
  const img_data& src, double th,
  img_data& dst)
{
  AddMaskThCaller c(th);
  img_access_dispatch2Scalars(src,dst, c);
}

// entry function
void ipp::addMaskMargin(
  img_data& img, int margin)
{
  AddMaskMarginCaller c(margin);
  img_access_dispatch1(img, c);
}

