#include "ipp_annotations.hpp"


#include "../grid/root/grid_primitives.hpp"
#include "../grid/root/grid_region.hpp"

#include "../win/ann/win_ann_abstract.hpp"


#include <sstream>


//////////////////////////
// local functions

namespace{

void annotateContour(
  const grid::ContourWt& contour, 
  int id, char const* str, 
  win_ann_abstract& ann)  
{
  int sz=static_cast<int>(contour.size());

  grid::Contour ctmp;
  ctmp.resize(sz);
  for (int j=0; j<sz; j++)
    ctmp[j]=contour[j];

  ann.addContour(ctmp, id, str, 
    win_ann_abstract::PS_Dot);  
}

} // end unnamed namespace

////////////////////////////////////
//ContourWt

void ipp::annotateContoursNodata(
  const std::vector<grid::WtContourWt>& contours,
  win_ann_abstract& ann)
{
  for (unsigned i=0; i<contours.size(); i++)
    annotateContour(contours[i], i,
      "", ann);
}

void ipp::annotateContoursId(
  const std::vector<grid::WtContourWt>& contours,
  win_ann_abstract& ann)
{
  for (unsigned i=0; i<contours.size(); i++){
    const grid::WtContourWt& cw=contours[i];
    std::ostringstream oss;
    oss <<i;
        //<<':' <<cw.wt();

    annotateContour(cw, i,
      oss.str().c_str(), ann);
  }
}

void ipp::annotateContoursIdSz(
  const std::vector<grid::WtContourWt>& contours,
  win_ann_abstract& ann)
{
  for (unsigned i=0; i<contours.size(); i++){
    const grid::WtContourWt& cw=contours[i];
    std::ostringstream oss;
    oss <<i 
        <<':' <<static_cast<int>(cw.size()); 
//        <<':' <<cw.wt();

    annotateContour(cw, i,
      oss.str().c_str(), ann);
  }
}

////////////////////////////////////
//Contour

void ipp::annotatePlainContoursIdSz(
  const std::vector<grid::Contour>& contours,
  win_ann_abstract& ann)
{
  for (unsigned i=0; i<contours.size(); i++){
    const grid::Contour& ct=contours[i];
    std::ostringstream oss;
    oss <<i
        <<':' <<static_cast<int>(ct.size()); 
    ann.addContour(ct, i, oss.str().c_str(), 
      win_ann_abstract::PS_Dot);  
  }
}

void ipp::annotatePlainContoursId(
  const std::vector<grid::Contour>& contours,
  win_ann_abstract& ann)
{
  for (unsigned i=0; i<contours.size(); i++){
    const grid::Contour& ct=contours[i];
    std::ostringstream oss;
    oss <<i;
    ann.addContour(ct, i, oss.str().c_str(),
      win_ann_abstract::PS_Dot);  
  }
}

////////////////////////////////////
//Regions

void ipp::annotateRegion(
  const grid::WtRegion& r, 
  int colour, char const* str, 
  win_ann_abstract& ann)
{
  annotateRectangle(
    r.left(), r.bottom(), r.right(), r.top(),
    colour, str,
    win_ann_abstract::LS_Normal, 1, ann);
}  

void ipp::annotateRegions(
  const std::vector<grid::WtRegion>& regions,
  win_ann_abstract& ann)
{
  for (unsigned i=0; i<regions.size(); ++i){
    std::ostringstream oss;
    oss <<i;
    annotateRegion(regions[i], 0, oss.str().c_str(), ann);
  }
}  

////////////////////////////////////
//Rectangle
void ipp::annotateRectangle(
  int left, int bottom, int right, int top,
  int colour, char const* str,
  win_ann_abstract::LineStyle style, 
  int width,
  win_ann_abstract& ann)
{
  ann.addLine(grid::Line(
    grid::Pixel(left,bottom), 
    grid::Pixel(right,bottom)), 
    colour, "", style, width);
  ann.addLine(grid::Line(
    grid::Pixel(left,bottom), 
    grid::Pixel(left,top)), 
    colour, "", style, width);
  ann.addLine(grid::Line(
    grid::Pixel(right,bottom), 
    grid::Pixel(right,top)), 
    colour, "", style, width);
  ann.addLine(grid::Line(
    grid::Pixel(left,top), 
    grid::Pixel(right,top)), 
    colour, str, style, width);

}
