#include "ipp_binarize.hpp"

#include "../img/img_data.hpp"
#include "../img/access/img_access_dispatch.hpp"
#include "../img/access/img_access_guard.hpp"
#include "../img/access/img_access_wrap.hpp"
#include "../img/access/img_access_pix.hpp"
#include "../img/access/img_access_comb.hpp"

#include <stdexcept>

#include <math.h>




///////////////////////////////////////////////
// do_binarize - 
//   finds non-maximal suppression mask

template<class PixelSrc, class PixelDst>
static void do_binarize(
  const img_data& img_src,
  int threshold, int margin,
  img_data& img_dst,
  const PixelSrc&, const PixelDst&)
{
  //standard prolog
  typedef img_access_guard<img_access_wrap_const<PixelSrc> > AccessorSrc;
  typedef img_access_guard<img_access_wrap<PixelDst> > AccessorDst;

  int width =img_src.fmt().width();
  int height=img_src.fmt().height();
  if (
    width !=img_dst.fmt().width() ||
    height!=img_dst.fmt().height())
      throw std::runtime_error("do_binarize() - got images of different sizes");

  //binarizing algorithm
  typedef float Data;

  AccessorSrc rowSrc(img_src);
  rowSrc.seek(margin, margin);
  AccessorSrc rowSrcEnd(img_src); 
  rowSrcEnd.seek(height-margin);
  
  AccessorDst rowDst(img_dst);
  rowDst.seek(margin, margin);

  while (rowSrc<rowSrcEnd){
    AccessorDst dst(rowDst);

    AccessorSrc src(rowSrc);
    AccessorSrc srcEnd(rowSrc);
    srcEnd+=width-2*margin;

    while (src<srcEnd){
      typename PixelSrc::Scalar mag=src->scalar();
      if (mag>threshold)
        img_pix_white(*dst);
      else
        img_pix_black(*dst);

      ++src;++dst;
    }
    rowSrc.incLine();
    rowDst.incLine();
  }//all rows
}


///////////////////////////////////////////////
// class BinarizeCaller

// member template functions defined in-class
// because of the buggy MS compiler

// dummy needed because the buggy MSVC supports 
// only implicit resolution of template functions

// typedefs are used by dispatch template functions

class BinarizeCaller{
  int threshold_;
  int margin_;
public:
  typedef const img_data& Param1;
  typedef img_data& Param2;
public:
  BinarizeCaller(int t, int m):
    threshold_(t), margin_(m) {}

public:
  template <class PixelSrc, class PixelDst>
  inline int operator() (
    const img_data& src, img_data& binary, 
    const PixelSrc& dummy1, const PixelDst& dummy2)
  {
    do_binarize(
      src, threshold_, margin_, binary, dummy1, dummy2);
    return margin_;
  }
};





int ipp::binarize(
  const img_data& img_src, 
  int threshold, int margin,
  img_data& img_binary)
{
  BinarizeCaller w(threshold, margin);
  return img_access_dispatch2Scalars(img_src,img_binary, w);
}
  

