#include "ipp_findMax.hpp"

#include "../img/img_data.hpp"
#include "../img/img_util.hpp"
#include "../img/access/img_access_dispatch.hpp"
#include "../img/access/img_access_guard.hpp"
#include "../img/access/img_access_wrap.hpp"
#include "../img/access/img_access_pix.hpp"

#include "../grid/root/grid_primitives.hpp"

#include <algorithm>
#include <iostream>

#include <assert.h>


class MaxWorker{
  int x0_,x1_, y0_,y1_;
  grid::Pixel& pt_;
  double max_;
public:
  typedef const img_data& Param1;
public:
  MaxWorker(
    int x0,int x1, int y0,int y1,
    grid::Pixel& pt)
  :
    x0_(x0),x1_(x1), y0_(y0),y1_(y1), 
    pt_(pt)
  {}
  double max(){return max_;}
public:
  template <class Pixel>
  inline int operator() (
    const img_data& src, const Pixel& )
  {
    img_access_guard<img_access_wrap_const<Pixel> > psrc(src);
    psrc.seek(y0_, x0_);
    double effmax=psrc->effScalar();
    int maxx=x0_;
    int maxy=y0_;
    
    for (int y=y0_; y<y1_; ++y){
      for (int x=x0_; x<x1_; ++x){
        psrc.seek(y, x);
        if (psrc->effScalar()>effmax){
          effmax=psrc->effScalar();
          maxx=x;
          maxy=y;
        }
      }
    }
    
    //psrc.seek(maxy, maxx);
    //max_=psrc->scalar();
    max_=effmax;
    pt_=grid::Pixel(maxx, maxy);
    return 0;
  }
};

double ipp::findMax(
  const img_data& src, 
  int x0, int x1, int y0, int y1,
  grid::Pixel& pt)
{
  x1=std::min(x1, src.fmt().width());
  y1=std::min(y1, src.fmt().height());
  assert(x0>=0 && x0<x1);
  assert(y0>=0 && y0<y1);

  MaxWorker w(x0,x1, y0,y1, pt);
  img_access_dispatch1(src, w);
  return w.max();
}
