#include "ipp_homography.hpp"
#include "ipp_transform.hpp"

#include "../img/img_data.hpp"
#include "../img/access/img_access_dispatch.hpp"

#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/io.hpp>

#include <string.h>

using boost::numeric::ublas::matrix;


namespace{

class TransformHomography{
  typedef boost::numeric::ublas::vector<double> Vector;
  matrix<double> Hi_;  
public:
  TransformHomography(const matrix<double>& Hi):Hi_(Hi){}

  inline bool operator()(
    int idst, int jdst, 
    double& isrc, double& jsrc)
  {
    Vector y(3);
    y(0)=jdst;
    y(1)=idst;
    y(2)=1;
    Vector x(prod(Hi_,y));
    assert( !memcmp(Vector(prod(Hi_,y)).data().begin(),
                    x.data().begin(), 3*sizeof(x(0))));
    
    bool rv=false;
    double t=x(2);
    if (t!=0){
      rv=true;
      isrc=x(1)/t;
      jsrc=x(0)/t;
    }
    return rv;
  }
};

}

/////////////////////////////
// entry functions

void ipp::homography(
  const img_data& src,
  const boost::numeric::ublas::matrix<double>& Hi,
  img_data& dst)
{
  TransformHomography th(Hi);
  TransformLoop<TransformHomography> tl(th);
  img_access_dispatch2Scalars(src,dst, tl);
}
