#include "ipp_lincomb.hpp"

#include "../img/img_data.hpp"
#include "../img/access/img_access_dispatch.hpp"
#include "../img/access/img_access_guard.hpp"
#include "../img/access/img_access_wrap.hpp"
#include "../img/access/img_access_pix.hpp"
#include "../img/access/img_access_comb.hpp"

#include <stdexcept>




template<class PixelSrc,class PixelDst>
void LincombWorker(
  const img_data& img_src, 
  img_data& img_dst,
  double a, double b)
{
  //standard prolog
  typedef img_access_guard<img_access_wrap_const<PixelSrc> > AccessorSrc;
  typedef img_access_guard<img_access_wrap<PixelDst> > AccessorDst;

  int w=img_src.fmt().width();
  int h=img_src.fmt().height();
  if (w !=img_dst.fmt().width() ||
      h !=img_dst.fmt().height())
  {
    throw std::runtime_error("ipp::lincomb - incompatible formats");
  }

  typename PixelSrc::Combination comb;

  AccessorSrc itSrcRow(img_src);
  AccessorDst itDstRow(img_dst);
  for (int row=0; row<h; ++row){
    AccessorSrc itSrc(itSrcRow);
    AccessorDst itDst(itDstRow);
    for (int col=0; col<w; ++col){
      comb.reset();
      comb.add(*itSrc,a);
      comb.add(*itDst,b);
      comb.put(*itDst);
      ++itDst;
      ++itSrc;
    }
    itSrcRow.incLine();
    itDstRow.incLine();
  }
}






///////////////////////////////////////////////
// class LincombCaller
// typedefs are used by dispatch template functions

class LincombCaller{
  double a_;
  double b_;
public:
  typedef const img_data& Param1;
  typedef img_data& Param2;
public:
  LincombCaller(double a, double b): a_(a),b_(b) {}
public:
  template <class PixelSrc, class PixelDst>
  inline int operator() (
    const img_data& src, 
    img_data& dst, 
    const PixelSrc&,
    const PixelDst&)
  {
    LincombWorker<PixelSrc,PixelDst>(src,dst, a_,b_);
    return 0;
  }
};



void ipp::lincomb(
  const img_data& src, 
  double a, double b,
  img_data& dst)
{
  LincombCaller c(a,b);
  img_access_dispatch2(src,dst, c);
}
  

