#ifndef ip_loop_hpp
#define ip_loop_hpp

#include "../img/img_data.hpp"
#include "../img/access/img_access_guard.hpp"
#include "../img/access/img_access_wrap.hpp"

namespace{

///////////////////////////////////////////////
// loop - with source and destination img

template<class PixelSrc, class PixelDst, class Worker>
void loop2(
  const img_data& img_src, img_data& img_dst, 
  int margin,
  Worker& worker,
  const PixelSrc& , const PixelDst& )
{
  //standard prolog
  typedef img_access_guard<img_access_wrap_const
    <PixelSrc> > AccessorSrc;
  typedef img_access_guard<img_access_wrap
    <PixelDst> > AccessorDst;

  int width =img_src.fmt().width();
  int height=img_src.fmt().height();
  img_dst.reformat(img_fmt(width,height,img_dst.fmt().szPixel()));

  AccessorSrc rowSrc(img_src);
  rowSrc.seek(margin, margin);
  AccessorSrc rowSrcEnd(img_src); 
  rowSrcEnd.seek(height-margin);
  
  AccessorDst rowDst(img_dst);
  rowDst.seek(margin, margin);

  while (rowSrc<rowSrcEnd){
    AccessorDst dst(rowDst);

    AccessorSrc src(rowSrc);
    AccessorSrc srcEnd(rowSrc);
    srcEnd+=width-2*margin;

    while (src<srcEnd){
      worker.process(src, dst);

      ++src;++dst;
    }
    rowSrc.incLine();
    rowDst.incLine();
  }//all rows
}

///////////////////////////////////////////////
// loop - with source and mask img

template<class PixelSrc, class Worker>
void loop1m(
  const img_data& img_src, 
  const img_data& img_mask, 
  int margin,
  Worker& worker,
  const PixelSrc& )
{
  //standard prolog
  typedef img_access_guard<img_access_wrap_const
    <PixelSrc> > AccessorSrc;
  typedef typename Worker::PixelMask PixelMask;
  typedef img_access_guard<img_access_wrap_const
    <PixelMask> > AccessorMask;

  int width =img_src.fmt().width();
  int height=img_src.fmt().height();
  if (
    img_mask.fmt().szPixel()!=1 ||
    width !=img_mask.fmt().width() ||
    height!=img_mask.fmt().height())
      throw "ipp::loop() - "
            "illegal mask image format";

  AccessorSrc rowSrc(img_src);
  rowSrc.seek(margin, margin);
  AccessorSrc rowSrcEnd(img_src); 
  rowSrcEnd.seek(height-margin);
  
  AccessorMask rowMask(img_mask);
  rowMask.seek(margin, margin);

  while (rowSrc<rowSrcEnd){
    AccessorMask colMask(rowMask);

    AccessorSrc src(rowSrc);
    AccessorSrc srcEnd(rowSrc);
    srcEnd+=width-2*margin;

    while (src<srcEnd){
      worker.process(src, colMask);

      ++src;++colMask;
    }
    rowSrc.incLine();
    rowMask.incLine();
  }//all rows
}
///////////////////////////////////////////////
// loop - with source img

template<class PixelSrc, class Worker>
void loop1(
  const img_data& img_src, 
  int margin,
  Worker& worker,
  const PixelSrc& )
{
  //standard prolog
  int width =img_src.fmt().width();
  int height=img_src.fmt().height();
  typedef img_access_guard<img_access_wrap_const
    <PixelSrc> > AccessorSrc;

  AccessorSrc rowSrc(img_src);
  rowSrc.seek(margin, margin);
  AccessorSrc rowSrcEnd(img_src); 
  rowSrcEnd.seek(height-margin);
  
  while (rowSrc<rowSrcEnd){
    AccessorSrc src(rowSrc);
    AccessorSrc srcEnd(rowSrc);
    srcEnd+=width-2*margin;

    while (src<srcEnd){
      worker.process(src);

      ++src;
    }
    rowSrc.incLine();
  }//all rows
}


} //end unnamed namespace

#endif



