#include "ipp_mask.hpp"

#include "../img/img_data.hpp"
#include "../img/access/img_access_dispatch.hpp"
#include "../img/access/img_access_guard.hpp"
#include "../img/access/img_access_wrap.hpp"
#include "../img/access/img_access_pix.hpp"

#include <iostream>

///////////////////////////////////////////////
// do_mask - 
//   masks the bits from img_src
//   according to img_mask and stores
//   the result into img_dst

template<class PixelSrc, class PixelDst>
static void do_mask(
  const img_data& img_src, const img_data& img_mask, 
  int margin, img_data& img_dst,
  const PixelSrc& , const PixelDst& )
{
  //standard prolog
  typedef img_access_guard<img_access_wrap_const<PixelSrc> > AccessorSrc;
  typedef img_access_guard<img_access_wrap<PixelDst> > AccessorDst;
  typedef img_access_guard<img_access_wrap_const<img_pix_char> > AccessorMask;

  int width =img_src.fmt().width();
  int height=img_src.fmt().height();
  if (
    img_mask.fmt().szPixel()!=1 ||
    width !=img_mask.fmt().width() ||
    height!=img_mask.fmt().height())
      throw "ipp::do_mask() - "
            "illegal mask image format";

  //mask algorithm
  AccessorSrc rowSrc(img_src);
  rowSrc.seek(margin, margin);
  AccessorSrc rowSrcEnd(img_src);
  rowSrcEnd.seek(height-margin);

  AccessorDst rowDst(img_dst);
  rowDst.seek(margin, margin);

  AccessorMask rowMask(img_mask);
  rowMask.seek(margin, margin);

  while (rowSrc<rowSrcEnd){

    AccessorSrc src(rowSrc);
    AccessorSrc srcEnd(rowSrc);
    srcEnd+=width-margin-margin;

    AccessorDst dst(rowDst);
    AccessorMask mask(rowMask);

    while (src<srcEnd){
      if (img_pix_isBlack(*mask)){
        img_pix_black(*dst);
      } else{
        *dst=*src;
      }

      ++src;
      ++dst; ++mask;
    }
    rowSrc.incLine();
    rowDst.incLine();

    rowMask.incLine();
  }//all rows
}


///////////////////////////////////////////////
// class MaskWorker

// member template functions defined in-class
// because of the buggy MS compiler

// dummy needed because the buggy MSVC supports 
// only implicit resolution of template functions

// typedefs are used by dispatch template functions

class MaskWorker{
  const img_data& mask_;
  int margin_;
public:
  typedef const img_data& Param1;
  typedef img_data& Param2;
public:
  MaskWorker(const img_data& mask, int m):
    mask_(mask), margin_(m) {}

public:
  template <class PixelSrc, class PixelDst>
  inline int operator() (
    const img_data& src, img_data& dst, 
    const PixelSrc& dummy1, const PixelDst& dummy2)
  {
    do_mask(
      src, mask_, margin_, dst, dummy1, dummy2);
    return margin_;
  }
private:
  const MaskWorker& operator=(const MaskWorker& );
};





int ipp::mask(
  const img_data& src, const img_data& mask, 
  int margin, img_data& dst)
{
  MaskWorker w(mask, margin);
  return img_access_dispatch2Scalars(src,dst, w);
}
  

