#include "ipp_negate.hpp"
#include "ipp_loop.hpp"

#include "../img/access/img_access_dispatch.hpp"
#include "../img/access/img_access_pix.hpp"

#include "../util/util_cxx.hpp"

#include <algorithm>
#include <stdlib.h>


//////////////////////////////////////////////
// the worker classes
// * provide function: process
// * supposed to be called from loop

template <class PixelSrc, class PixelDst>
class NegateWorker{
public:
  typedef img_access_guard<img_access_wrap_const
    <PixelSrc> > AccessorSrc;
  typedef img_access_guard<img_access_wrap
    <PixelDst> > AccessorDst;
  typedef typename PixelSrc::Scalar SrcScalar;
private:
public:
  NegateWorker(){}
  void process( const AccessorSrc& psrc, 
                const AccessorDst& pdst)
  {
    SrcScalar rv=PixelSrc::MaxScalar-psrc->scalar();
    assert(rv>=0);
    pdst->setScalar(rv);
  } 
};

template <class PixelSrc, class PixelDst>
class NegateWithOffsetWorker{
public:
  typedef img_access_guard<img_access_wrap_const
    <PixelSrc> > AccessorSrc;
  typedef img_access_guard<img_access_wrap
    <PixelDst> > AccessorDst;
  typedef typename PixelSrc::Scalar SrcScalar;
private:
  SrcScalar offset_;
public:
  NegateWithOffsetWorker(double l):
    offset_(static_cast<SrcScalar>(l))
  {}
  void process( const AccessorSrc& psrc, 
                const AccessorDst& pdst)
  {
    SrcScalar value=psrc->scalar();
    SrcScalar rv=offset_-value;
    assert(rv>=0);
    pdst->setScalar(rv);
  } 
};





///////////////////////////////////////////////
// classes for static dispatch and 
// marshalling parameters
// * member template functions defined in-class
//   because of the buggy MS compiler
// * dummies needed because the buggy MSVC supports 
//   only implicit resolution of template functions
// * typedefs are used by dispatch template functions

class NegateCaller{
public:
  typedef const img_data& Param1;
  typedef img_data& Param2;
  NegateCaller(){}
  template <class PixelSrc, class PixelDst>
  inline int operator() (
    const img_data& src, img_data& dst, 
    const PixelSrc& dummy1, const PixelDst& dummy2)
  {
    NegateWorker<PixelSrc,PixelDst> w;
    loop2(src, dst, 0, w, dummy1, dummy2);
    return 0;
  }
};

class NegateWithOffsetCaller{
  double limit_;
public:
  typedef const img_data& Param1;
  typedef img_data& Param2;
public:
  NegateWithOffsetCaller(double l):limit_(l){}
  template <class PixelSrc, class PixelDst>
  inline int operator() (
    const img_data& src, img_data& dst, 
    const PixelSrc& dummy1, const PixelDst& dummy2)
  {
    NegateWithOffsetWorker<PixelSrc,PixelDst> w(limit_);
    loop2(src, dst, 0, w, dummy1, dummy2);
    return 0;
  }
};


/////////////////////////////
// entry functions
void ipp::negate(
  const img_data& src, 
  img_data& dst)
{
  NegateCaller c;
  img_access_dispatch2Scalars(src,dst, c);
}

void ipp::negate(
  const img_data& src, double limit,
  img_data& dst)
{
  NegateWithOffsetCaller c(limit);
  img_access_dispatch2Scalars(src,dst, c);
}

