#include "ipp_projection.hpp"

#include "../img/img_data.hpp"
#include "../img/access/img_access_dispatch.hpp"
#include "../img/access/img_access_guard.hpp"
#include "../img/access/img_access_wrap.hpp"
#include "../img/access/img_access_pix.hpp"
#include "../img/access/img_access_comb.hpp"

#include <vector>



///////////////////////////////////////////////
// class projection hor worker

// member template functions defined in-class
// because of the buggy MS compiler


class ProjectionHorWorker{
  std::vector<int>& proj_;

public:
  //these typedefs are used  by 
  //img_access_dispatch()
  typedef const img_data& Param1;
public:
  ProjectionHorWorker(std::vector<int>& p):proj_(p){}

public:
  template <class PixelSrc>
  inline int operator() (
    const img_data& img_src, 
    const PixelSrc& )
  {
    typedef img_access_guard<img_access_wrap_const<PixelSrc> > AccessorSrc;
    typedef img_access_guard<img_access_wrap<img_pix_char> > AccessorDst;

    //check dimensions
    int width =img_src.fmt().width();
    int height=img_src.fmt().height();

    proj_.assign(height,0);

    AccessorSrc rowSrc(img_src);
    AccessorSrc rowSrcEnd(img_src); 
    rowSrcEnd.seek(height);

    int row=0;
    while (rowSrc<rowSrcEnd){
      AccessorSrc src(rowSrc);
      AccessorSrc srcEnd(rowSrc);
      srcEnd+=width;
      while (src<srcEnd){
        proj_[row]+=src->uint();
        ++src;
      }
      row++;
      rowSrc.incLine();
    }//all rows
    return 0;
  }
private:
  const ProjectionHorWorker& operator=(const ProjectionHorWorker& );
};



///////////////////////////////////////////////
// class projection ver worker

// member template functions defined in-class
// because of the buggy MS compiler

class ProjectionVerWorker{
  std::vector<int>& proj_;

public:
  //these typedefs are used  by 
  //img_access_dispatch()
  typedef const img_data& Param1;
public:
  ProjectionVerWorker(std::vector<int>& p):proj_(p){}

public:
  template <class PixelSrc>
  inline int operator() (
    const img_data& img_src, 
    const PixelSrc& )
  {
    typedef img_access_guard<img_access_wrap_const<PixelSrc> > AccessorSrc;
    typedef img_access_guard<img_access_wrap<img_pix_char> > AccessorDst;

    //check dimensions
    int width =img_src.fmt().width();
    int height=img_src.fmt().height();

    //find projection and store it into std::vector
    proj_.assign(width,0);

    AccessorSrc rowSrc(img_src);
    AccessorSrc rowSrcEnd(img_src); 
    rowSrcEnd.seek(height);

    while (rowSrc<rowSrcEnd){
      AccessorSrc src(rowSrc);
      AccessorSrc srcEnd(rowSrc);
      srcEnd+=width;
      int col=0;
      while (src<srcEnd){
        proj_[col]+=src->uint();
        col++;
        ++src;
      }
      rowSrc.incLine();
    }//all rows
    return 0;
  }
private:
  const ProjectionVerWorker& operator=(const ProjectionVerWorker& );
};





void ipp::projection_hor(const img_data& src, std::vector<int>& proj){
  ProjectionHorWorker w(proj);
  img_access_dispatch1(src,w);
}
void ipp::projection_ver(const img_data& src, std::vector<int>& proj){
  ProjectionVerWorker w(proj);
  img_access_dispatch1(src,w);
}

