#include "ipp_radial.hpp"
#include "ipp_transform.hpp"

#include "../img/access/img_access_dispatch.hpp"


class TransformRadial{
  double alpha_;
  double x0_, y0_;
  double k1_,k2_;
public:
  TransformRadial(double a,
    double x0,double y0,
    double k1,double k2)
  :
    alpha_(a),
    x0_(x0), y0_(y0),
    k1_(k1),k2_(k2)
  {}

  inline bool operator()(
    int idst, int jdst, 
    double& isrc, double& jsrc)
  {
    double y=(idst-y0_)/alpha_;
    double x=(jdst-x0_)/alpha_;
        
    double r2=x*x+y*y;
    double f=1+k1_*r2+k2_*r2*r2;

    double x_rd=x*f;
    double y_rd=y*f;
        
    jsrc=alpha_*x_rd+x0_;
    isrc=alpha_*y_rd+y0_;

    return true;
  }
};


/////////////////////////////
// entry functions
void ipp::radial(
  const img_data& src, 
  double alpha,
  double u0, double v0,
  double k1, double k2,
  img_data& dst)
{
  double x0=u0+src.fmt().width()/2;
  double y0=v0+src.fmt().height()/2;

  TransformRadial tr(alpha, x0,y0, k1,k2);
  TransformLoop<TransformRadial> tl(tr);
  dst.reformat(src.fmt());

  img_access_dispatch2Scalars(src,dst, tl);
}

