#include "ipp_range.hpp"
#include "ipp_loop.hpp"

#include "../img/img_data.hpp"
#include "../img/access/img_access_dispatch.hpp"
#include "../img/access/img_access_guard.hpp"
#include "../img/access/img_access_wrap.hpp"
#include "../img/access/img_access_pix.hpp"
#include "../img/access/img_access_comb.hpp"


#include "../util/util_cxx.hpp"

#include <vector>
#include <iostream>




//////////////////////////////////////////////
// the worker classes (see ipp_lopp.hpp fo more info)

template <class PixelSrc>
class RangeWorker{
public:
  typedef img_access_guard<img_access_wrap_const<PixelSrc> > AccessorSrc;
  typedef typename PixelSrc::Scalar Scalar;
public:
  RangeWorker():
    minval_(PixelSrc::MaxScalar), maxval_(0)
  {}
  void process(const AccessorSrc& psrc){
    minval_=std::min(psrc->red(), minval_);
    minval_=std::min(psrc->green(), minval_);
    minval_=std::min(psrc->blue(), minval_);

    maxval_=std::max(psrc->red(), maxval_);
    maxval_=std::max(psrc->green(), maxval_);
    maxval_=std::max(psrc->green(), maxval_);
  }
  Scalar minval(){return minval_;}
  Scalar maxval(){return maxval_;}
private:
  Scalar minval_;
  Scalar maxval_;
};


template <class PixelSrc>
class RangeWorkerGray{
public:
  typedef img_access_guard<img_access_wrap_const<PixelSrc> > AccessorSrc;
  typedef typename PixelSrc::Scalar Scalar;
public:
  RangeWorkerGray():
    minval_(PixelSrc::MaxScalar), maxval_(0)
  {}
  void process(const AccessorSrc& psrc){
    if (psrc->scalar()<minval_)
      minval_=psrc->scalar();
    if (psrc->scalar()>maxval_)
      maxval_=psrc->scalar();
  }
  Scalar minval(){return minval_;}
  Scalar maxval(){return maxval_;}
private:
  Scalar minval_;
  Scalar maxval_;
};

template <>
class RangeWorker<img_pix_char>:
  public RangeWorkerGray<img_pix_char>
{};
template <>
class RangeWorker<img_pix_fixed16>:
  public RangeWorkerGray<img_pix_fixed16>
{};



///////////////////////////////////////////////
// classes for marshalling parameters
// * typedefs are used by dispatch template functions

class RangeCaller{
  double& minval_;
  double& maxval_;
public:
  typedef const img_data& Param1;
  typedef img_data& Param2;

  RangeCaller(double& minv, double& maxv):
    minval_(minv), maxval_(maxv) {}

public:
  template <class PixelSrc>
  inline int operator() (
    const img_data& src, 
    const PixelSrc& dummy1)
  {
    RangeWorker<PixelSrc> w;
    loop1(src, 0, w, dummy1);
    minval_=w.minval();
    maxval_=w.maxval();
    return 0;
  }
};


////////////////////////////////
// entry functions

void ipp::range(
  const img_data& src, 
  double& minval, 
  double& maxval)
{
  RangeCaller c(minval, maxval);
  img_access_dispatch1(src,c);
}

