#include "ipp_stretch.hpp"

#include "../img/img_data.hpp"
#include "../img/access/img_access_dispatch.hpp"
#include "../img/access/img_access_guard.hpp"
#include "../img/access/img_access_wrap.hpp"
#include "../img/access/img_access_pix.hpp"
#include "../img/access/img_access_comb.hpp"

#include <stdexcept>




template<class PixelSrc,class PixelDst>
void StretchWorker(
  const img_data& img_src, 
  img_data& img_dst, 
  int size)
{
  //standard prolog
  typedef img_access_guard<img_access_wrap_const<PixelSrc> > AccessorSrc;
  typedef img_access_guard<img_access_wrap<PixelDst> > AccessorDst;

  int w=img_src.fmt().width();
  int h=img_src.fmt().height();
  if (w*size !=img_dst.fmt().width() ||
      h*size !=img_dst.fmt().height())
    img_dst.reformat(img_fmt(w*size,h*size, sizeof(PixelDst)));

  AccessorSrc itSrcRow(img_src);
  for (int row=0; row<h; ++row){
    AccessorSrc itSrc(itSrcRow);
    for (int col=0; col<w; ++col){
      AccessorDst itDstRow(img_dst);
      itDstRow.seek(row*size);
      for (int i=0; i<size; ++i){
        AccessorDst itDst(itDstRow);
        itDst+=(col*size);
        for (int j=0; j<size; ++j){
          *itDst=*itSrc;
          ++itDst;
        }
        itDstRow.incLine();
      }
      ++itSrc;
    }
    itSrcRow.incLine();
  }
}






///////////////////////////////////////////////
// class StretchCaller
// typedefs are used by dispatch template functions

class StretchCaller{
  int dimension_;
  img_data& dst_;
public:
  typedef const img_data& Param1;
  typedef img_data& Param2;
public:
  StretchCaller(int d, img_data& dst):
    dimension_(d), dst_(dst) {}
public:
  template <class Pixel>
  inline int operator() (const img_data& src, const Pixel&)
  {
    switch (sizeof(Pixel)){
    case 3:
    case 6:
      StretchWorker<Pixel, img_pix_3char>(src, dst_, dimension_);
      break;
    default:
      StretchWorker<Pixel, img_pix_char>(src, dst_, dimension_);
    }
    return 0;
  }
};



void ipp::stretch(
  const img_data& src, 
  int dimension,
  img_data& dst)
{
  StretchCaller ssc(dimension,dst);
  img_access_dispatch1(src,ssc);
}
  

