#ifndef ipp_transform_hpp
#define ipp_transform_hpp

#include "../img/img_data.hpp"
#include "../img/access/img_access_guard.hpp"
#include "../img/access/img_access_wrap.hpp"
#include "../img/access/img_access_pix.hpp"
#include "../img/access/img_access_comb.hpp"

#include <iostream>

#include <math.h>



// supposed to be given as the argument to 
// img_access_dispatch2(src,dst, tl);

// the template parameter must provide a function:
//   inline bool operator()(
//     int idst, int jdst, 
//     double& isrc, double& jsrc);
// which for given destination image coordinates
// determines the corresponding source image point
// and returns false if such a point does not exist

namespace ipp{

template <class T>
class TransformLoop{
  T& t_;  
public:
  typedef const img_data& Param1;
  typedef img_data& Param2;
public:
  TransformLoop(T& t):t_(t){}
  template <class PixelSrc, class PixelDst>
  inline int operator() (
    const img_data& imgsrc, img_data& imgdst, 
    const PixelSrc& , const PixelDst& )
  {
    // standard prolog
    typedef img_access_guard<img_access_wrap_const
      <PixelSrc> > AccessorSrc;
    typedef img_access_guard<img_access_wrap
      <PixelDst> > AccessorDst;
    int width =imgsrc.fmt().width();
    int height=imgsrc.fmt().height();
    if (
        width !=imgdst.fmt().width() ||
        height!=imgdst.fmt().height())
      throw "ipp::TransformLoop::operator()"
            " - got images of different sizes";
    const int margin=0;

    typename PixelSrc::Combination comb;
    AccessorSrc src(imgsrc);

    AccessorDst rowDst(imgdst);
    rowDst.seek(margin, margin);
    AccessorDst rowDstEnd(imgdst); 
    rowDstEnd.seek(height-margin);
    int i=0;

    while (rowDst<rowDstEnd){
      AccessorDst dst(rowDst);
      AccessorDst dstEnd(rowDst);
      dstEnd+=width-2*margin;
      int j=0;

      while (dst<dstEnd){

        double jsrc, isrc;
        bool pixelOk=t_(i,j, isrc,jsrc);
        
        int jsrc0=static_cast<int>(jsrc);
        int isrc0=static_cast<int>(isrc);
        
        if (pixelOk &&
            isrc0  >=0     && jsrc0   >=0 &&
            isrc0+1< height && jsrc0+1< width)
        {
          // std::cerr 
          //   <<isrc0 <<", " <<jsrc0 <<"  -> " 
          //   <<i    <<", " <<j <<"\n";
            
          double kj=1-(jsrc-jsrc0);
          double ki=1-(isrc-isrc0);


          comb.reset();
          src.seek(isrc0,jsrc0);
          comb.add(*src, ki*kj);
          ++src;
          comb.add(*src, ki*(1-kj));        
          src.incLine();
          comb.add(*src, (1-ki)*(1-kj));
          --src;
          comb.add(*src, (1-ki)*kj);
          
          comb.put(*dst);
        } else{
          img_pix_black(*dst);
        }

        ++dst;
        ++j;
      }
      rowDst.incLine();
      ++i;
    }//all rows
    
    return 0;
  }
};

}

#endif
