#include "ipp_vector.hpp"

#include "../img/img_data.hpp"
#include "../img/access/img_access_dispatch.hpp"
#include "../img/access/img_access_guard.hpp"
#include "../img/access/img_access_wrap.hpp"
#include "../img/access/img_access_pix.hpp"
#include "../img/access/img_access_comb.hpp"

#include <algorithm>
#include <stdexcept>



///////////////////////////////////////////////
// class DrawVectorHor worker

// member template functions defined in-class
// because of the buggy MS compiler

class DrawVectorHorWorker{
  const std::vector<int>& proj_;
  const DrawVectorHorWorker& operator=(const DrawVectorHorWorker&);
public:
  //these typedefs are used  by 
  //img_access_dispatch()
  typedef img_data& Param1;
public:
  DrawVectorHorWorker(const std::vector<int>& p):proj_(p){}

public:
  template <class PixelDst>
  inline int operator() (
    img_data& img_dst, 
    const PixelDst& )
  {
    typedef img_access_guard<img_access_wrap<PixelDst> > AccessorDst;

    //check dimensions
    int width =static_cast<int>(proj_.size());
    int height=img_dst.fmt().height();

    if (width !=img_dst.fmt().width())
      throw std::runtime_error("DrawVectorHorWorker::operator() "
        "- got image of a wrong size");

    //normalize proj_
    int max=*std::max_element(proj_.begin(), proj_.end());
    double factor=height; factor/=max;

    //put proj_ into destination
    AccessorDst rowDst(img_dst);
    for (int row=0; row<height; row++){
      AccessorDst dst(rowDst);
      for (int col=0; col<width; col++){
        if (row<factor*proj_[col]+0.5)
          img_pix_black(*dst);
        else
          img_pix_white(*dst);
        ++dst;
      }//all columns
      rowDst.incLine();
    }//all rows
    return 0;
  }
};

///////////////////////////////////////////////
// class DrawVectorVer worker

class DrawVectorVerWorker{
  const std::vector<int>& proj_;
  const DrawVectorVerWorker& operator=(const DrawVectorVerWorker&);

public:
  //these typedefs are used  by 
  //img_access_dispatch()
  typedef img_data& Param1;
public:
  DrawVectorVerWorker(const std::vector<int>& p):proj_(p){}

public:
  template <class PixelDst>
  inline int operator() (
    img_data& img_dst, 
    const PixelDst& )
  {
    typedef img_access_guard<img_access_wrap<PixelDst> > AccessorDst;

    //check dimensions
    int height=static_cast<int>(proj_.size());
    int width =img_dst.fmt().width();

    if (height !=img_dst.fmt().height())
      throw std::runtime_error("DrawVectorVerWorker::operator() "
        "- got image of a wrong size");

    //normalize proj_
    int max=*std::max_element(proj_.begin(), proj_.end());
    double factor=width; factor/=max;

    //put proj_ into destination
    AccessorDst rowDst(img_dst);
    for (int row=0; row<height; row++){
      AccessorDst dst(rowDst);
      int threshold=static_cast<int>(factor*proj_[row]+0.5);
      for (int col=0; col<width; col++){
        if (col<threshold)
          img_pix_black(*dst);
        else
          img_pix_white(*dst);
        ++dst;
      }//all columns
      rowDst.incLine();
    }//all rows
    return 0;
  }
};





void ipp::drawVectorHor(const std::vector<int>& p, img_data& dst){
  DrawVectorHorWorker w(p);
  img_access_dispatch1(dst,w);
}
void ipp::drawVectorVer(const std::vector<int>& p, img_data& dst){
  DrawVectorVerWorker w(p);
  img_access_dispatch1(dst,w);
}

