#include "thr_base.hpp"

#include <iostream>

////////////////////////////
// Threads

void* thr::ThreadEntryFunction(void* p){
  int rv=-1;
  try{
    ThreadSimpleWorker& w= *
      static_cast<ThreadSimpleWorker*>(p);
    rv=w();
    w.done_=true;
  } catch(char const* msg){
     std::cerr <<"thr::ThreadEntryFunction(" <<p 
       <<"): got exception: " <<msg <<std::endl;
  } catch(std::exception& e){
     std::cerr <<"thr::ThreadEntryFunction(" <<p 
       <<"): got exception: " <<e.what() <<std::endl;
  } catch (...){
     std::cerr <<"thr::ThreadEntryFunction(" <<p 
       <<"): got unknown exception." <<std::endl;
  }
  return reinterpret_cast<void*>(((char*)0)+rv);
}

// simple thread
thr::ThreadSimple::ThreadSimple(
  const boost::shared_ptr<ThreadSimpleWorker>& pw)
:
  pw_(pw), 
  pth_(
    threadCreate(&ThreadEntryFunction, pw_.get(), Joinable))
{}
thr::ThreadSimple::~ThreadSimple(){
  threadDetach(pth_);
  threadRelease(pth_);
}
void thr::ThreadSimple::join(){
  threadJoin(pth_);
}

// simple thread worker
thr::ThreadSimpleWorker::ThreadSimpleWorker():
  done_(false)
{}
thr::ThreadSimpleWorker::~ThreadSimpleWorker(){}

// synchronized thread
thr::ThreadSync::ThreadSync(
  const boost::shared_ptr<ThreadSyncWorker>& pw)
: 
  ThreadSimple(pw)
{}
thr::ThreadSync::~ThreadSync(){
  // we are quite sure that pw_ is kind of ThreadSyncWorker
  // since that was the type of the constructor argument
  ThreadSyncWorker& tsw=
    static_cast<ThreadSyncWorker&>(worker());
  tsw.cleanup();
  join();
}
 
// deferred synchronized thread 
thr::ThreadDeferred::ThreadDeferred(
  const boost::shared_ptr<ThreadSyncWorker>& pw)
: 
  ThreadSimple(pw)
{}
thr::ThreadDeferred::~ThreadDeferred(){
  // we are quite sure that pw_ is kind of ThreadSyncWorker
  // since that was the type of the constructor argument
  ThreadSyncWorker& tsw=
    static_cast<ThreadSyncWorker&>(worker());
  tsw.cleanup();
}

// synchronized thread worker

thr::ThreadSyncWorker::~ThreadSyncWorker(){}

////////////////////////////
// Mutex

thr::Mutex::Mutex():
  pimp(mutexCreate()),
  closed(false)
{}
  
thr::Mutex::~Mutex(){
  if (closed){
    leave();
  }
  mutexDestroy(pimp);
}

void thr::Mutex::enter(){
  mutexEnter(pimp);
  closed=true;
}
void thr::Mutex::leave(){
  closed=false;
  mutexLeave(pimp);
}

////////////////////////////
// Lock

thr::Lock::Lock(Mutex& m):
  mtx_(m)
{
  mtx_.enter();
}
  
thr::Lock::~Lock(){
  mtx_.leave();
}



////////////////////////////
// CondVar

thr::CondVar::CondVar():
  pcv(condVarCreate())
{
}
thr::CondVar::~CondVar(){
  condVarDestroy(pcv);
}


void thr::CondVar::wait(Mutex& m){
  condVarWait(pcv, m.pimp);
}
bool thr::CondVar::timedWait(Mutex& m, int t_ms){
  return condVarTimedWait(pcv, m.pimp, t_ms);
}

void thr::CondVar::signal(){
  condVarSignal(pcv);
}
void thr::CondVar::broadcast(){
  condVarBroadcast(pcv);
}
