#ifndef thr_primitives_hpp
#define thr_primitives_hpp

/******************************************************
*
** thr_primitives.hpp bu Sinisa Segvic
*
** encapsulation of basic OS dependent
** multithreading calls 
*
** (threads, mutexes, conditions)
* 
*******************************************************/


namespace thr{
  ///////////////////////////
  // 0. general
  //puts the thread to sleep,
  //argument is in milliseconds
  void sleep(int t_ms);

  ///////////////////////////
  // 1. threads
  struct ThreadHandle;
  typedef void* (*PtrFunThread)(void*);
  //after the thread exits, threadRelease()
  //should be called with the thread identifier
  enum ThreadCreationParam{Detached, Joinable};
  ThreadHandle* threadCreate(
    PtrFunThread pf, 
    void* param, 
    ThreadCreationParam tcp);
  void threadRelease(ThreadHandle* );
  //exits the current thread
  void threadExit(int value);
  //waits for the termination of the specified thread
  void threadJoin(ThreadHandle* other);
  //marks the the specified thread as detached
  void threadDetach(ThreadHandle* other);

  ///////////////////////////
  // 2. mutexes
  struct MutexHandle;
  MutexHandle* mutexCreate();
  void mutexDestroy(MutexHandle*);
  void mutexEnter(MutexHandle*);
  void mutexLeave(MutexHandle*);

  ///////////////////////////
  // 3. condition variables
  struct CondVarHandle;

  //the initial state is "closed";
  CondVarHandle* condVarCreate();
  void condVarDestroy(CondVarHandle*);
  //on enter, m must be owned by the caller;
  //on return, the caller again owns m.
  void condVarWait(CondVarHandle*,MutexHandle*m);
  //returns false on time out
  bool condVarTimedWait(CondVarHandle*,MutexHandle*m, int t_ms);

  //the operation should be guarded by the
  //mutex given to condvarWait();
  //only one thread is released!
  void condVarSignal(CondVarHandle*);
  //all waiting threads are released!
  void condVarBroadcast(CondVarHandle*);
}

#endif
