#ifndef ui_hpp
#define ui_hpp

/******************************************************
*
**  ui by Sinisa Segvic:
*
** textual user interface for the cvsh application
** see ui_report::ui_usage for help on available commands
*
** destination window commands:
**   <Esc>          ... stop processing, return to prompt
**   <Ctrl+Shift> S ... toggle recording
*
*******************************************************/

#include <boost/smart_ptr.hpp>

#include <memory>
#include <vector>

class cli_wrap;
class vs_base;
class alg_base;
class vd_base;

class ui_imp;

class ui{
public:
  //construction
  ui(
    cli_wrap& cli,
    std::vector<boost::shared_ptr<vs_base> >& vecSpVs,
    boost::shared_ptr<alg_base>& spWorker,
    std::vector<boost::shared_ptr<vd_base> >& vecSpVd,
    int minDelayMs);
  ~ui();

public:
  //operations
  void loop(char const*);
  void batch(int repetitions, int offset);
  void config(char const*);

private:
  boost::scoped_ptr<ui_imp> pimp;
private:
  const ui& operator=(const ui& );
};
#endif
