#include "ui_cmd.hpp"

#include "../cli/cli_wrap.hpp"

#include "../util/util_string.hpp"

#include <iostream>
#include <sstream>
#include <string>
#include <stdexcept>

#include <assert.h>



//////////////////
// local functions

namespace{

//returns the offset of the next substring
//or 0 if the last substring is extracted
  
int getNext(
  const std::string& str, 
  std::string& substr, 
  int pos)
{
  // std::cout <<"in getNext for " <<str <<" and " <<pos <<std::endl;

  bool atEnd=false;
  std::string::size_type posEnd;

  if (str[pos]=='"'){
    pos++;
    posEnd=str.find('"', pos);
    if (posEnd==std::string::npos){
      throw std::runtime_error("ui::getNext - no closing apostroph");
    } 
    substr=str.substr(pos, posEnd-pos);
    posEnd++;
  } else{
    posEnd=str.find(' ', pos);
    if (posEnd==std::string::npos){
      substr=str.substr(pos, str.length()-pos);
      atEnd=true;
    }
    substr=str.substr(pos, posEnd-pos);
  } 

  std::string::size_type nextpos=str.find_first_not_of(' ', posEnd);
  if (atEnd){
    nextpos=0;
  } else{
    nextpos=str.find_first_not_of(' ', posEnd);
    if (nextpos==std::string::npos)
      nextpos=0;
  }

  // std::cout <<"getNext produced " <<substr 
  //           <<" and " <<nextpos <<std::endl;

  return static_cast<int>(nextpos);
}

} //unnamed namespace end

//////////////////
// members' definitions

ui_cmd::ui_cmd():
  action_(sentinelAction), 
  location_(sentinelLocation),
  param_(1)
{}

ui_cmd::ui_cmd(
  Action a,
  Location l,
  int p)
:
  action_(a), 
  location_(l),
  param_(p)
{}

ui_cmd::ui_cmd(char const* command):
  action_(Show), 
  location_(Addr),
  param_(0)
  //location_(Next),param_(1)
  //location_(Addr), param_(0)  
{
  if (*command!=0){
    std::string s(command);
    get(s);
  }
}

void ui_cmd::get(cli_wrap& cli){
  static ui_cmd last(Process, Next, 1);

  std::string line=cli.get();

  if (cli.eof()){
    action_=Quit;
  } else{
    //std::ostringstream oss;
    //oss <<"Got Line: " <<line <<std::endl;
    //cli.msg(oss.str.c_str());

    if (!line.empty()){
      get(line);
      last=*this;
    } else{
      *this=last;
    }
  }
}


void ui_cmd::get(const std::string& line){

  int pos=0;
  std::string str;

  //extract action
  action_=sentinelAction;
  pos=getNext(line, str, pos);
  for (int i=0; i<countAction; i++){
    Action a=enumAction(i);
    if (std::string(strAction(a)).find(str)==0){
      action_=a;
      break;
    }
  }
  if (action_==sentinelAction)
    throw std::runtime_error("ui_cmd::get - unrecognized action");

  if (pos){
    if (action_==Help || action_==Quit)
      throw std::runtime_error("ui_cmd::get - too many args");

    pos=getNext(line, str, pos);

    switch (argType(action_)){
    case None:
      throw std::runtime_error("ui_cmd::get - too many args");
    case String:
    case String_Param:
      str_=str;
      break;
    case Location_Param:
    {
      //extract location
      location_=sentinelLocation;
      for (int i=0; i<countLocation; i++){
        Location l=enumLocation(i);
        if (std::string(strLocation(l)).find(str)==0){
          location_=l;
          break;
        }
      }
      if (location_==sentinelLocation)
        throw std::runtime_error("ui_cmd::get - unrecognized location");
      break;
    }
    default:
      assert(0);
    }

    if (pos){
      pos=getNext(line, str, pos);
      std::istringstream iss(str);
      switch (argType(action_)){
      case None:
      case String:
        throw std::runtime_error("ui_cmd::get - too many args");
      case String_Param:
      case Location_Param:
        iss >>param_;
        if (param_<-1)
          throw std::runtime_error("ui_cmd::get - got illegal parameter");
        if (pos)
          throw std::runtime_error("ui_cmd::get - too many args");
        break;
      default:
        assert(0);
      }
    } else{
      if (action_==Source || action_==Destination){
        // index of the source/destination 
        // which is to be changed
        param_=0;
      } else{
        param_=1;
      }
    }
  } else{
    location_=Next;
    param_=1;
    str_.erase();
  }
}



void ui_cmd::prompt(
  std::ostream& os,
  char const* name, int pos, int count)
{
  std::string str;
  util_string::retractPath(name, str);
  
  os <<str <<'[' <<pos <<'/';
  if (count==INT_MAX)
    os <<"inf";
  else
    os <<count;
  os <<"]$ ";
}

char const* ui_cmd::strAction(Action a){
  static const std::string cmdStrAct[]={
    "show", "process", 
    "config", "source", "destination", "algorithm",
    "help", "quit"
  };
  return cmdStrAct[a].c_str();
}
char const* ui_cmd::strLocation(Location l){
  static const std::string cmdStrLoc[]={
    "next", "previous", "this", "address"
  };
  return cmdStrLoc[l].c_str();
}
ui_cmd::ArgType ui_cmd::argType(int a){
  static const ArgType cmdArgType[]={
    Location_Param, Location_Param, 
    String, String_Param, String_Param, String,
    None, None};
  return cmdArgType[enumAction(a)];
}


ui_cmd::Action ui_cmd::enumAction(int a){
  if (a<=sentinelAction || a>=countAction)
    throw std::runtime_error("ui_cmd::enumAction - illegal index");
  return static_cast<Action>(a);
}
ui_cmd::Location ui_cmd::enumLocation(int l){
  if (l<=sentinelLocation|| l>=countLocation)
    throw std::runtime_error("ui_cmd::enumLocation - illegal index");
  return static_cast<Location>(l);
}



////////////////////////
// friends' definitions

std::istream& operator>>(std::istream& is, ui_cmd& c){
  std::ostringstream oss;
  cli_wrap cli(is, oss, oss);
  c.get(cli);
  return is;
}
std::ostream& operator<<(std::ostream& os, const ui_cmd& c){
  os <<ui_cmd::strAction(c.action());
  switch (c.argType(c.action())){
  case ui_cmd::None:
    break;
  case ui_cmd::String:
    os <<" " <<c.str();
    break;
  case ui_cmd::String_Param:
    os <<" " <<c.str()
       <<" " <<c.param();
    break;
  case ui_cmd::Location_Param:
    os <<" " <<ui_cmd::strLocation(c.location())  
       <<" " <<c.param();
    break;
  default:
    assert(0);
  }

  return os;
}
