#ifndef ui_cmd_hpp
#define ui_cmd_hpp

#include <iosfwd>
#include <string>

class cli_wrap;

/******************************************************
*
** ui_cmd by Sinisa Segvic:
*
** holds the data which represent the user command
*
** the representable command set:
*
** Command --> [ActionLoc [Location]]
** Command --> [ActionStr [String]]
** Command --> [ActionSimple]
*
** ActionLoc    --> process | show
** ActionStr    --> config
** ActionStr    --> source
** ActionStr    --> destination
** ActionStr    --> algorithm
** ActionSimple --> help | quit
*
** Location   --> next [#howMany]     | 
**                previous [#howMany] | 
**                this [#howMany]     | 
**                address #frame
*
*******************************************************/


class ui_cmd{
  friend std::istream& operator>>(std::istream& is, ui_cmd& c);
  friend std::ostream& operator<<(std::ostream& os, const ui_cmd& c);

public:
  enum Action {sentinelAction=-1, 
    Show, Process, 
    Config, Source, Destination, Algorithm, 
    Help, Quit, countAction};
  enum ArgType {sentinelArgType=-1, 
    Location_Param, String, String_Param, None, countArgType};
  enum Location {sentinelLocation=-1, 
    Next, Prev, This, Addr, countLocation};

public:
  ui_cmd();
  ui_cmd(char const*);
private:
  ui_cmd(Action a, Location l, int p);

public:
  static void prompt(std::ostream&, char const* name, int pos, int count);
  void get(cli_wrap&);
private:
  void get(const std::string&);

public:
  Action action() const {return action_;}
  Location location() const {return location_;}
  int param() const {return param_;}
  char const* str() const {return str_.c_str();}

private:
  Action action_;
  Location location_;
  int param_;
  std::string str_;

  static char const* strAction(Action action);
  static char const* strLocation(Location location);
  static Action enumAction(int action);
  static Location enumLocation(int location);
  
  static ArgType argType(int Action);
};

std::istream& operator>>(std::istream& is, ui_cmd& c);
std::ostream& operator<<(std::ostream& os, const ui_cmd& c);

#endif
