#include "ui_report.hpp"

#include "../util/util_registry.hpp"

#include "../alg/alg_base.hpp"


#include <iostream>
#include <time.h>

////////////////////////////////////////////////
// ui_report implementation 

ui_report::ui_report(){}
ui_report::~ui_report(){}

void ui_report::push_back(
  char const* str, int tms)
{
  vec_.push_back(
    Entry(std::string(str), tms));
}

char const* ui_report::
description(int i) const
{
  return vec_[i].first.c_str();
}

int ui_report::
duration(int i) const
{
  return vec_[i].second;
}

int ui_report::size() const{
  return static_cast<int>(vec_.size());
}


////////////////////////////////////////////////
// implementation of exported functions

void ui_profile(
  std::ostream& os,
  char const* description,
  const ui_reportAbstract& pd,
  long frequency)
{
  os.setf(std::ios::fixed, std::ios::floatfield);
  os.precision(1);

  os <<description <<"\n";
  for (int i=0; i<pd.size(); i++){
    double tms=pd.duration(i)*1000.0/frequency;
    os <<"  " <<pd.description(i) 
              <<"=" <<tms <<"ms\n";
  }
}


//processing report for a batch job
void ui_reportBatch(
  std::ostream& os,
  int nFrames, int nClocks, 
  long frequency)
{
  double dt=nClocks;dt/=frequency;
  double fps=dt?nFrames/dt:0;

  os.setf(std::ios::fixed, std::ios::floatfield);
  os.precision(1);
  os <<"Processed " <<nFrames <<" frames in "
     <<dt <<" s" <<std::endl;
  os <<"Throughput: " <<fps <<" fps." <<std::endl;
}


void ui_usage(
  std::ostream& os)
{
  os <<" Summary of commands:\n";
  os <<" Command --> [ActionLoc [Location]] \n";
  os <<" Command --> [ActionStr [String]] \n";
  os <<" Command --> [ActionSimple] \n";
  os <<"\n";

  os <<" ActionLoc    --> process | show \n";
  os <<" ActionStr    --> config \n";
  os <<" ActionStr    --> source \n";
  os <<" ActionStr    --> algorithm \n";
  os <<" ActionSimple --> help | quit \n";
  os <<"\n";

  os <<" Algorithm   -->\n";
  os <<"   a plug-in library or one of the following:\n";
  util::Registry& reg=util::Registry::instance();
  util::Registry::Iterator it=reg.begin<alg_base>();
  while (it != reg.end<alg_base>()){
    os <<"     " <<it->name_derived() <<"\n";
    ++it;
  }
  os <<"\n";

  os <<" Location   --> next [#howMany]     |  \n";
  os <<"                previous [#howMany] |  \n";
  os <<"                this [#howMany]     |  \n";
  os <<"                address #frame         \n";
  os <<"\n";
}
