#ifndef util_bmp_h 
#define util_bmp_hpp

#include "util_endian.hpp"

/////////////////////////////////////////////////////////////
//  util_bmp_RgbQuad - 
//     wrapper for RGBQUAD,
//     layout from wingdi.h

struct util_bmp_RgbQuad{
  util_endian::ONE_BYTE    rgbBlue;
  util_endian::ONE_BYTE    rgbGreen;
  util_endian::ONE_BYTE    rgbRed;
  util_endian::ONE_BYTE    rgbReserved;
};

union util_bmp_RgbUnion{
  util_bmp_RgbQuad s;
  util_endian::ONE_BYTE a[4];
};




/////////////////////////////////////////////////////////////
//  util_bmp_BIH - 
//     wrapper for BitmapInfoHeader,
//     layout from wingdi.h

class util_bmp_BIH{

  enum compression{BI_RGB_, BI_RLE8_, BI_RLE4_, BI_BITFIELDS_};

  util_endian::ONE_BYTE      biSize[4];
  util_endian::ONE_BYTE      biWidth[4];
  util_endian::ONE_BYTE      biHeight[4];
  util_endian::ONE_BYTE      biPlanes[2];
  util_endian::ONE_BYTE      biBitCount[2];
  util_endian::ONE_BYTE      biCompression[4];
  util_endian::ONE_BYTE      biSizeImage[4];
  util_endian::ONE_BYTE      biXPelsPerMeter[4];
  util_endian::ONE_BYTE      biYPelsPerMeter[4];
  util_endian::ONE_BYTE      biClrUsed[4];
  util_endian::ONE_BYTE      biClrImportant[4];
public:
  util_bmp_BIH();

  void init(int bpp, int w, int h, int countEntries);

  void check();
  int bpp();
  int w();
  int h();

public:
  static int sizeofBits(int bpp, int w, int h);
  static int calcSzScanline(int bytes);
};





/////////////////////////////////////////////////////////////
//  util_bmp_BI
//     wrapper for BitmapInfo,
//     layout from wingdi.h

class util_bmp_BI{
  util_bmp_BIH bmiHeader;
  util_bmp_RgbUnion bmiColors[256];//max supported colours = 256
  int nColours_;                   //count of colours in the palette
public:
  //suply count of colours in color table if known
  util_bmp_BI(int ncols=-1);

  //used for construction
  int init(int bpp, int w, int h, char const* palette=0, int ncols=0);
  //used after filling with BITMAPINFO, for inferring nColours
  void detectNColours();
  void setNColours(int);

  void check();
  int bpp();
  int w();
  int h();

  util_bmp_RgbUnion *palette();
  int nColours();
  void greyScalePalette(unsigned char* dst);
};





/////////////////////////////////////////////////////////////
//  util_bmp_BFH - 
//     wrapper for BitmapFileHeader,
//     layout from wingdi.h
class util_bmp_BFH{
  enum magic{BITMAP=0x4d42};

  util_endian::ONE_BYTE bfType[2];
  util_endian::ONE_BYTE bfSize[4];
  util_endian::ONE_BYTE bfReserved1[2];
  util_endian::ONE_BYTE bfReserved2[2];
  util_endian::ONE_BYTE bfOffBits[4];
public:
  util_bmp_BFH();

  int init(int sz_bi, int sz_bits);

  void check();
  int sz_bi();
  int sz_bits();

  static bool checkMagic(unsigned char const first, unsigned char const second);
};


#endif




