#include "util_debug.hpp"
#include "util_time.hpp"

#include <fstream>
#include <string>
#include <assert.h>

class LogImp{
  public:
    std::string name;
    std::ofstream fs;
    util_time timer;
    long t0;
    long divisor_ms;
  public:
    long time_ms(){
      return (timer()-t0)/divisor_ms;
    }
    LogImp(char const* str):
      name(str), 
      fs(name.c_str(), std::ofstream::out),
      t0(timer()), 
      divisor_ms(timer.frequency()/1000)
    {
      if (divisor_ms==0) {
        assert(0);
        divisor_ms=1;
      }
    }
};

util::Log::Log(char const* str):
  pimp(new LogImp(str))
{
  prolog();
  pimp->fs <<"Opening log\n";
}

util::Log::~Log() {
  prolog();
  pimp->fs <<"Closing log\n";
  pimp->fs.close();
}

void util::Log::flush(){
  //pimp->fs.close();
  //pimp->fs.open(pimp->name.c_str(), 
  //  std::ofstream::app|std::ofstream::out);
  pimp->fs.flush();
}

void util::Log::prolog(){
  pimp->fs <<pimp->time_ms() <<": ";
}

util::Log& util::Log::operator<<(char const* str){
  pimp->fs <<str;
  return *this;
}

util::Log& util::Log::operator<<(int i){
  pimp->fs <<i;
  return *this;
}
util::Log& util::Log::operator<<(unsigned int i){
  pimp->fs <<i;
  return *this;
}
util::Log& util::Log::operator<<(unsigned long i){
  pimp->fs <<i;
  return *this;
}
util::Log& util::Log::operator<<(long i){
  pimp->fs <<i;
  return *this;
}
util::Log& util::Log::operator<<(double d){
  pimp->fs <<d;
  return *this;
}

