#include "util_fs.hpp"

#include <stdlib.h>

#include <string>
#include <iostream>
#include <sstream>
#include <stdexcept>


std::string util_fs::uniqueFilename(
  char const* stem)
{
  std::string newName;
  std::string pom=stem;
  size_t pos=pom.find_last_of('.');
  bool gotExt=true;
  if (pos==std::string::npos)
    gotExt=false;

  std::string ext(
    gotExt?
    std::string(pom.begin()+pos+1, pom.end()):
    ""
  );
  std::string base(
    gotExt?
    std::string(pom.begin(), pom.begin()+pos):
    pom
  );

  int count=0;
  while(1) {
    if (count==10000){
      throw std::runtime_error(
        "util_fs::uniqueFilename - file count exceeded -"
        "check contents of getLogDir()");
    }

    std::ostringstream oss;
    oss.width(4);
    oss.fill('0');
    oss <<count;

    newName=base;
    newName+="-";
    newName+=oss.str();
    if (gotExt){
      newName+=".";
      newName+=ext;
    }

    bool exists=checkFileRead(newName.c_str());
    if (!exists)
      break;

    ++count;
  }
  return newName;
}


// getXXXDir private one-time worker
class DirFinder{
  std::string logDir_;
  std::string tmpDir_;
  DirFinder();
  bool checkEnvDirWrite(char const* dir){
    return dir!=0 && *dir!=0 &&
           util_fs::checkFileWrite(dir);
  }
public:
  static DirFinder& instance(){
    static DirFinder singleton;
    return singleton;
  }
public:
  const std::string& logDir(){return logDir_;}
  const std::string& tmpDir(){return tmpDir_;}
};

DirFinder::DirFinder(){
  char const* dir;
  dir=getenv("TMPDIR");
  if (!checkEnvDirWrite(dir)){
    dir=getenv("TMP");
    if (!checkEnvDirWrite(dir)){
      dir=getenv("TEMP");
      if (!checkEnvDirWrite(dir)){
        dir=util_fs::defaultTempDir();
        if (!checkEnvDirWrite(dir)){
          throw std::runtime_error(
            "DirFinder::DirFinder - "
            "could not locate directory for temporary files\n"
            "Fix by declaring LOGDIR or TMPDIR environment variable");
        }
      }
    }
  }
  tmpDir_=dir;
  tmpDir_+=util_fs::dirDelimiter();


  dir=getenv("LOGDIR");
  if (!checkEnvDirWrite(dir)){
    dir=tmpDir_.c_str();
  }
  logDir_=dir;
  logDir_+=util_fs::dirDelimiter();

  // std::cerr <<"Writing logs to " <<logDir_ <<std::endl;
}


std::string util_fs::getLogDir(){
  return DirFinder::instance().logDir();
}
std::string util_fs::getTmpDir(){
  return DirFinder::instance().tmpDir();
}


