#include "util_ras.hpp"

#include <assert.h>
#include <stdexcept>
#include <iostream>

util_ras_hdr::util_ras_hdr(){
  util_endian::putBig4(RAS_MAGIC, ras_magic);
  util_endian::putBig4(RT_STANDARD, ras_type);
  util_endian::putBig4(RMT_NONE, ras_maptype);
  util_endian::putBig4(0, ras_maplength);
}
void util_ras_hdr::init(int bpp, int w, int h){
  if (bpp!=8 && bpp!=24 && bpp!=32)//32: extension added on 000105
    throw std::runtime_error("util_ras_hdr::init - unsupported depth");

  util_endian::putBig4(w, ras_width);
  util_endian::putBig4(h, ras_height);

  util_endian::putBig4(bpp, ras_depth);
  util_endian::putBig4(w*h*bpp/8, ras_length);
}

int util_ras_hdr::w(){
  return util_endian::getBig4(ras_width);
}
int util_ras_hdr::h(){
  return util_endian::getBig4(ras_height);
}
int util_ras_hdr::bpp(){
  return util_endian::getBig4(ras_depth);
}

int util_ras_hdr::size(){
  return util_endian::getBig4(ras_length);
}
int util_ras_hdr::maplen(){
  return util_endian::getBig4(ras_maplength);
}


void util_ras_hdr::check(){
  if (RAS_MAGIC !=util_endian::getBig4(ras_magic))
    throw std::runtime_error("util_ras_hdr::check - unknown magic");

  // RT_FORMAT_RGB has been used by ImageMagick, probably in error
  if (RT_STANDARD   !=util_endian::getBig4(ras_type) &&
      RT_FORMAT_RGB !=util_endian::getBig4(ras_type))
    throw std::runtime_error("util_ras_hdr::check - unstandard type");

  bool rmt_none=true;
  if (util_endian::getBig4(ras_maptype) == RMT_EQUAL_RGB){
    rmt_none=false;
    if (maplen()!=3*256 || bpp() !=8)
      throw std::runtime_error("util_ras_hdr::check - error for RMT_EQUAL_RGB");
  } else{
    if (util_endian::getBig4(ras_maptype) != RMT_NONE)
      throw std::runtime_error("util_ras_hdr::check - unknown maptype");
    if (maplen()!=0)
      throw std::runtime_error("util_ras_hdr::check - error for RMT_NONE");
  }

  if (w()%2!=0) //was: w()%4==0
    throw std::runtime_error("util_ras_hdr::check - odd width");
  int expected_len=w()*h()*bpp()/8;
  if (!size())
    util_endian::putBig4( expected_len, ras_length);
  if (size()!=expected_len)
    throw std::runtime_error("util_ras_hdr::check - error in ras_length");
}
bool util_ras_hdr::has_palette(){
  return (RMT_NONE!=util_endian::getBig4(ras_maptype));
}

//  static 
bool util_ras_hdr::checkMagic(
  unsigned char const c0, unsigned char const c1,
  unsigned char const c2, unsigned char const c3)
{
  util_endian::ONE_BYTE magic[4];
  util_endian::putBig4(RAS_MAGIC, magic);
  return (magic[0]==c0 &&
          magic[1]==c1 &&
          magic[2]==c2 &&
          magic[3]==c3);
}



//from /usr/openwin/share/include/pixrect/rasterfile.h

//#define RT_OLD		0	/* Raw pixrect image in 68000 byte order */
//#define RT_STANDARD	1	/* Raw pixrect image in 68000 byte order */
//#define RT_BYTE_ENCODED	2	/* Run-length compression of bytes */
//#define RT_FORMAT_RGB	3	/* XRGB or RGB instead of XBGR or BGR */
//#define RT_FORMAT_TIFF	4	/* tiff <-> standard rasterfile */
//#define RT_FORMAT_IFF	5	/* iff (TAAC format) <-> standard rasterfile */
//#define RT_EXPERIMENTAL 0xffff	/* Reserved for testing */

///* Sun registered ras_maptype's */
//#define RMT_RAW		2
///* Sun supported ras_maptype's */
//#define RMT_NONE	0	/* ras_maplength is expected to be 0 */
//#define RMT_EQUAL_RGB	1	/* red[ras_maplength/3],green[],blue[] */

