#ifndef util_ras_hpp
#define util_ras_hpp

#include "util_endian.hpp"

class util_ras_hdr {
  util_endian::ONE_BYTE	ras_magic[4];		 /* magic number */
  util_endian::ONE_BYTE	ras_width[4];		 /* width (pixels) of image */
  util_endian::ONE_BYTE	ras_height[4];	 /* height (pixels) of image */
  util_endian::ONE_BYTE	ras_depth[4];		 /* depth (1, 8, or 24 bits) of pixel */
  util_endian::ONE_BYTE	ras_length[4];	 /* length (bytes) of image */
  util_endian::ONE_BYTE	ras_type[4];		 /* type of file; see RT_* below */
  util_endian::ONE_BYTE	ras_maptype[4];	 /* type of colormap; see RMT_* below */
  util_endian::ONE_BYTE	ras_maplength[4];/* length (bytes) of following map */
 /* color map follows for ras_maplength bytes, followed by image */

  enum magic{RAS_MAGIC=0x59a66a95};
  enum type{RT_OLD=0,       RT_STANDARD,   RT_BYTE_ENCODED,
            RT_FORMAT_RGB,  RT_FORMAT_TIFF,RT_FORMAT_IFF, 
            RT_EXPERIMENTAL=0xffff };
  enum map_type{RMT_NONE=0, RMT_EQUAL_RGB, RMT_RAW};

public:
  util_ras_hdr();
  void init(int bpp, int w, int h);

public:
  void check();
  bool has_palette();

  int bpp();
  int w();
  int h();

  int size();
  int maplen();

  static bool checkMagic(unsigned char const c0, unsigned char const c1,
                         unsigned char const c2, unsigned char const c3);
};

#endif

