#include "util_registry.hpp"

#include <utility>
#include <string>

#include <assert.h>

util::Registry::CreatorMap& 
util::Registry::getRegistry(
  char const *name_base)
{
  RegistryMap::iterator it=map_.find(name_base);
  if (it==map_.end()){
    std::string str("Registry::getCreators - illegal name_base: ");
    str += name_base;
    throw std::runtime_error(str);
  }
  return it->second;
}





/////////////////////////////////////
//audience: CreatorBase ctor
//(the calling object is supposed to be static,
// so that its pointer should persist until exit())
void util::Registry::registerCreator(
  char const* name_base, 
  RegistryCreatorBase* pb)
{
  std::string strBase(name_base);

  // std::cerr <<"In registerCreator() for " 
  //   <<strBase <<"->" <<pb->name_derived() <<"\n";

  RegistryMap::iterator it=map_.lower_bound(strBase);
  if (it==map_.end() ||
      it->first!=strBase)
  {
    it=map_.insert(it, std::make_pair(strBase, CreatorMap()));
  }
  CreatorMap& cm=it->second;
  // std::cerr <<"  CreatorMap= " <<&cm <<"\n";

  std::pair<CreatorMap::iterator, bool> rv;
  rv=cm.insert(std::make_pair(pb->name_derived(), pb));

  assert(rv.second==true);  
  assert(cm.find(pb->name_derived()) != cm.end());
}



/////////////////////////////////////
//audience: clients 
util::Registry& util::Registry::instance(){
  static Registry theRegistry;
  return theRegistry;
}


