#include "util_string.hpp"

/******************************************************
*
*        util_string.cpp  by Sinisa Segvic
*
** Various string utility functions
*
*******************************************************/

#include <ctype.h>
#include <assert.h>
#include <string.h>
#include <stdio.h>

#include <stdexcept>
#include <string>
#include <sstream>
#include <algorithm>



int util_string::compareNoCase(
  char const *s, char const *t)
{
  if (s==0 || t==0){
    return s!=t;
  }
  while (1){
    const char c=*s++;
    const char d=*t++;
    if (::tolower(c)!=::tolower(d)){
      assert(c!=0 || d!=0);
      return c-d;
    }
    if (c==0 && d==0)
      break;
  }
  return 0;
}

void util_string::tolower(
  std::string& s)
{
  std::string::iterator it=s.begin();
  while (it != s.end()){
    *it=::tolower(*it);
    ++it;
  }
}

void util_string::trimWs(
  std::string& str)
{
  char const* ws="\t\n\f\r ";

  //kill leading and trailing spaces
  while (str.find_first_of(ws)==0)
    str.erase(0, 1);
  if (str.length()==0)
    return;
  while (str.find_last_of(ws)==str.length()-1)
    str.erase(str.length()-1, 1);
}



void util_string::fixBackslashEscapes(
  char const* src, std::string& dst)
{
  bool escaped=false;
  dst="";
  while (*src){
    if (escaped){
      switch (*src){
      default:
      case '"':
      case '\\':
        dst+=*src;
      case 't':
        dst+='\t';
        break;
      case 'n':
        dst+='\n';
        break;
      }
      escaped=false;
    } else{
      if (*src=='\\'){
        escaped=true;
      } else{
        dst+=*src;
      }
    }
    src++;
  }
}

void util_string::incrementName(
  std::string& name)
{
  bool virgin=true;
  const char separator='-';
  size_t pos=name.find_last_of(separator);
  if (pos!=std::string::npos){
    virgin=false;
    for (size_t i=pos+1; i<name.length(); i++){
      if (!isdigit(name[i])){
        virgin=true;
        break;
      }
    }
  }
  if (virgin){
    name=name+"-0";
  } else{
    int n=strToInt(name.substr(pos+1));
    std::ostringstream oss;
    oss <<n+1;
    name=name.substr(0,pos)+oss.str();
  }
}

void util_string::split(
  char const*s, char delim, 
  std::vector<std::string>& vec)
{
  std::string str(s);
  vec.clear();
  
  while (1){
    size_t pos=str.find(delim);
    if (pos==std::string::npos){
      vec.push_back(str);
      break;
    }
    vec.push_back(
      std::string(str.begin(), str.begin()+pos));
    str.erase(0,pos+1);
  }
}

void util_string::indent(
  const std::string& str, 
  std::string& strout)
{
  strout="";
  size_t i=0;
  while (1){
    size_t j=str.find("\n",i);
    if (j == std::string::npos){
      break;
    }
    strout+="  ";
    strout+=str.substr(i,j+1-i);
    i=j+1;
  }

  if (i<str.size()){
    strout+="  ";
    strout+=str.substr(i);
  }
}

int util_string::strToInt(const std::string& s){
  char *error=0;
  long rv=strtol(s.c_str(), &error, 10);

  if (*error!=0){
    std::string str("util_string::strToInt - "
      "could not convert \'");
    str=str+s+'\'';
    throw std::runtime_error(str);
  }

  return static_cast<int>(rv);
}






















char const *util_string::getFileExtension(char const *source){

  std::string pom=source;
  char const* rv=source+pom.size();

  std::string::size_type pos=pom.find_last_of('.');
  if (pos!=std::string::npos)
    rv=source+pos+1;

  return rv;
}

void util_string::retractPath(
  char const* fullPath,
  std::string& fileName)
{
  fileName=fullPath;
  std::string::size_type index=
    fileName.find_last_of("/\\");
  if (index==fileName.size()-1)
    index=fileName.find_last_of("/\\", index-1);
  if (index!=std::string::npos){
    fileName.erase(0, index+1);
  }
}

void util_string::splitPath(
  char const* fullPath,
  std::string& dir,
  std::string& file)
{
  std::string str(fullPath);
  std::string::size_type index=
    str.find_last_of("/\\");
  if (index!=std::string::npos){
    file=str.substr(index+1);
    dir=str.substr(0, index+1);
  } else{
    dir="";
    file=str;
  }
}




int util_string::getHostNumber(
  char const* ipaddr)
{
  std::string str(ipaddr);
  size_t pos=str.rfind('.');
  assert (pos!=std::string::npos);
  int hostNumber= strToInt(str.substr(pos+1));
  return hostNumber;
}



