#include "util_time.hpp"

#define TARGET_API_MAC_CARBON 1
#include <Carbon/Carbon.h>
#include <QuickTime/QuickTime.h>

#include <time.h>
//#include <sys/time.h>

#include <string>
#include <sstream>
#include <stdexcept>


/******************************************************
*
*        util_time_macx.cpp  by Sinisa Segvic
*
** timing and profiling on macx systems
** using systemMicrosecondClock Component
*
*******************************************************/
#ifndef NTIMEPROFILE

namespace{
  void CE(OSErr err, char const* str){
    if (err!=noErr){
      std::ostringstream oss;
      oss <<"MAC OS-X error on " <<str <<": " <<err;
      throw std::runtime_error(oss.str());
    }
  }
}

class ClockWrapper{
public:
  ClockWrapper();
  void fill(TimeRecord &);
private:
  ComponentInstance ci_;
};
ClockWrapper::ClockWrapper(){
  CE(OpenADefaultComponent(
    clockComponentType, systemMicrosecondClock, &ci_), 
    "OpenADefaultComponent ");
}
void ClockWrapper::fill(TimeRecord &tr){
  CE(ClockGetTime (ci_,&tr), "ClockGetTime");
}
namespace{
  ClockWrapper clk;
  TimeScale ts1k=1000;
  TimeScale ts10k=10000;
}


util_time::util_time(){
  frequency_=10*1000;
}

util_time::Data util_time::operator()(){
  TimeRecord tr;
  clk.fill(tr);
  ConvertTimeScale (&tr, ts10k);
  return tr.value.lo;
}

  
int util_time::secEpoch(){
  time_t tmUnix;
  time (&tmUnix );
  return tmUnix; 
}

int util_time::msDayUtc(){
  TimeRecord tr;
  clk.fill(tr);
  ConvertTimeScale (&tr, ts1k);
  return tr.value.lo;
}
#endif
