#include "util_time.hpp"

#include <windows.h>
#include <time.h>
#include <sys/types.h>
#include <sys/timeb.h>

#include <iostream>
#include <stdexcept>

#include <math.h>
#include <assert.h>

#ifndef NTIMEPROFILE


class MSTiming{
  MSTiming();
public:
  ~MSTiming(){}
  typedef unsigned long TimeType;
  static MSTiming& instance();
public:
  TimeType frequency(){return f_;}
  TimeType time();
private:
  TimeType convert(LONGLONG ll);
  unsigned long f_;
  int shift_;

  ULONGLONG mask_;
};


MSTiming::MSTiming(){
  mask_=static_cast<ULONGLONG>(-1)>>(8*sizeof(TimeType));
  assert(mask_==0xffffffff);

  LARGE_INTEGER li;
  BOOL ok=QueryPerformanceFrequency(&li);
  if (!ok){
    throw std::runtime_error("MSTiming::MSTiming - QueryPerformanceFrequency returned 0");
  }

  // sizeof(long)!=8 ... :-(
  LONGLONG llFreq=li.QuadPart;
  double f=static_cast<double>(llFreq);
  double logf=log(f);
  // we require at least 1024*8 clocks per sec
  shift_=static_cast<int>(logf/log(2.0))-13;
  if (shift_<0){
    throw std::runtime_error(
      "MSTiming::MSTiming - QueryPerformanceFrequency"
      "reported frequency less than 8*1024");
  }

  f_=convert(llFreq);
}

MSTiming& MSTiming::instance(){
  static MSTiming instance;
  return instance;
}

MSTiming::TimeType MSTiming::time(){
  LARGE_INTEGER li;
  QueryPerformanceCounter(&li);
  return convert(li.QuadPart);
}

MSTiming::TimeType MSTiming::convert(LONGLONG ll){
  // sizeof(long)!=8 ... :-(
  LONGLONG llNormalized=ll>>shift_;
  // assert(llNormalized<(1<<30));
  return static_cast<TimeType>(llNormalized & mask_);
}





util_time::util_time():
  frequency_(MSTiming::instance().frequency())
{
}

util_time::Data util_time::operator()(){
  return MSTiming::instance().time();
}

  
int util_time::secEpoch(){
  time_t tmUnix;

  time (&tmUnix );
  return static_cast<int>(tmUnix); 
}

int util_time::msDayUtc(){
  time_t tmUnix;

  time (&tmUnix);
  int secs=static_cast<int>(tmUnix) % (24*60*60);

  struct _timeb timebuffer;
  _ftime( &timebuffer );

  int ms=1000*secs + timebuffer.millitm;
  return ms;
}

#endif
