#ifndef vd_base_hpp
#define vd_base_hpp

/******************************************************
*
** vd_base by Sinisa Segvic: 
*
** storing a sequence of image frames 
** format may be common (vd_file) or variable (vd_win)
*
** uses img_fmt, img_data
*
*******************************************************/

#include "../img/img_data.hpp"

#include <stdexcept>

class vd_base{
  img_fmt fmt_;

//construction
protected:
  vd_base(const img_fmt& f):fmt_(f){}
public:
  virtual ~vd_base();

//attributes
public:
  virtual bool isFile()=0;
  virtual bool isWin()=0;
  virtual bool canReformat()=0;

  const img_fmt& fmt(){return fmt_;}
  void reformat(const img_fmt& f){
    if (! canReformat())
      throw "vd_base::reformat - unsupported"
            "by the concrete class";
    fmt_=f;
  }

//operations
public:
  //puts the image frame into the stream
  virtual void operator()(const img_data& img, int time=0, int frame=-1)=0;

//utilities
protected:
  void checkFmt(const img_fmt& f){
    if (! (f==fmt()))
      throw std::runtime_error("vd_base::checkFmt - format mismatch.");
  }

//forbidden
private:
  vd_base(const vd_base&);
  const vd_base& operator=(const vd_base&);
};



#endif
