#include <cassert>

#include "vd_file_avi.hpp"

#include "../util/util_avi.hpp"
#include "../util/util_endian.hpp"
#include "../img/img_util.hpp"

vd_file_avi::vd_file_avi(
  const img_fmt& f,
  const char* filename)
:
  vd_file(f),
  aw_(* new util::AviWrite(
    filename, 
    fmt().width(), fmt().height(), 
    fmt().szPixel(), fmt().szBits()))
{}

vd_file_avi::~vd_file_avi(){
  delete &aw_;
}

void vd_file_avi::operator()(const img_data& img, int time, int){
  checkFmt(img.fmt());

  img_data imgTmp(img);

  //16bpp imapges are stored in little endian
  //unlike BMP format, we use rgbrgbrgb for 24bpp images
  //unlike BMP format, we use rgbxrgbxrgbx for 32bpp images
  switch (imgTmp.fmt().szPixel()){
    case 1: break;
    case 2:
      if (util_endian::endianBig())
        img_util::flipEndian2(imgTmp);
      break;
    case 3:
    case 4:
      img_util::flipEndian(imgTmp);
      break;
    default:
      throw "img_io_save_bmp::operator() - "
            "illegal szPixel()";
  }
  //BMP maps scanlines in a bottom-up manner
  img_util::flipVertical(imgTmp);

  aw_(imgTmp.bits(), time);
}


