#ifndef vs_base_hpp
#define vs_base_hpp

/******************************************************
*
** vs_base.hpp by Sinisa Segvic:
*
** videosource: 
**   acquisition of a sequence of image
**   frames with possibly variable format
*
*******************************************************/

class img_fmt;
class img_data;

class vs_base{
//construction
protected:
  vs_base();
public:
  virtual ~vs_base();

//constant attributes
public:
  virtual bool isFile()=0;
  virtual bool isLive()=0;
  //the distinguishing name
  virtual char const* name()=0;

//total count of frames
public:
  virtual int count() =0;
  
//input range
public:
  void setRange(int from, int to);
  int size();
  int from();
  int to();

//random access
public:
  int tell();
  void seek(int);
protected:
  int frame(){return frame_;}

//current format
public:
  virtual const img_fmt& fmt()=0;

//acquisition methods
public:
  virtual int msDayUtc()=0;
public:
  void getFrame(img_data&);
  void getFrame(img_data&, int);
  void getNextFrame(img_data&);
private:
  virtual void getFrameSimple(img_data&)=0;

//data
private:
  int frame_;
  int from_;
  int to_;

//forbidden
private:
  vs_base(const vs_base&);
  const vs_base& operator=(const vs_base&);
};


//////////////////////////
// operations
inline void vs_base::getFrame(img_data& d){
  getFrameSimple(d);
}



#endif
