#include "vs_file_avi.hpp"

#include "../img/img_data.hpp"
#include "../img/img_util.hpp"

#include <avifile-0.7/avifile.h>

#include <string>
#include <stdexcept>

/////////////////////////////////////
// vs_file_avi private implementation

struct vs_file_avi_imp{
public:
  vs_file_avi_imp(char const* filename);
  ~vs_file_avi_imp();
public:
  avm::IReadFile *pfile_;
  avm::IReadStream *pstream_;
  avm::StreamInfo *pinfo_;
  img_fmt fmt_;
  std::string name_;
};

vs_file_avi_imp::vs_file_avi_imp(
  const char* filename)
:
  pfile_(avm::CreateReadFile(filename)),
  name_(filename)
{
  int n=pfile_->VideoStreamCount();
  if (n==0)
    throw std::runtime_error(
      "vs_file_avi_unix - no video streams in given file!");
  pstream_=pfile_->GetStream(0, avm::IStream::Video);
  pstream_->SetDirection(true);
  pstream_->StartStreaming();
  pinfo_= pstream_->GetStreamInfo();
  
  int w=pinfo_->GetVideoWidth();
  int h=pinfo_->GetVideoHeight();
  int szpix=pstream_->GetFrameSize()/w/h;
  fmt_=img_fmt(w,h, szpix);  
}
vs_file_avi_imp::~vs_file_avi_imp(){
  pstream_->StopStreaming();
  delete pfile_;
}


/////////////////////////////////////
// vs_file_avi construction

vs_file_avi::vs_file_avi(
  const char* filename)
:
  pimp (new vs_file_avi_imp(filename))
{
}
vs_file_avi::~vs_file_avi(){
  delete pimp;
}


//attributes

int vs_file_avi::count(){
  return pimp->pstream_->GetLength();
}
int vs_file_avi::msDayUtc(){
  return static_cast<int>(pimp->pstream_->GetTime()*1000+.5);
}
char const* vs_file_avi::name(){
  return pimp->name_.c_str();
}

const img_fmt& vs_file_avi::fmt(){
  return pimp->fmt_;
}



// operations
void vs_file_avi::getFrameSimple(img_data& img){

  img.reformat(fmt());
  if (frame()+0u != pimp->pstream_->GetPos()){
    pimp->pstream_->SeekToKeyFrame(frame());
  }
  CImage* pimg=pimp->pstream_->GetFrame(true);
  img.setBits(pimg->Data(0));
  pimg->Release();
  
  if (img.fmt().szPixel()==3 ||
       img.fmt().szPixel()==4)
     img_util::flipEndian3(img);
}  


