#ifndef vs_file_many_hpp
#define vs_file_many_hpp

/******************************************************
*
*   vs_file_many.hpp by Sinisa Segvic
*
** describes acquisition from multiple files
** with different formats through 
** a vs_base interface, 
*
** each string on the spec vector denotes
** a separate vs_file with the following syntax:
** path.ext#from_frame-to_frame#from_frame2-to_frame2...
*
** specifying only a path implies the range [0,1>!
*
** e.g:
*
** d:/home/sinisa/work/data/tie/D338/west/15/5.bmp
** d:/home/sinisa/work/data/avioni/hrt3.srs#100-200
** d:/home/sinisa/work/data/auti/auti1.avi#20-50
*
** vs_file_many IS_A vs_file IS_A vs_base
*
*******************************************************/

#include "vs_file.hpp"
#include <boost/scoped_ptr.hpp>
#include <vector>


class vs_file_many_imp;

class vs_file_many:
  public vs_file
{
public:
  vs_file_many(
    const std::vector<std::string>& vecStrFile);
  virtual ~vs_file_many();

//vs_base methods
public:
  virtual int count();
  virtual char const* name()=0;
  virtual const img_fmt& fmt();
  virtual int msDayUtc();
private:
  virtual void getFrameSimple(img_data&);

// vs_file_many methods
public:
  static int parseRange(const std::string& str, 
    std::string& path, 
    std::vector<int> from, 
    std::vector<int> to);

// vs_file_many data
private:
  boost::scoped_ptr<vs_file_many_imp> pimp;
};


#endif


