#include "vs_file_txt.hpp"

#include "../util/util_string.hpp"

#include <string>
#include <sstream>
#include <vector>
#include <fstream>
#include <stdexcept>



// private helpers

namespace{
  bool pathsep(char c){
    return (c=='/' || c=='\\');
  }

  struct MyThrower{
    MyThrower(char const* name){
      context_="vs_file_txt (";
      context_+=name;
      context_+=") - ";
    }

    void operator()(const std::string &msg){
      throw std::runtime_error(context_+msg);
    }

    std::string context_;
  };


  std::vector<std::string> TxtFileParser(
    char const* filename)
  {
    std::vector<std::string> vecVsFile;
    MyThrower error(filename);

    std::ifstream ifs(filename);
    if (!ifs.good())
      error("access error");

    std::string strDir, strTxtFile;
    util_string::splitPath(filename, strDir, strTxtFile);

    while (!ifs.eof()){
      // get a valid input line
      std::string strVsFile;
      std::getline(ifs, strVsFile);

      size_t commentPos=strVsFile.find('#');
      if (commentPos!=std::string::npos)
        strVsFile.erase(commentPos);
      util_string::trimWs(strVsFile);
      if (strVsFile.length()==0 || strVsFile[0]=='#')
        continue;

      if (strVsFile[0]=='@'){
        std::string strNewDir=strVsFile.substr(1);
        if (!pathsep(strNewDir[0])){
          strDir+=strNewDir;
        } else{
          strDir=strNewDir;
        }
        if (!pathsep(strNewDir[strNewDir.size()-1])){
          strDir+='/';
        }
        continue;
      }

      vecVsFile.push_back(strDir+strVsFile);
    }
    return vecVsFile;
  }
}








//////////////////////////////////////////////////
//vs_file_txt implementation

vs_file_txt::vs_file_txt(
  char const* name)
:
  vs_file_many(TxtFileParser(name)),
  strTxtFile_(name)
{
}

vs_file_txt::~vs_file_txt(){
}



//////////////////////////////////////////////////
//attributes
char const* vs_file_txt::name(){
  return strTxtFile_.c_str();
}  
