#include "vs_file.hpp"

#include "vs_file_srs.hpp"
#include "vs_file_single.hpp"
#include "vs_file_avi.hpp"
#include "vs_file_wmv.hpp"
#include "vs_file_txt.hpp"
#include "vs_file_dir.hpp"

#include "../util/util_string.hpp"
#include "../util/util_fs.hpp"

#include <iostream>
#include <stdexcept>

#include <assert.h>

/******************************************************
*
*        vs_file_util.CPP  by Sinisa Segvic
**  utility functions for the whole vs_file hierarchy
**
**************************************************/
#include "vs_file_util.hpp"


vs_file* vs_file_util::create(const char* name){
  vs_file *pvsf=0;

  size_t len=strlen(name);
  if (name[len-1]==util_fs::dirDelimiter()){
    pvsf=new vs_file_dir(name);
  } else{
    char const* ext=util_string::getFileExtension(name);
    if (!ext){
      throw std::runtime_error("vs_file_util::create - got file without extension");
    }

    if (!util_string::compareNoCase(ext, vs_file_srs::extension())){
      pvsf=new vs_file_srs(name);
    }
    else if (!util_string::compareNoCase(ext, vs_file_avi::extension())){
      pvsf=new vs_file_avi(name);
    }
    else if (!util_string::compareNoCase(ext, vs_file_wmv::extension())){
      pvsf=new vs_file_wmv(name);
    }
    else if (!util_string::compareNoCase(ext, vs_file_txt::extension())){
      pvsf=new vs_file_txt(name);
    }
    else{
      pvsf=new vs_file_single(name);
    }
  }
  return pvsf;
}

