#include "vs_file_wmv.hpp"
#include "../img/img_data.hpp"

#include <wmsdk.h>
#include <boost/smart_ptr.hpp>

#include <string>
#include <iostream>

//source: 
//http://msdn2.microsoft.com/en-us/windowsmedia/bb190309.aspx
//http://www.codeproject.com/KB/audio-video/readingWMVASFWMA.aspx

class vs_file_wmv_imp{
public:
  vs_file_wmv_imp(const char* filename);
  ~vs_file_wmv_imp();
public:
  int count() {return count_;}
  char const* name() {return name_.c_str();}
  const img_fmt& fmt() {return fmt_;}
  int msDayUtc() {return static_cast<int>(m_cnsSampleTime/(10*1000.0)+0.5);}
  void getFrame(img_data&, int frame);
private:
  std::string name_;
  img_fmt fmt_;
  int count_;
  int frame_;

	IWMSyncReader* m_ISyncReader;
	int m_iVideoOutputNumber;
	int m_iVideoStreamNumber;

  INSSBuffer* m_pINSSBuffer;
	BYTE* m_bitmapBuffer;
  QWORD m_cnsSampleTime;
  QWORD m_cnsSampleDuration;
	DWORD m_dwrdBitmapBufferLength;
	QWORD m_qwCurrrentPostionInSeconds;
};


vs_file_wmv_imp::vs_file_wmv_imp(const char* filename):
  name_(filename), fmt_(10,10,1), count_(0), frame_(0),
  m_bitmapBuffer(0)
{
  HRESULT hr  = WMCreateSyncReader(NULL,0,&m_ISyncReader);
  assert (hr==S_OK);

  wchar_t *wfilename=new wchar_t[strlen(filename) + 1];
  mbstowcs_s(0, wfilename, strlen(filename) + 1, filename, _TRUNCATE);
  //std::wcerr <<wfilename <<"\n";
  hr = m_ISyncReader->Open(wfilename);
  if (hr!=S_OK)
    throw "vs_file_wmv_imp - couldn't open input file";
    
  // Analyise output streams in the file 
  IWMOutputMediaProps* IVideoOutputProps=0;
  WM_MEDIA_TYPE* theMediaType=0;
  DWORD theOutputCount;
  m_ISyncReader->GetOutputCount(&theOutputCount);
  //determine which stream is video
  for (unsigned i=0; i<theOutputCount; ++i) {
    m_ISyncReader->GetOutputProps(i,&IVideoOutputProps);
 
    DWORD theSize;
    hr = IVideoOutputProps->GetMediaType(NULL,&theSize);
    theMediaType = (WM_MEDIA_TYPE*) new BYTE [theSize];
    hr = IVideoOutputProps->GetMediaType(theMediaType,&theSize);
    assert(!FAILED(hr));
    if (WMMEDIATYPE_Video == theMediaType->majortype){
      m_iVideoOutputNumber = i;
      m_ISyncReader->GetStreamNumberForOutput(m_iVideoOutputNumber,(WORD*)&m_iVideoStreamNumber);
      if (theMediaType->formattype == WMFORMAT_VideoInfo)
      {
        WMVIDEOINFOHEADER* pwmhdr=(WMVIDEOINFOHEADER*) theMediaType->pbFormat;
        int width= pwmhdr->bmiHeader.biWidth;
        int height=pwmhdr->bmiHeader.biHeight;
        int szpix= pwmhdr->bmiHeader.biBitCount/8;
        fmt_=img_fmt(width,height, szpix);
      }
    }
  }
  //release the memory
  if (IVideoOutputProps) IVideoOutputProps->Release();
  delete [] theMediaType;
    
  //receive correct sample durations.
  hr = m_ISyncReader->SetOutputSetting(m_iVideoOutputNumber,
    g_wszVideoSampleDurations, WMT_TYPE_BOOL, (BYTE const*)"TRUE", sizeof(BYTE*));
  assert(!FAILED(hr));
  
  //receive uncompressed Samples
  assert(!FAILED(hr));
    
  //Get wmv Duration (total time)
  IWMMetadataEditor *pEditor;
  hr= WMCreateEditor(&pEditor);
  assert (hr==S_OK);
  pEditor->Open(wfilename);
  IWMHeaderInfo3* pHdrInfo;
  pHdrInfo    = NULL;
  hr = pEditor->QueryInterface(IID_IWMHeaderInfo3,(void**)&pHdrInfo);
  
  WORD wStream =0;// for any stream;
  WMT_ATTR_DATATYPE dType=WMT_TYPE_QWORD;
  WORD wSize =sizeof(QWORD);
  BYTE* pValue= new BYTE[wSize];
  hr = pHdrInfo->GetAttributeByName(&wStream,L"NumberOfFrames",&dType,pValue,&wSize);
  if (hr==ASF_E_NOTFOUND){
    hr = pHdrInfo->GetAttributeByName(&wStream,L"Duration",&dType,pValue,&wSize);
    count_=static_cast<int>(*((QWORD*)pValue)/(10*1000*40));
  } else{
    count_=static_cast<int>(*((QWORD*)pValue));
  }

  delete [] pValue;
  if (pHdrInfo) pHdrInfo->Release();
  if (pEditor) pEditor->Release();
}

vs_file_wmv_imp::~vs_file_wmv_imp(){
  if (m_ISyncReader) m_ISyncReader->Release();
}
  
    

void vs_file_wmv_imp::getFrame(img_data& img, int frame)
{
  if (frame_!=frame){
    frame_=frame;
    m_ISyncReader->SetRangeByFrame(
      m_iVideoStreamNumber, frame_, 0);
  }

  DWORD dwFlags = 0;
  HRESULT hr = m_ISyncReader->GetNextSample(
    m_iVideoStreamNumber,
    &m_pINSSBuffer,
    &m_cnsSampleTime,
    &m_cnsSampleDuration,
    &dwFlags,
    NULL,//&dwOutputNumber,
    NULL);
  if (hr== NS_E_NO_MORE_SAMPLES){
    assert(0);
    throw "vs_file_wmv_imp::getFrame - undetected EOF";
  }
  if (hr!= S_OK){
    assert(0);
    throw "vs_file_wmv_imp::getFrame - did not succeed to get a frame";
  }
  if (dwFlags!=WM_SF_CLEANPOINT){
    //assert(0);
    //throw "vs_file_wmv_imp::getFrame - did not get a WM_SF_CLEANPOINT";
  }


  unsigned char *buffer;
  m_pINSSBuffer->GetBufferAndLength(&buffer,&m_dwrdBitmapBufferLength);
	if (m_bitmapBuffer==0)
		m_bitmapBuffer = new unsigned char[m_dwrdBitmapBufferLength];
  memcpy(m_bitmapBuffer,buffer,m_dwrdBitmapBufferLength);
  //calc the current postion in seconds
  m_qwCurrrentPostionInSeconds = (m_cnsSampleTime*100)/1000000000;
  ++frame_;

  img.reformat(fmt());
  memcpy(img.bits(), m_bitmapBuffer, img.fmt().szBits());


  m_pINSSBuffer->Release();
  m_pINSSBuffer = NULL;
}
  
  
vs_file_wmv::vs_file_wmv(
  const char* filename)
:
  pimp (new vs_file_wmv_imp(filename))
{
}
vs_file_wmv::~vs_file_wmv(){
  delete pimp;
}


//attributes

int vs_file_wmv::count(){
  return pimp->count();
}
int vs_file_wmv::msDayUtc(){
  return pimp->msDayUtc();
}
char const* vs_file_wmv::name(){
  return pimp->name();
}

const img_fmt& vs_file_wmv::fmt(){
  return pimp->fmt();
}



// operations
void vs_file_wmv::getFrameSimple(img_data& img){
  pimp->getFrame(img, frame());
}  

