#ifndef vs_grab_hpp
#define vs_grab_hpp

/******************************************************
*
*        vs_grab.hpp by Sinisa Segvic
*
*  component:
*     vs_grab
*
** implements image acquisition 
** from a frame grabber
*
** in order to support the
** parameterized factory method pattern,
** each concrete class should declare:
**  - public constructor with a single CtorArg argument
**  - static data member of the type 
**    Registry<vs_grab>::Creator<vs_grab_concrete>
**    which interfaces to the registry object
**  - static member s_name() which should be 
**    used to implement name()
*
** vs_grab IS_A vs_base
*
*******************************************************/

#include "vs_base.hpp"


class vs_grab: 
  public vs_base
{
//construction
protected:
  vs_grab();
public:
  virtual ~vs_grab();

//vs_base methods
public:
  virtual char const* name()=0;
  virtual int count();
  virtual bool isFile(){ return false;}
  virtual bool isLive(){ return true;}
public:
  virtual const img_fmt& fmt()=0;
public:
  //this should be overrriden with a more 
  //precise implementation whenever possible!
  virtual int msDayUtc();
private:
  virtual void getFrameSimple(img_data&)=0;

//vs_grab construction
public:
  // concrete classes ctor argument
  struct CtorArg{
    const CtorArg& operator=(const CtorArg&);
  public:
    int board;   // the board index
    int channel; // the channel index
    int mode;    // 0=PAL, 1=NTSC
    const img_fmt& fmt;
  public:
    CtorArg(int b, int ch, int m, const img_fmt& f):
      board(b), channel(ch), mode(m), fmt(f)
    {}
  };
  //concrete classes factory
  static vs_grab* createCheck(
    const char* name, 
    const CtorArg& arg);
  static vs_grab* create(
    const char* name, 
    const CtorArg& arg);
  
//data
private:
  int t0_;
};


#endif

