#include "vs_grabUtil.hpp"

#include <string.h>

// yuv2rgb source: coriander: src/conversions.h

typedef unsigned char BYTE;
typedef BYTE* PBYTE;
typedef BYTE const* PCBYTE;

namespace{
  inline void yuv2rgb(
    const int y, const int u, const int v,
    int& r, int& g, int& b)
  {
    r = y + ((v*1436) >>10);
    g = y - ((u*352 + v*731) >> 10);
    b = y + ((u*1814) >> 10);
    r = r < 0 ? 0 : r;
    g = g < 0 ? 0 : g;
    b = b < 0 ? 0 : b;
    r = r > 255 ? 255 : r;
    g = g > 255 ? 255 : g;
    b = b > 255 ? 255 : b;
  }
  inline void rgbput(
    const int r, const int g, const int b,
    PBYTE& pdst)
  {
    *pdst++ = static_cast<BYTE>(r); 
    *pdst++ = static_cast<BYTE>(g); 
    *pdst++ = static_cast<BYTE>(b);
  }
  inline void yuvput(
    const int y, const int u, const int v,
    PBYTE& pdst)
  {
    int r,g,b;
    yuv2rgb(y,u,v, r,g,b); 
    rgbput(r,g,b, pdst);
  }
}
// source: coriander v0.24, src/conversions.c

void vs_grabUtil::yuv444ToRgb(
  int w, int h, void const* ps, 
  void* pd)
{
  PCBYTE psrc=static_cast<PCBYTE>(ps);
  PBYTE pdst=static_cast<PBYTE>(pd);

  for (int l = 0; l < h; l++) {
    for (int i = 0; i < w; i++) {
      int u0 = (*psrc++) - 128; 
      int y0 = *psrc++; 
      int v0 = (*psrc++) - 128;
      yuvput(y0,u0,v0, pdst); 
    }
  }
}

void vs_grabUtil::yuv422ToRgb(
  int w, int h, void const* ps, 
  void* pd)
{
  PCBYTE psrc=static_cast<PCBYTE>(ps);
  PBYTE  pdst=static_cast<PBYTE>(pd);

  for (int l = 0; l < h; l++) {
    for (int i = 0; i < w/2; i++) {
      int u = (*psrc++) - 128; 
      int y0 = *psrc++; 
      int v = (*psrc++) - 128; 
      int y1 = *psrc++;
      yuvput(y0,u,v, pdst); 
      yuvput(y1,u,v, pdst); 
    }
  }
}

void vs_grabUtil::yuv411ToRgb(
  int w, int h, void const* ps, 
  void* pd)
{
  PCBYTE psrc=static_cast<PCBYTE>(ps);
  PBYTE  pdst=static_cast<PBYTE>(pd);

  for (int l = 0; l < h; l++) {
    for (int i = 0; i < w/4; i ++) {
      int u = (*psrc++) - 128; 
      int y0 = *psrc++; 
      int y1 = *psrc++; 
      int v = (*psrc++) - 128; 
      int y2 = *psrc++; 
      int y3 = *psrc++;
      yuvput(y0,u,v, pdst); 
      yuvput(y1,u,v, pdst); 
      yuvput(y2,u,v, pdst);
      yuvput(y3,u,v, pdst);
    }
  }
}
void vs_grabUtil::yuv422ToGray(
  int w, int h, void const* ps, 
  void* pd)
{
  PCBYTE psrc=static_cast<PCBYTE>(ps);
  PBYTE  pdst=static_cast<PBYTE>(pd);

  for (int l = 0; l < h; l++) {
    for (int i = 0; i < w; i++) {
      psrc++; //uv
      *pdst++ = *psrc++; 
    }
  }
}

void vs_grabUtil::grayToRgb(
  int w, int h, void const* ps, 
  void* pd)
{
  PCBYTE psrc=static_cast<PCBYTE>(ps);
  PBYTE  pdst=static_cast<PBYTE>(pd);

  for (int l = 0; l < h; l++) {
    for (int i = 0; i < w/4; i ++) {
      int y = *psrc++; 
      rgbput(y,y,y, pdst); 
    }
  }
}

void vs_grabUtil::flipVertical(
  int w, int h, int szpix, 
  void* pd)
{
  int szline=w*szpix;
  PBYTE pdst1=static_cast<PBYTE>(pd);
  PBYTE pdst2=pdst1+(h-1)*szline;
  PBYTE buf= new BYTE[szline];
  
  for (int l = 0; l < h/2; l++) {
    memcpy(buf, pdst1, szline);
    memcpy(pdst1, pdst2, szline);
    memcpy(pdst2, buf, szline);
    pdst1+=szline;
    pdst2-=szline;
  }
  
  delete [] buf;
}


#include <fstream>
#include <sstream>
#include <stdexcept>

void vs_grabUtil::writePPM(
  int w, int h, int szpix, 
  void const* bits)
{
  static int i=-1;
  std::ostringstream oss;
  oss <<"proba-" <<++i <<".ppm";
  std::ofstream of(oss.str().c_str());
  if (szpix==1){
    of <<"P5 ";
  } else if (szpix==3){
    of <<"P6 ";
  } else{
    throw std::runtime_error("vs_grabUtil::writePPM - unsupported szpix");
  }
  of <<w <<' ' <<h <<" 255" <<'\n';

  char const *charbits=static_cast<char const*>(bits);
  int szline=w*szpix;
  for (char const*p=charbits+szline*(h-1); p>=charbits; p-=szline) {
    of.write(p, szline);
  }
}
