#include "vs_grab_linux1394.hpp"
#include "vs_grab_linuxV4L.hpp"
#include "vs_grabUtil.hpp"

#include "../img/img_data.hpp"
#include "../util/util_registry.hpp"

#include <boost/scoped_ptr.hpp>

#include <stdexcept>
#include <iostream>
#include <string>

/*
g++ -g vs_grab_linuxV4L*cpp \
g++ -g vs_grab_linux1394*cpp \
  vs_grab_test.cpp vs_grabUtil.cpp vs_grab.cpp vs_base.cpp \
  ../img/img_data.cpp ../img/img_util.cpp ../img/access/img_access_fill.cpp \
  ../util/util_registry.cpp ../util/util_time_linux.cpp \
  ../util/util_debug.cpp ../util/util_endian.cpp \
  -ldc1394_control -lraw1394
*/


void getone(vs_grab& vs, int t0){
  img_data img;
  vs.getFrame(img);
  /*
  vs_grabUtil::writePPM(
    img.fmt().width(), 
    img.fmt().height(), 
    img.fmt().szPixel(),
    img.bits()); 
  */
  std::cout <<"Got frame on " <<vs.msDayUtc()-t0 <<"\n";
}

int main(int argc, char* argv[]){
  std::string grabber=
    util::Registry::instance().
    begin<vs_grab>()->name_derived();
  int width=768;
  if (argc>=2){
    grabber=argv[1];
  }
  if (argc>=3){
    width=atoi(argv[2]);
  }
  int nframes=100;
  if (argc>=4){
    nframes=atoi(argv[3]);
  }
  std::cout <<"Testing " <<grabber 
    <<"@" <<width <<" " <<nframes <<"times\n";
  try{
    boost::scoped_ptr<vs_grab> pvsg(
      vs_grab::createCheck(grabber.c_str(),
        vs_grab::CtorArg(0,0,0, img_fmt(width,0,1))));
    std::cout <<"Created vs_grab " 
      <<grabber <<" " 
      <<pvsg->fmt().width() <<"x"
      <<pvsg->fmt().height()<<"x"
      <<pvsg->fmt().szPixel()<<"\n";
    getone(*pvsg, 0);
    int t0=pvsg->msDayUtc();
    for (int i=0; i<nframes; ++i){
      getone(*pvsg, t0);
    }
    double dt=pvsg->msDayUtc()-t0;
    std::cout.precision(3);
    std::cout
      <<nframes <<" frames in "
      <<dt/1000 <<"s = "
      <<1000*nframes/dt  <<" Hz.\n";
  } catch (std::exception& e){
    std::cerr <<"Caught exception: " <<e.what() <<"\n";
  } catch (char const* str){
    std::cerr <<"Caught exception: " <<str <<"\n";
  }
}
