#ifndef win_ann_hpp
#define win_ann_hpp

#include "win_ann_abstract.hpp"

#include <vector>
#include <string>
#include <utility>



class win_ann:
  public win_ann_abstract
{
public:
  win_ann(){}
  win_ann(const win_ann_abstract&);
  const win_ann& operator=(const win_ann_abstract&);
  void assign(const win_ann_abstract&, int scale);
//mutator operations
public:
  virtual void clear();

  virtual void addSound(int t_ms, int freq);
  virtual void addPixel(const grid::Pixel& p, 
      int col, char const* str, PixelStyle style);
  virtual void addLine(const grid::Line& l, 
      int col, char const* str,
      LineStyle style, int width);
  virtual void addCircle(
      const grid::Pixel& c, int radius,
      int col, char const* str,
      LineStyle ls, FillStyle fs, 
      int width);
  virtual void addContour(const grid::Contour& c, 
      int col, char const* str,
      PixelStyle style);

  virtual void addStr(const grid::Pixel& p, 
      int col, char const* str, int size);
//const operations
public:
//  virtual void dump(win_ann_abstract&) const;
//  virtual void dump(grid::draw& dc) const;
  virtual bool empty() const;
 
  int nSounds() const;
  const Sound& sound(int i) const;

  int nPoints() const;
  const Point2D& point(int i) const;  

  int nLines() const;
  const Line& line(int i) const;  

  int nCircles() const;
  const Circle& circle(int i) const;  

  int nContours() const;
  const Contour& contour(int i) const;  

  int nStrings() const;
  const String& string(int i) const;  
//data
private:
  std::vector<Sound> sounds_;
  std::vector<Point2D> points_;
  std::vector<Line> lines_;
  std::vector<Circle> circles_;
  std::vector<Contour> contours_;
  std::vector<String> strings_;
};


#endif
