#ifndef win_ann_abstract_hpp
#define win_ann_abstract_hpp

/******************************************************
 *
 *  component:
 **    win_ann_abstract 
 *
 ** the abstract representation of 
 ** an object holding a collection
 ** of vector graphic components
 ** and simple sounds
 *
 ** author: Sinisa Segvic
 ** version 1.1 (2001-09-18)
 *
 ******************************************************/

#include "../../grid/root/grid_primitives.hpp"


class win_ann_abstract{
public:
  virtual ~win_ann_abstract(){}
  

//types
public:
  struct Sound{
    int t;
    int f;
  };
  enum PixelStyle{PS_Dot, PS_BoldSmall, PS_BoldLarge, PS_BoldHuge};
  struct Point2D{
    grid::Pixel p;
    int colour;
    std::string text;
    PixelStyle style; 
  };
  enum LineStyle{LS_Normal, LS_Dotted};
  struct Line{
    grid::Line l;
    int colour;
    std::string text;
    LineStyle style; 
    int width; 
  };
  enum FillStyle{FS_None, FS_Solid};
  struct Circle{
    grid::Pixel c;
    int r;
    int colour;
    std::string text;
    LineStyle ls; 
    FillStyle fs; 
    int width; 
  };
  struct Contour{
    grid::Contour c;
    int colour;
    std::string text;
    PixelStyle style; 
  };
  struct String{
    grid::Pixel p;
    int colour;
    std::string text;
    int size; 
  };

//mutator operations
  virtual void clear() =0;
  virtual void addSound(int t_ms, int freq) =0;

  virtual void addPixel(const grid::Pixel& p, 
      int col, char const* str, PixelStyle style) =0;
  virtual void addLine(const grid::Line& l, 
      int col, char const* str,
      LineStyle style, int width) =0;
  virtual void addCircle(
      const grid::Pixel& c, int radius,
      int col, char const* str,
      LineStyle ls, FillStyle fs, 
      int width) =0;
  virtual void addContour(const grid::Contour& c, 
      int col, char const* str,
      PixelStyle style) =0;
  virtual void addStr(const grid::Pixel& p, 
      int col, char const* str, int size) =0;

//const operations
//  virtual void dump(win_ann_abstract&) const =0;
//  virtual void dump(grid::draw& dc) const =0;
  virtual bool empty() const =0;

  virtual int nSounds() const =0;
  virtual const Sound& sound(int i) const =0;

  virtual int nPoints() const =0;
  virtual const Point2D& point(int i) const =0;  

  virtual int nLines() const =0;
  virtual const Line& line(int i) const =0;  

  virtual int nCircles() const =0;
  virtual const Circle& circle(int i) const =0;  

  virtual int nContours() const =0;
  virtual const Contour& contour(int i) const =0;  

  virtual int nStrings() const =0;
  virtual const String& string(int i) const =0;  
};


#endif
