#include "win_ann_dump.hpp"
#include "win_ann_abstract.hpp"

#include "../../img/img_data.hpp"
#include "../../img/io/img_io_save_ras.hpp"

#include "../../grid/draw/grid_draw_fileEps.hpp"

#include "../../util/util_fs.hpp"

#include <string>

namespace{

std::string getFnEps(char const* basename){
  std::string dirLog(util_fs::getLogDir());
  std::string fnEps(dirLog+basename+".eps");
  std::string fnUniqueEps=
    util_fs::uniqueFilename(fnEps.c_str());
  return fnUniqueEps;
}
void dumpToFnEps(
  char const* file, 
  int w, int h,
  const win_ann_abstract& a)
{
  grid::draw_fileEps dc(file, w,h);
  win_ann_dump::toDC(a,dc);
}
void dumpToFnEps(
  char const* file, 
  const img_data& img,
  const win_ann_abstract& a)
{
  grid::draw_fileEps dc(file, 
    img.fmt().width(),img.fmt().height(), img);
  win_ann_dump::toDC(a,dc);
}

}



namespace win_ann_dump{

void toOneFile(
  const win_ann_abstract& a,
  const img_data& d, 
  char const* basename)
{
  // dumping window annotation to an .eps file
  std::string fnUniqueEps=getFnEps(basename);
  dumpToFnEps(fnUniqueEps.c_str(), d, a);
}

void toTwoFiles(
  const win_ann_abstract& a,
  const img_data& d, 
  char const* basename)
{
  // dumping window annotation to an .eps file
  std::string fnUniqueEps=getFnEps(basename);
  dumpToFnEps(fnUniqueEps.c_str(), d.fmt().width(), d.fmt().height(), a);
  // storing window image to a .ras file
  std::string fnUniqueRas=
    fnUniqueEps.substr(0,fnUniqueEps.size()-3)+"ras"; 
  img_io_save_ras dsRas(fnUniqueRas.c_str());
  dsRas(d);
}

void toFile(
  const win_ann_abstract& a,
  int w, int h,
  char const* basename)
{
  // dumping window annotation to an .eps file
  std::string fnUniqueEps=getFnEps(basename);
  dumpToFnEps(fnUniqueEps.c_str(), w,h, a);
}


void toDC(
  const win_ann_abstract& a, 
  grid::draw& dc) 
{
  //play sounds
  {for (int i=0; i<a.nSounds(); i++){
    const win_ann_abstract::Sound& s=a.sound(i);
    dc.playSound(s.t, s.f);
  }}

  //draw points
  {for (int i=0; i<a.nPoints(); i++){
    const win_ann_abstract::Point2D& ptWin=a.point(i);
    const grid::Pixel& ptGeom=ptWin.p;
    dc.colour(ptWin.colour);
    grid::Pixel strOffset(0,0);
    int strSize=-1;
    switch(ptWin.style){
    case win_ann_abstract::PS_Dot:
      dc.drawPtDot(ptGeom);
      break;
    case win_ann_abstract::PS_BoldSmall:
      dc.drawPtBoldSmall(ptGeom);
      break;
    case win_ann_abstract::PS_BoldLarge:
      strOffset+=grid::Pixel(2,2);
      strSize=0;
      dc.drawPtBoldLarge(ptGeom);
      break;
    case win_ann_abstract::PS_BoldHuge:
      strOffset+=grid::Pixel(4,4);
      strSize=1;
      dc.drawPtBoldHuge(ptGeom);
      break;
    default:
      throw "win_ann_dump::toDC - "
            "got illegal point type index";
    }
    dc.drawStr(ptGeom+strOffset, ptWin.text.c_str(), strSize);
  }}

  //draw lines
  {for (int i=0; i<a.nLines(); i++){
    const win_ann_abstract::Line& lnWin=a.line(i);
    const grid::Pixel& p0=lnWin.l.first();
    const grid::Pixel& p1=lnWin.l.second();
    dc.colour(lnWin.colour);
    dc.drawLine(p0, p1, lnWin.width, lnWin.style);

    grid::Pixel mid(
      (p0.x()+p1.x())/2,
      (p0.y()+p1.y())/2
    );
    dc.drawStr(mid, lnWin.text.c_str(), -1);
  }}

  //draw circles
  {for (int i=0; i<a.nCircles(); i++){
    const win_ann_abstract::Circle& cWin=a.circle(i);
    const grid::Pixel& c=cWin.c;
    dc.colour(cWin.colour);
    dc.drawCircle(c, cWin.r, cWin.ls, cWin.fs, cWin.width);
    dc.drawStr(c, cWin.text.c_str(), -1);
  }}

  //draw contours
  {for (int i=0; i<a.nContours(); i++){
    const win_ann_abstract::Contour& cwin=a.contour(i);
    const grid::Contour& cgrid=cwin.c;
    if (cgrid.empty())
      continue;
    dc.colour(cwin.colour);
    dc.drawContour(cgrid,cwin.style);
    int sz=cwin.style?0:-1;
    dc.drawStr(cgrid[0], cwin.text.c_str(), sz);
  }}

  //draw strings
  {for (int i=0; i<a.nStrings(); i++){
    const win_ann_abstract::String& strWin=a.string(i);
    const grid::Pixel& pt=strWin.p;
    dc.colour(strWin.colour);
    dc.drawStr(pt, strWin.text.c_str(), strWin.size);
  }}

  dc.flush(); 
}


}

