#ifndef win_event_queue_hpp
#define win_event_queue_hpp

#include "win_event.hpp"

#include <vector>
#include <utility>
#include <assert.h>

////////////////////////////
// queue of fixed size

class win_event_queue{
public:
  win_event_queue();
  void push_front(const win_event&);
  bool pop_back(int& id, win_event_abstract&);

private:
  inline int address(int id);

private:
  enum QueueSize{size_=1024};
  std::vector<std::pair<int, win_event> > vec_;
  int count_;
};

////////////////////////////////
// queue inline implementations

inline void win_event_queue::
push_front(const win_event& e)
{
  int pos=address(count_);
  vec_[pos]=std::pair<int, win_event>(count_,e);
  count_++;
}
inline bool win_event_queue::
pop_back(int& id, win_event_abstract& e)
{
  bool rv=false;
  if (id<0) id=0;
  if (id<count_){
    int pos=address(id);
    if (vec_[pos].first==id){
      e.assign(vec_[pos].second);
      ++id;
      rv=true;
    } else{
      //buffer oferflow
      assert(count_>=1);
      id=count_-1;
      rv=pop_back(id, e);
    }
  }
  return rv;
}

inline int win_event_queue::
address(int a)
{
  return a%size_;
}



#endif

