#include <iostream>
#include <iomanip>
#include <sstream>
#include <string>
#include <vector>
#include <memory>

#include "win_wrap.hpp"
#include "../ann/win_ann.hpp"
#include "../event/win_event.hpp"

#include "../../util/util_string.hpp"
#include "../../util/util_time.hpp"

#include "../../img/img_data.hpp"
#include "../../img/img_util.hpp"
#include "../../img/access/img_access_fill.hpp"

/*
(fali thr, win_w32)
NAME=wintest
PACKAGES=
  win/
  grid/
  util/util_time
  util/util_string
  util/util_fs
  util/util_endian
  util/util_registry
  util/util_sr
  util/util_pxm
  util/util_bmp
  img/
FILTER=
UNFILTER=
  win_test.cpp
CONFIG= win_x
*/                                                                                


//////////////////////////////////////////////////////
// error handling

int handleError(
  char const*msg)
{
  std::ostringstream oss;
  oss <<"\nCaught exception in main:\n" ;
  std::string indented;
  util_string::indent(msg, indented);
  oss <<indented <<std::endl;
  oss <<"  Invoke with no parameters for "
      <<"more information.\n" <<std::endl;
  std::cerr <<oss.str().c_str();
  return 1;
}
                                                                                

void checkLMB(win_base& win, char const* desc){
  int xclick=-1;
  int yclick=-1;

  int winEventPos=0;
  win_event e;
  while (win.event(winEventPos, e)){
    if (e.type()==win_event::MouseLeft){
      xclick=e.par1();
      yclick=e.par2();
      assert(xclick>=0 && yclick>=0);
    }
  }

  if (xclick>=0){
    std::cerr <<"Got LMB click in window " <<desc
        <<"  x=" <<xclick
        <<", y=" <<yclick
        <<std::endl;
  }
}

//////////////////////////////////////////////////////
// work procedure

void process(){

  img_data imgBig(img_fmt(640,480,3));
  img_access_fillCheckboard(imgBig,100);

  win_wrap winBig("Test window Big", 
    imgBig.fmt().width(), 
    imgBig.fmt().height());
  winBig.setImg(imgBig);

  img_fmt fmt(320,240,1);
  img_data img(fmt);
  img_access_fillCheckboard(img,50);

  win_wrap win("Test window", 
    fmt.width(), fmt.height());
  win.setImg(img);
  win.position(20,30);

  
  const int n=100;
  const int t0=util_time::msDayUtc();
  for (int i=0; i<n; i++){
    // std::cerr <<"Iteration " <<i <<std::endl;
    int j=i%fmt.width();
    int k=fmt.width()-j;

    win_ann ann;
    ann.addLine(
      grid::Line(grid::Pixel(j,10), grid::Pixel(100,100)),
      3,"proba", win_ann_abstract::LS_Dotted,2);
    ann.addPixel(
      grid::Pixel(k,200),
      6, "bold small", win_ann_abstract::PS_BoldSmall);
    ann.addPixel(
      grid::Pixel(150,10),
      4, "bold large", win_ann_abstract::PS_BoldLarge);
    win.setImgAnn(img,ann);
    winBig.setImgAnn(imgBig,ann);
  }
  int t=util_time::msDayUtc()-t0;
  
  std::cerr <<"Performance: " <<std::setprecision(2) <<(1000.0*n)/t <<"\n";


  std::string str;
  do{
    std::cerr <<"Type zero for exit." <<std::endl;
    std::cin >>str;

    std::ostringstream oss;
    oss <<"Win1: you typed " <<str;
    win.caption(oss.str().c_str());
    oss.str("");
    oss <<"WinBig: you typed " <<str;
    winBig.caption(oss.str().c_str());

    // check events for LMB
    checkLMB(win,  "1");
    checkLMB(winBig, "2");
  } while(str!="0");

}



/*int main(){
  std::cerr <<"Starting the program.\n";
  try{
    process();
  } catch(char const* msg){
    return handleError(msg);
  } catch(std::exception& e){
    return handleError(e.what());
  } catch (...){
    std::cerr <<"Caught unknown exception in main.";
    return 1;
  }
  std::cerr <<"Exiting the program.\n";
  return 0;
}*/

