#include "win_wrap.hpp"

#include <algorithm>

win_wrap::win_wrap(
  const char* caption, 
  int cx, int cy)
{
  static char const* winDefault=
#if defined(_WIN32)
  "w32";
#else
  "x";
#endif
  //limit initial window size
  cx=std::min(cx, 768);
  cy=std::min(cy, 576);

  // create the window
  pwin=win_base::create(winDefault, 
    win_base::CtorArg(cx,cy,caption));
}
win_wrap::~win_wrap()
{
  delete pwin;
}


int win_wrap::width(){
  return pwin->width();
}
int win_wrap::height(){
  return pwin->height();
}


void win_wrap::position(int x, int y){
  pwin->position(x,y);
}
void win_wrap::size(int x, int y){
  pwin->size(x,y);
}
void win_wrap::caption(char const* str){
  pwin->caption(str);
}



void win_wrap::flash(){
  pwin->flash();
}

void win_wrap::putImg(const img_data& d){
  pwin->putImg(d);
}
void win_wrap::putImgAnn(
  const img_data& d,
  const win_ann_abstract& ann)
{
  pwin->putImgAnn(d,ann);
}
void win_wrap::putAnn(const win_ann_abstract& ann){
  pwin->putAnn(ann);
}


bool win_wrap::event(int& it, win_event_abstract& e) const {
  return pwin->event(it, e);
}
