#include "win_w32.hpp"
#include "win_w32_worker.hpp"

#include "../../img/img_data.hpp"

#include "../../thr/thr_base.hpp"

#include "../../util/util_registry.hpp"

#include <boost/shared_ptr.hpp>

///////////////////////////////////////////////
// requirements for the creation from registry

char const* win_w32::s_name(){
  return "w32";
}

namespace{
  util::RegistryCreator<win_base,win_w32> creator;
} // unnamed namespace end


///////////////////////////
// private implementation
struct win_w32_imp{
  win_w32_imp(int w, int h, char const* s);
  ~win_w32_imp();

  boost::shared_ptr<win_w32_worker> pWorker;
  thr::ThreadSync thread;
};
win_w32_imp::win_w32_imp(
  int w, int h, char const* s)
:
  pWorker(new win_w32_worker(w,h,s)),
  thread(pWorker)
{
  thr::Lock guard(pWorker->mutex());
  while (!pWorker->isReady()){
    pWorker->cv().wait(pWorker->mutex());
  }
}
win_w32_imp::~win_w32_imp(){
}


///////////////////////////
// members' implementation
win_w32::win_w32(const CtorArg& arg):
  pimp(new win_w32_imp(arg.width,arg.height, arg.caption))
{
  pimp->pWorker->show();
}

win_w32::~win_w32(){
  delete pimp;
}

int win_w32::width(){
  return pimp->pWorker->width();
}
int win_w32::height(){
  return pimp->pWorker->height();
}


void win_w32::position(int x, int y){
  pimp->pWorker->position(x,y);
}
void win_w32::size(int cx, int cy){
  pimp->pWorker->size(cx,cy);
}
void win_w32::caption(char const* title){
  pimp->pWorker->caption(title);
}


void win_w32::putAnn(
  const win_ann_abstract& a)
{
  pimp->pWorker->putAnn(a);
}
void win_w32::flash(){
  pimp->pWorker->beep();
}

void win_w32::putImg(
  const img_data& img)
{
  pimp->pWorker->putImg(img);
}
void win_w32::putImgAnn(
  const img_data& img,
  const win_ann_abstract& ann)
{
  pimp->pWorker->putImgAnn(img, ann);
}



bool win_w32::event(int& id, win_event_abstract& e) const {
  return pimp->pWorker->queryEvent(id,e);
}





