#ifndef win_w32_worker_hpp
#define win_w32_worker_hpp

//////////////////////////////////////////////
// win_w32_worker  - 
//    handles all window operations 
//    in a separate thread starting
//    from operator()

#include "../ann/win_ann.hpp"
#include "../event/win_event_queue.hpp"

#include "../../img/img_data.hpp"
#include "../../thr/thr_base.hpp"


#include <windows.h>

#include <string>


class win_w32_worker:
public thr::ThreadSyncWorker{
public:    
  win_w32_worker(int width, int height, char const* title);
  ~win_w32_worker();

  //entry function
public:    
  virtual int operator()();
  virtual void cleanup();

  //operations
public:    
  void show();
  void hide();
  void close();
  void position(int, int);
  void size(int, int);
  void caption(char const*);

  //output
public:    
  void putImg(const img_data& img);
  void putImgAnn(const img_data& img, const win_ann_abstract&);
  void putAnn(const win_ann_abstract&);
  void beep(int t_ms=500, int f=50);
private:
  void copyImg(const img_data& img);
  void update();

  //user input
public:    
  //called by clients
  bool queryEvent(int& id, win_event_abstract& e);
  //called by WndProc
  void addClick(int button, int, int, int mod);
  void addChar(int type, int, int mod);

  //attributes
public:    
  int width(){return width_;}
  int height(){return height_;}
  char const* text(){return caption_.c_str();}
  const img_data& img(){return img_;}
  const win_ann_abstract& ann(){return ann_;}

  //sync
public:    
  bool isReady(){return hwnd_!=0;}
  thr::Mutex& mutex(){return mtx_;}
  thr::CondVar& cv(){return cv_;}
private:

  HWND hwnd_;

  //client area
  int width_;
  int height_;
  //window-client area
  int borderx_;
  int bordery_;
  //caption
  std::string caption_;

  //input data
  win_event_queue log_;

  //paint data
  img_data img_;
  win_ann ann_;
  //bool imgDirty_;

  //syncronization objects
  thr::Mutex mtx_;
  thr::CondVar cv_;
};

#endif

