#include "win_x.hpp"
#include "win_x_thread.hpp"

#include "../../util/util_registry.hpp"

#include "../../img/img_data.hpp"
#include "../../img/img_util.hpp"

//#include <iostream>

///////////////////////////////////////////////
// requirements for the creation from registry
                                                                     
char const* win_x::s_name(){
  return "x";
}

namespace{
  util::RegistryCreator<win_base,win_x> creator;
} // unnamed namespace end
                                                                     

////////////////////////////////
// private implementation 

struct win_x_imp{
  win_x_imp(int cx, int cy);
  ~win_x_imp();
  int id;

  int width;
  int height;
};
win_x_imp::win_x_imp(
  int cx, int cy)
{
  width=cx;
  height=cy;
}
win_x_imp::~win_x_imp(){}


////////////////////////////////
// win_x is a win_base implementation 
// for the X windows system

win_x::win_x(const CtorArg& arg):
  pimp(new win_x_imp(arg.width, arg.height))
{
  pimp->id=win_x_thread::instance().
    createWindow(arg.width, arg.height);
  caption(arg.caption);
}

win_x::~win_x(){
  win_x_thread::instance().
    destroyWindow(pimp->id);
  delete pimp;
}



//////////////////////////////
// window properties

int win_x::width(){
  return pimp->width;
}
int win_x::height(){
  return pimp->height;
}


void win_x::position(int x, int y){
  win_x_thread::instance().
    position(pimp->id, x,y);
}
void win_x::size(int x, int y){
  pimp->width=x;
  pimp->height=y;

  win_x_thread::instance().
    size(pimp->id, x,y);
}

void win_x::caption(char const* title){
  win_x_thread::instance().
    caption(pimp->id, title);
}

void win_x::flash(){
  win_x_thread::instance().
    flash(pimp->id);
}




//////////////////////////////
// window output

void win_x::putAnn(
  const win_ann_abstract& ann)
{
  win_x_thread::instance().
    putAnn(pimp->id, ann);
}


void win_x::putImg(
  const img_data& img)
{
  win_x_thread::instance().
    putImg(pimp->id, img);
}
void win_x::putImgAnn(
  const img_data& img,
  const win_ann_abstract& ann)
{
  win_x_thread::instance().
    putImgAnn(pimp->id, img, ann);
}



//////////////////////////////
// window input

bool win_x::event(
  int& iterator, win_event_abstract& e) const
{
  return win_x_thread::instance().
    queryEvent(pimp->id, iterator, e);
}



                                                                     

