#include "../alg/alg_base.hpp"

#include "../cli/cli_wrapAbstract.hpp"

#include "../util/util_registry.hpp"

#include "../img/img_util.hpp"
#include "../img/img_vector.hpp"

#include "../win/ann/win_ann.hpp"

#include <iostream>
#include <sstream>
#include <fstream>
#include <stdexcept>
#include <stdlib.h>

namespace{

  void tokenize(
    const std::string& str, 
    std::vector<std::string>& tokens,
    char const* delimiters)
  {
    std::string::size_type lastPos = 
      str.find_first_not_of(delimiters, 0);
    std::string::size_type pos     = 
      str.find_first_of(delimiters, lastPos);

    while (std::string::npos != pos || std::string::npos != lastPos)
    {
        tokens.push_back(str.substr(lastPos, pos - lastPos));
        lastPos = str.find_first_not_of(delimiters, pos);
        pos = str.find_first_of(delimiters, lastPos);
    }
  }
}


////////////////////////////////////
// alg_signgt declaration

class alg_signgt: 
  public alg_base
{
public:
  alg_signgt(const CtorArg& arg);
  virtual ~alg_signgt();
public:
  virtual char const* name(){return s_name();}
  static char const* s_name(){return "signgt";}
private:
  virtual void process( 
    const img_vectorAbstract& src, 
    const win_event_vectorAbstract& events,
    int msDayUtc, int nframe); 
  virtual void config(cli_wrapAbstract& cli);
private:
  virtual int countDst(); // >=1!
  virtual const img_data& imgDst(int id);
  virtual win_ann_abstract const* pAnnDst(int id); 
private:
  void seek(int fpos);
  void initIfs();
private:
  win_ann ann_;
  img_data img_;
  std::string strInfoFile_;
  std::ifstream ifs_;
  int fpos_;
};

////////////////////////////////////
// static registry creator object

namespace{
  util::RegistryCreator<alg_base,alg_signgt> creator;
} // unnamed namespace end

  
////////////////////////////////////
// alg_signgt implementation

alg_signgt::alg_signgt(const CtorArg& ):
  strInfoFile_("gt.txt")
{
  initIfs();
}

alg_signgt::~alg_signgt(){}

void alg_signgt::process(
  const img_vectorAbstract& src,
  const win_event_vectorAbstract&, int, int nframe)
{
  if (!ifs_.good()){
    throw std::runtime_error(std::string(
      "alg_signgt::process() - can't open ") + strInfoFile_);
  }

  //std::cerr <<nframe <<"\n";
  seek(nframe);

  img_=src[0];
  int height=img_.fmt().height();
  
  std::string line;
  std::getline(ifs_, line);
  ++fpos_;
  
  ann_.clear();
  ann_.addStr(grid::Pixel(10,10), 0, line.c_str(), 2);
  std::vector<std::string> signs;
  tokenize(line, signs, "#");
  for (unsigned i=0; i<signs.size(); ++i){
    std::vector<std::string> elems;
    tokenize(signs[i], elems, "@,:");
    if (elems.size()<4)
      continue;
    int x=atoi(elems[1].c_str());
    int y=height-atoi(elems[2].c_str());
    int d=atoi(elems[3].c_str());
    
    ann_.addPixel(grid::Pixel(x,y), i, "",
      win_ann_abstract::PS_BoldSmall);
    ann_.addStr(grid::Pixel(x,y), i, elems[0].c_str(), 3);
    ann_.addCircle(grid::Pixel(x,y), d, i, "",
      win_ann_abstract::LS_Normal,
      win_ann_abstract::FS_None,1);
  }
}

int alg_signgt::countDst(){
  return 1;
}

const img_data& alg_signgt::imgDst(int id){
  checkId(id, "alg_signgt::imgDst");
  return img_;
}
win_ann_abstract const* alg_signgt::pAnnDst(int id){
  checkId(id, "alg_decoratorScale");
  return &ann_;
}


///////////////////////////////////
// facultative operations

void alg_signgt::config(cli_wrapAbstract& cli)
{
  
  //introduction
  std::ostringstream oss;
  oss <<"Current configuration:\nstrInfoFile: " 
      <<strInfoFile_ <<std::endl;
  oss <<"Please supply the new value for strInfoFile: ";
  cli.put(oss.str().c_str());

  oss.str("");
  //input a line
  char const* buf=cli.get();
  if (strlen(buf)>0){
    std::istringstream iss(buf);
    iss >>strInfoFile_;
    oss <<std::endl;
  }
  
  //report the new state
  oss <<"New configuration:\nstrInfoFile: " 
      <<strInfoFile_ <<std::endl;
  cli.put(oss.str().c_str());

  initIfs();
}

void alg_signgt::seek(int nframe){
  if (nframe!=fpos_){
    ifs_.seekg(0, std::ios::beg);
    if (nframe>0){
      for (int i=0; i<nframe; ++i){
        std::string line;
        std::getline(ifs_, line);
      }
    }
    fpos_=nframe;
  }
}


void alg_signgt::initIfs(){
  ifs_.close();
  ifs_.open(strInfoFile_.c_str());
  fpos_=0;
  
  std::cerr <<ifs_.good() <<"\n";
}
