#include "RGBtoHSI.h"
#include "nadjiBoju.h"
#include "narastanje.h"

#include "../alg/alg_base.hpp"

#include "../util/util_registry.hpp"

#include "../cli/cli_wrapAbstract.hpp"
#include "../img/img_util.hpp"
#include "../img/img_vector.hpp"
#include "../win/ann/win_ann.hpp"
#include "../win/ann/win_ann_util.hpp"

#include <iostream>
#include <sstream>

#include <stdlib.h>
#include <stdio.h>
#include <malloc.h>


// vd_file_srs vdf(img_fmt(384,288,1),"/var/tmp/proba.srs");

////////////////////////////////////
// alg_znakovi declaration

class alg_znakovi: 
  public alg_base
{
public:
  alg_znakovi(const CtorArg& arg);
  virtual ~alg_znakovi();
public:
  virtual char const* name(){return s_name();}
  static char const* s_name(){return "znakovi";}
private:
  virtual void process( 
    const img_vectorAbstract& src, 
    const win_event_vectorAbstract& events,
    int msDayUtc); 
  virtual void config(cli_wrapAbstract& cli);
  void dumpParams(std::ostream& os, char const* intro);
private:
  virtual int countDst(); // >=1!
  virtual const img_data& imgDst(int id);
  virtual win_ann_abstract const* pAnnDst(int id);
private:
  img_vector imgs_;
  std::vector<win_ann> anns_;
  double hueLo_;
  double hueHi_;
};

////////////////////////////////////
// static registry creator object

namespace{
  util::RegistryCreator<alg_base,alg_znakovi> creator;
} // unnamed namespace end

  
////////////////////////////////////
// alg_znakovi implementation

alg_znakovi::alg_znakovi(const CtorArg& arg):
  imgs_(3),
  anns_(2),
  hueLo_(3.3),
  hueHi_(4)
{
}

alg_znakovi::~alg_znakovi(){}


void alg_znakovi::process(
  const img_vectorAbstract& src,
  const win_event_vectorAbstract&, int)
{
  imgs_[0]=src[0];
  imgs_[1].reformat(src[0].fmt());
  img_util::fillBits(imgs_[1], 0);
  imgs_[2].reformat(src[0].fmt());
  img_util::fillBits(imgs_[2], 0);

  //TODO RGBtoHSI(src[0], imgs_[1]);

  //printf ("\n\n%d", imgs_[0].bits());
  //test(imgs_[0].fmt().height(),	  imgs_[0].fmt().width(),	  imgs_[0].bits(),	  (float*) imgs_[1].bits() );
  


  //printf ("%d,", sizeof (double));
  int sirina, visina, *prav;
  float *imgHsi, *imgBin, *imgRje;
  visina = imgs_[0].fmt().height();
  sirina = imgs_[0].fmt().width();
  printf ("\n\n%d %d", sirina, visina);
  imgHsi = (float*) malloc (3*sirina*visina* sizeof(float));
  imgBin = (float*) malloc (3*sirina*visina* sizeof(float));



  RGBtoHSI(
	  imgs_[0].fmt().width(),
	  imgs_[0].fmt().height(),
	  (unsigned char*) imgs_[0].bits(),
	  imgHsi );

  
  nadjiBoju (
	  imgs_[0].fmt().width(),
	  imgs_[0].fmt().height(),
	  imgHsi,
	  imgBin, 3.3, 4 );


  prav = narastanje (
	  imgs_[0].fmt().width(),
	  imgs_[0].fmt().height(),
	  imgBin);


  //ss
  int i=0;
  anns_[0].clear();
  while (prav[i]!=-1){
    win_ann_util::addRectangle(anns_[0], 
	    grid::Pixel(prav[i],  prav[i+1]),
	    grid::Pixel(prav[i+2],prav[i+3]), 0, 1);
	i+=4;
  }

  free (imgHsi);
  free (imgBin);
}

int alg_znakovi::countDst(){
  return 1;
}

const img_data& alg_znakovi::imgDst(int id){
  checkId(id, "alg_znakovi::imgDst");
    return imgs_[0];
}
win_ann_abstract const* alg_znakovi::pAnnDst(int id){
  checkId(id, "alg_znakovi::imgDst");
  if (id==0)
    return &anns_[0];
  else
    return &anns_[1];
}

/*
char const* alg_znakovi::captionDst(int id){
  checkId(id, "alg_histogram::captionDst");
  if (id==0)
    return "destination image";
  else
    return "source image";
}*/


void alg_znakovi::config(
  cli_wrapAbstract& cli)
{
  //introduction
  std::ostringstream oss;
  dumpParams(oss, "Current znakovi configuration:\n");
  oss <<"Please supply the new values for hueLo, hueHi: ";
  cli.put(oss.str().c_str());

  oss.str("");
  //input a line
  char const* buf=cli.get();
  if (strlen(buf)>0){
    std::istringstream iss(buf);
    iss >>hueLo_ >>hueHi_;
    oss <<std::endl;
  }
  
  //report the new state
  dumpParams(oss, "The new value is:\n");
  cli.put(oss.str().c_str());
}

void alg_znakovi::dumpParams(
  std::ostream& os, 
  char const* intro)
{
  os <<intro <<"hueLo, HueHi: " <<"(" <<hueLo_ 
	          <<", " <<hueHi_ <<")" <<std::endl;
}

